SystemOrganization addCategory: #'FileSystem-Tests-Core'!
SystemOrganization addCategory: #'FileSystem-Memory'!
SystemOrganization addCategory: #'FileSystem-Disk'!
SystemOrganization addCategory: #'FileSystem-AnsiStreams'!
SystemOrganization addCategory: #'FileSystem-Core-Implementation'!
SystemOrganization addCategory: #'FileSystem-Core-Kernel'!
SystemOrganization addCategory: #'FileSystem-Core-Public'!
SystemOrganization addCategory: #'FileSystem-Tests-AnsiStreams'!
SystemOrganization addCategory: #'FileSystem-Tests-Disk'!
SystemOrganization addCategory: #'FileSystem-Tests-Memory'!
SystemOrganization addCategory: #'FileSystem-Zip'!

!FileStream class methodsFor: '*FileSystem' stamp: 'abc 5/11/2012 23:23'!
onHandle: aFileSystemHandle
	^ self concreteStream new
		open: aFileSystemHandle fullName
		forWrite: aFileSystemHandle isWritable! !

RWBinaryOrTextStream subclass: #FileSystemFileStream
	instanceVariableNames: 'store path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!FileSystemFileStream commentStamp: 'cwp 11/18/2009 11:24' prior: 0!
I am an abstract superclass for classes that provide FileStream-compatible streams, for using alternate filesystems from legacy code.!

!FileSystemFileStream class methodsFor: 'instance creation' stamp: 'cwp 2/19/2011 01:17'!
on: aCollection store: aStore path: aPath
	^ (self on: aCollection)
		setStore: aStore path: aPath;
		yourself! !

!FileSystemFileStream methodsFor: 'private' stamp: 'DamienPollet 2/5/2011 00:58'!
on: aCollection
	super on: aCollection.
	readLimit := collection size.! !

!FileSystemFileStream methodsFor: 'accessing' stamp: 'cwp 2/19/2011 01:18'!
setStore: aStore path: aPath
	store := aStore.
	path := aPath.! !

FileSystemFileStream subclass: #MemoryFileStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!MemoryFileStream commentStamp: 'cwp 11/18/2009 13:04' prior: 0!
I am a legacy-compatibility stream. I am protocol compatible with StandardFileStream, but I operate on an in-memory file in a MemoryFilesystem.!

!MemoryFileStream methodsFor: 'file status' stamp: 'cwp 2/19/2011 01:20'!
close
	
	store 
		replaceFile: path 
		in: [ :bytes | bytes first: position ]
! !

!MemoryFileStream methodsFor: 'private' stamp: 'cwp 2/19/2011 01:20'!
growTo: anInteger
	
	collection := store growFile: path to: anInteger.
	writeLimit := collection size! !

!MemoryFileStream methodsFor: 'private' stamp: 'cwp 2/19/2011 01:20'!
pastEndPut: anObject
	| oldSize newSize |
	oldSize := collection size.
	newSize := oldSize + ((oldSize max: 20) min: 1000000).
	collection := store growFile: path to: newSize.
	writeLimit := collection size.
	collection at: (position := position + 1) put: anObject.
	^ anObject! !

Error subclass: #FileSystemError
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileSystemError commentStamp: 'cwp 11/18/2009 12:32' prior: 0!
I am an abstract superclass for errors that may occur during filesystem operations.!

FileSystemError subclass: #DirectoryDoesNotExist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!DirectoryDoesNotExist commentStamp: 'cwp 11/18/2009 12:33' prior: 0!
I am raised when I an operation is attempted inside a directory that does not exist. !

FileSystemError subclass: #DirectoryExists
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!DirectoryExists commentStamp: 'cwp 11/18/2009 12:35' prior: 0!
I am raised on an attempt to create a directory that already exists.!

FileSystemError subclass: #FileDoesNotExist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileDoesNotExist commentStamp: 'cwp 11/18/2009 12:35' prior: 0!
I am raised when an operation is attempted on a file that does not exist. This includes cases where a file operation is attempted on a directory.!

FileSystemError subclass: #FileExists
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileExists commentStamp: 'cwp 11/18/2009 12:37' prior: 0!
I am raised on an attempt to create a file or directory over top of an existing file.!

!FileSystemError class methodsFor: 'instance creation' stamp: 'cwp 11/14/2009 23:32'!
reference: aReference
	^ self basicNew initializeWithReference: aReference! !

!FileSystemError class methodsFor: 'instance creation' stamp: 'cwp 11/14/2009 23:31'!
signalWith: aReference
	^ (self reference: aReference) signal! !

!FileSystemError methodsFor: 'initialize-release' stamp: 'lr 8/16/2010 16:00'!
initializeWithReference: aReference
	reference := aReference.
	messageText := aReference printString! !

!FileSystemError methodsFor: 'testing' stamp: 'lr 8/16/2010 16:00'!
isResumable
	^ true! !

!FileSystemError methodsFor: 'accessing' stamp: 'lr 7/13/2010 15:31'!
reference
	^ reference! !

FileSystemError subclass: #IllegalName
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!IllegalName class methodsFor: 'instance creation' stamp: 'DamienPollet 2/28/2011 17:04'!
name: aName
	^ self basicNew initializeWithName: aName! !

!IllegalName class methodsFor: 'instance creation' stamp: 'DamienPollet 2/28/2011 17:03'!
signalWith: aName
	^ (self name: aName) signal! !

!IllegalName methodsFor: 'initialization' stamp: 'DamienPollet 2/28/2011 17:08'!
initializeWithName: aName
	name := aName.
	self messageText: aName! !

!IllegalName methodsFor: 'accessing' stamp: 'DamienPollet 2/28/2011 17:08'!
name
	^ name! !

!String methodsFor: '*filesystem-core-converting' stamp: 'SeanDeNigris 5/3/2012 00:00'!
asDirectory
	"Return a FileReference on disk"
	^ self asFileReference! !

!String methodsFor: '*filesystem-core-converting' stamp: 'SeanDeNigris 5/3/2012 00:00'!
asFile
	"Return a FileReference on disk"
	^ self asFileReference! !

!String methodsFor: '*filesystem-core-converting' stamp: 'SeanDeNigris 5/3/2012 00:00'!
asFileReference
	"Return a FileReference on disk"
	^ FileSystem disk referenceTo: self! !

!String methodsFor: '*filesystem-core-converting' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
asPath
	"convert myself to a path"
	"Examples:
		'.' asPath
		'~/Desktop' asPath
		'/home/foo' asPath
		'../../foo.bar' asPath"
	^ FileSystem disk resolve: self! !

!String methodsFor: '*filesystem-core-converting' stamp: 'cwp 11/21/2009 11:30'!
asPathWith: anObject 
	^ anObject pathFromString: self! !

!String methodsFor: '*filesystem-core-converting' stamp: 'EstebanLorenzano 4/12/2012 14:30'!
asReference
	self deprecated: 'Use #asFileReference instead'  on: '12 April 2012' in:  'Pharo 1.4' .
	^ self asFileReference! !

!String methodsFor: '*filesystem-core-converting' stamp: 'StephaneDucasse 2/9/2011 13:41'!
asResolvedBy: aFileSystem
	^ aFileSystem resolveString: self! !

Notification subclass: #ResolutionRequest
	instanceVariableNames: 'origin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!ResolutionRequest commentStamp: 'cwp 11/18/2009 11:38' prior: 0!
I represent a request for user assistance in resolving an origin. I am a resumable exception that gets raised when there is no way of automatically resolving a particular origin. !

!ResolutionRequest class methodsFor: 'instance creation' stamp: 'cwp 10/27/2009 10:13'!
for: origin
	^ self new
		origin: origin;
		signal! !

!ResolutionRequest methodsFor: 'exceptionDescription' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
defaultAction
	| filedir ref |
	filedir := UIManager default chooseDirectory: 'Where is ', origin, '?'.
	ref := filedir ifNotNil: [FileSystem disk referenceTo: filedir fullName].
	self resume: ref! !

!ResolutionRequest methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:22'!
origin: aSymbol
	origin := aSymbol! !

TestCase subclass: #DirectoryEntryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!DirectoryEntryTest methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
entry
	^ FileLocator image resolve entry! !

!DirectoryEntryTest methodsFor: 'tests' stamp: 'StephaneDucasse 1/27/2011 22:26'!
testCreationTimeIsADateAndTimeInstance
	"While creation is the message sent to a directory entry, creation returns a DateAndTime object"

	| creation |
	creation := self entry creation.
	self assert: creation class = DateAndTime.
	! !

!DirectoryEntryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testIsDirectory
	| ref entry |
	ref := FileLocator imageDirectory resolve.
	entry := ref entry.
	self assert: entry isDirectory! !

!DirectoryEntryTest methodsFor: 'tests' stamp: 'StephaneDucasse 1/27/2011 22:26'!
testIsFile
	self assert: self entry isFile.
	self deny: self entry isDirectory! !

!DirectoryEntryTest methodsFor: 'tests' stamp: 'cwp 11/15/2009 22:10'!
testIsNotDirectory
	self deny: self entry isDirectory! !

!DirectoryEntryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testIsNotFile
	| ref |
	ref := FileLocator imageDirectory resolve.
	self deny: ref entry isFile! !

!DirectoryEntryTest methodsFor: 'tests' stamp: 'StephaneDucasse 1/27/2011 22:27'!
testModificationTimeIsADateAndTimeInstance
	"While modification is the message sent to a directory entry, modification returns a DateAndTime object"

	| modification |
	modification := self entry modification.
	self assert: modification class = DateAndTime.
	! !

!DirectoryEntryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testReference
	| ref entry |
	ref := FileLocator image resolve.
	entry := ref entry.
	self assert: entry reference = ref! !

!DirectoryEntryTest methodsFor: 'tests' stamp: 'cwp 11/15/2009 22:05'!
testSize
	self assert: self entry size isInteger! !

TestCase subclass: #FileLocatorTest
	instanceVariableNames: 'locator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testAsAbsolute
	locator := FileLocator image.
	self assert: locator asAbsolute = locator! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testBasename
	locator := FileLocator image / 'griffle'.
	self assert: locator basename = 'griffle'! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testCommaAddsExtension
	locator := FileLocator image / 'griffle'.
	self assert: (locator , 'plonk') basename = 'griffle.plonk'! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testCommaAddsExtensionAgain
	locator := FileLocator image / 'griffle.plonk'.
	self assert: (locator , 'nurp') basename = 'griffle.plonk.nurp'! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testContainsLocator
	locator := FileLocator image.
	self assert: (locator contains: locator / 'griffle').! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testContainsPath
	"self debug: #testContainsPath"
	
	locator := FileLocator image.
	self assert: (locator contains: (locator resolve / 'griffle') path).! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testContainsReference
	locator := FileLocator image.
	self assert: (locator contains: (locator resolve / 'griffle')).! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testEqual
	| a b |
	a := FileLocator image.
	b := FileLocator image.
	self deny: a == b.
	self assert: a = b.! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testFileSystem
	locator := FileLocator image.
	self assert: (locator fileSystem isKindOf: FileSystem)! !

!FileLocatorTest methodsFor: 'resolution tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testImageDirectory
	locator := FileLocator image.
	self assert: locator resolve = FileLocator image resolve! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testIsAbsolute
	locator := FileLocator image.
	self assert: locator isAbsolute! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testIsNotRoot
	locator := FileLocator image.
	self deny: locator isRoot! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testIsRelative
	locator := FileLocator image.
	self deny: locator isRelative! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testIsRoot
	locator := FileLocator image.
	(locator resolve path size) timesRepeat: [locator := locator / '..'].
	self assert: locator isRoot! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testOriginBasename
	locator := FileLocator image.
	self assert: locator basename = FileLocator image resolve basename! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testParent
	locator := FileLocator image.
	self assert: locator parent resolve = FileLocator imageDirectory resolve! !

!FileLocatorTest methodsFor: 'resolution tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testResolveAbsoluteReference
	| result reference |
	locator := FileLocator image / 'plonk'.
	reference := FileSystem memory / 'griffle'.
	result := locator resolve: reference..
	self assert: result == reference! !

!FileLocatorTest methodsFor: 'resolution tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testResolveCompoundString
	| result compound |
	locator := FileLocator image / 'plonk'.
	compound := 'griffle', locator fileSystem delimiter asString, 'nurp'.
	result := locator resolve: compound.
	self assert: result class = locator class.
	self assert: result origin = locator origin.
	self assert: result path = ((Path * 'plonk') / 'griffle' / 'nurp')! !

!FileLocatorTest methodsFor: 'resolution tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testResolvePath
	| result path |
	locator := FileLocator image / 'plonk'.
	result := locator resolve: (Path * 'griffle').
	path := (Path * 'plonk') / 'griffle'.
	self assert: result class= locator class.
	self assert: result origin = locator origin.
	self assert: result path = path.! !

!FileLocatorTest methodsFor: 'resolution tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testResolveRelativeReference
	| result reference |
	locator := FileLocator image / 'plonk'.
	self flag: 'this is a bit weird...'.
	reference := FileSystem memory * 'griffle'.
	result := locator resolve: reference..
	self assert: result class= locator class.
	self assert: result origin = locator origin.
	self assert: result path = reference path.! !

!FileLocatorTest methodsFor: 'resolution tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testResolveString
	| result path |
	locator := FileLocator image / 'plonk'.
	result := locator resolve: 'griffle'.
	path := (Path * 'plonk') / 'griffle'.
	self assert: result class= locator class.
	self assert: result origin = locator origin.
	self assert: result path = path.! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testSlash
	locator := FileLocator image / 'griffle'.
	self assert: locator = (FileLocator image / 'griffle')! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testWithExtensionAddsExtension
	locator := FileLocator image / 'griffle'.
	self assert: (locator withExtension: 'plonk') basename = 'griffle.plonk'! !

!FileLocatorTest methodsFor: 'compatibility tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testWithExtensionReplacesExtension
	locator := FileLocator image / 'griffle.nurp'.
	self assert: (locator withExtension: 'plonk') basename = 'griffle.plonk'! !

TestCase variableSubclass: #FileReferenceCreationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!FileReferenceCreationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
testCPath
	| ref |
	ref := FileReference C / 'WINDOWS'.
	self assert: (ref fileSystem isKindOf: FileSystem).
	self assert: ref path = (Path / 'C:' / 'WINDOWS')! !

TestCase variableSubclass: #FileReferenceTest
	instanceVariableNames: 'filesystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!FileReferenceTest methodsFor: 'support' stamp: 'cwp 11/17/2009 21:23'!
createFile: aPath
	filesystem ensureDirectory: aPath parent.
	(filesystem writeStreamOn: aPath) close! !

!FileReferenceTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
setUp
	filesystem := FileSystem memory.! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
testAllChildren
	"allChildren returns all the files and folders recursively nested in a reference"
	<publicTest>
	"self debug: #testAllChildren"
	| ref children |
	filesystem createDirectory: '/alpha'.
	filesystem createDirectory: '/alpha/beta'.
	filesystem createDirectory: '/alpha/beta/delta'.
	filesystem createDirectory: '/alpha/gamma'.
	
	ref := filesystem / 'alpha'.
	children := ref allChildren.
	"all children returns the nodes: '/alpha', '/alpha/beta',  '/alpha/beta/delta', and '/alpha/gamma'."
	self assert: children size = 4.
	children do:
		[:child |
		self assert: child class = FileReference.
		self assert: (ref = child or: [ref contains: child])].
	self assert: (children collect: [:ea | ea basename]) = #('alpha' 'beta' 'gamma' 'delta')! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
testAllDirectories
	"allDirectories returns all folders recursively nested in a reference"
	<publicTest>
	"self debug: #testAllDirectories"
	| ref children |
	filesystem createDirectory: '/alpha'.
	filesystem createDirectory: '/alpha/beta'.
	filesystem createDirectory: '/alpha/beta/delta'.
	filesystem createDirectory: '/alpha/gamma'.
	
	ref := filesystem / 'alpha'.
	children := ref allDirectories.
	"all children returns the directories: '/alpha', '/alpha/beta', and '/alpha/gamma'."
	self assert: children size = 4.
	
	children do:
		[:child |
		self assert: child class = FileReference.
		self assert: (ref = child or: [ref contains: child])].
	self assert: (children collect: [:ea | ea basename]) = #('alpha' 'beta' 'gamma' 'delta')! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
testAllEntries
	| ref entries |
	filesystem createDirectory: '/alpha'.
	filesystem createDirectory: '/alpha/beta'.
	filesystem createDirectory: '/alpha/beta/delta'.
	filesystem createDirectory: '/alpha/gamma'.
	
	ref := filesystem / 'alpha'.
	entries := ref allEntries.
	self assert: entries size = 4.
	entries do:
		[:entry |
		self assert: entry class = FileSystemDirectoryEntry.
		self assert: (ref = entry reference or: [ref contains: entry reference])].
	self assert: (entries collect: [:ea | ea basename]) = #('alpha' 'beta' 'gamma' 'delta')! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:44'!
testAsAbsoluteConverted
	"Converts a relative reference to absolute, and asserts
	that it's absolute and still has the same path."
	
	| ref absolute |
	ref := filesystem * 'plonk'.
	absolute := ref asAbsolute.
	self assert: absolute isAbsolute.
	self assert: (absolute path at: 1) = 'plonk'! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:44'!
testAsAbsoluteIdentity
	| ref |
	ref := filesystem / 'plonk'.
	self assert: ref asAbsolute == ref! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
testAsReference
	| ref |
	ref := filesystem * 'plonk'.
	self assert: ref asFileReference == ref! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:44'!
testBaseAndExtension
	"self debug: #testBaseAndExtension"
	
	| ref |
	ref := filesystem * 'plonk' / 'griffle.taz'.
	"We create a reference to the plonk/griffle.taz in the context of filesystem"
	self assert: ref base = 'griffle'.
	self assert: ref extension = 'taz'.
	
	"Note that the extension of a complex extension starts from the first period up until the end"
	ref := filesystem * 'plonk' / 'griffle.taz.txt'.
	self assert: ref base = 'griffle'.
	self assert: ref extension = 'taz.txt'.! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:43'!
testBasename
	"self debug: #testBasename"
	
	| ref |
	
	ref := filesystem root.
	self assert: ref basename = '/'.
	
	ref := filesystem * 'plonk' / 'griffle'.
	self assert: ref basename = 'griffle'.
	
	ref := filesystem * 'plonk' / 'griffle.taz'.
	self assert: ref basename = 'griffle.taz'.! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
testChildDirectories
	| childDirectories |
	filesystem createDirectory: '/beta'.
	filesystem createDirectory: '/gamma'.	
	(filesystem / 'delta')
		writeStreamDo: [ :stream | stream nextPutAll: '1' ].
	(filesystem / 'epsilon')
		writeStreamDo: [ :stream | stream nextPutAll: '2' ].
	childDirectories := filesystem root directories.
	self assert: childDirectories size = 2.
	childDirectories do: [ :each | 
		self assert: each class = FileReference.
		self assert: each isDirectory description: 'Collection should not contain references to files.' ]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
testChildFiles
	| childFiles |
	filesystem createDirectory: '/beta'.
	filesystem createDirectory: '/gamma'.	
	(filesystem / 'delta')
		writeStreamDo: [ :stream | stream nextPutAll: '1' ].
	(filesystem / 'epsilon')
		writeStreamDo: [ :stream | stream nextPutAll: '2' ].
	childFiles := filesystem root files.
	self assert: childFiles size = 2.
	childFiles do: [ :each | 
		self assert: each class = FileReference.
		self assert: each isFile description: 'Collection should not contain references to directories.' ]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testChildOfPath
	| parent  child |
	parent := Path / 'griffle'.
	child := filesystem / 'griffle' / 'nurb'.
	self deny: (child isChildOf: parent).
	self deny: (parent isChildOf: child).! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:42'!
testChildOfReference
	| parent  child |
	parent := filesystem / 'griffle'.
	child := filesystem / 'griffle' / 'nurb'.
	self assert: (child isChildOf: parent).
	self deny: (parent isChildOf: child).! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
testChildren
	| ref children |
	filesystem createDirectory: '/alpha'.
	filesystem createDirectory: '/alpha/beta'.
	filesystem createDirectory: '/alpha/gamma'.
	
	ref := filesystem / 'alpha'.
	children := ref children.
	self assert: children size = 2.
	children do:
		[:child |
		self assert: child class = FileReference.
		self assert: (child isChildOf: ref).
		self assert: (#('beta' 'gamma') includes: child basename)]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:42'!
testCommaAddsExtension
	| ref result |
	ref := filesystem * 'plonk'.
	result := ref, 'griffle'.
	self assert: result basename = 'plonk.griffle'! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:42'!
testCommaAddsExtensionAgain
	| ref result |
	ref := filesystem * 'plonk.griffle'.
	result := ref, 'nurp'.
	self assert: result basename = 'plonk.griffle.nurp'! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testContainsLocator
	| ref |
	ref := FileLocator imageDirectory resolve parent.
	self assert: (ref contains: FileLocator image)! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:50'!
testContainsPath
	| ref |
	ref := filesystem  * 'griffle'.
	self assert: (ref contains: (ref / 'nurp') path)! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:49'!
testContainsReference
	| ref |
	ref := filesystem * 'griffle'.
	self assert: (ref contains: ref / 'nurp')! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testDoesntContainLocator
	| ref |
	ref := filesystem * 'griffle'. 
	self deny: (ref contains: FileLocator image)! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testDoesntContainPath
	| ref |
	ref := filesystem * 'griffle'.
	self deny: (ref contains: (Path * 'nurp'))! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 13:18'!
testDoesntContainReferenceFileSystem
	| ref other |
	ref := filesystem  * 'griffle'.
	other := FileSystem memory / 'griffle' / 'nurp'.
	self deny: (ref contains: other)! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:49'!
testDoesntContainReferencePath
	| ref other |
	ref := filesystem * 'griffle'.
	other := filesystem * 'nurp'.
	self deny: (ref contains: other)! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
testEntries

	| ref entries |
	filesystem createDirectory: '/alpha'.
	filesystem createDirectory: '/alpha/beta'.
	filesystem createDirectory: '/alpha/gamma'.
	
	ref := filesystem  / 'alpha'.
	entries := ref entries.
	self assert: entries size = 2.
	entries do:
		[:entry |
		self assert: entry class = FileSystemDirectoryEntry.
		self assert: (entry reference isChildOf: ref).
		self assert: (#('beta' 'gamma') includes: entry reference basename)]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:42'!
testEqual

	| a b |
	a := filesystem * 'plonk'.
	b := filesystem * 'plonk'.
	self deny: a == b.
	self assert: a = b.! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/9/2011 16:12'!
testGlob
	| ref children |
	filesystem createDirectory: '/alpha'.
	filesystem createDirectory: '/alpha/beta'.
	filesystem createDirectory: '/alpha/gamma'.
	
	ref := filesystem root.
	children := ref glob: [ :node| true].
	self assert: children size == 4. "including root"
	
	children := ref glob: [ :node| node basename size > 1].
	self assert: children size == 3. "without root"
	
	children := ref glob: [ :node| node basename = #gamma].
	self assert: children size == 1. "gamma"
	self assert: children first basename = #gamma.! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:42'!
testGrandchildOfReference
	| griffle  nurb |
	griffle := filesystem / 'griffle'.
	nurb := filesystem / 'griffle' / 'plonk' / 'nurb'.
	self deny: (griffle isChildOf: nurb).
	self deny: (nurb isChildOf: griffle).! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:41'!
testIndicator
	"self debug: #testIndicator"
	
	| ref |
	ref := filesystem * 'plonk' / 'griffle'.
	self deny: ref exists.
	self assert: ref indicator = '?'.
	
	ref := filesystem workingDirectory / 'plonk'.
	self deny: ref exists.
	[ref createDirectory.
	 self assert: ref exists.
	self assert: ref isDirectory.
	 self assert: ref indicator equals: '/' ] ensure: [ref delete].
	
	ref := filesystem workingDirectory / 'plonk'.
	self deny: ref exists.
	[ref writeStreamDo: [:stream | stream nextPutAll: 'foo'  ] ifPresent: [self fail].
	 self assert: ref exists.
	self assert: ref isFile.
	 self assert: ref indicator equals: '' ] ensure: [ref delete].! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:41'!
testIsAbsolute
	self assert: (filesystem / 'plonk') isAbsolute! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:41'!
testIsNotAbsolute
	self deny: (filesystem * 'plonk') isAbsolute! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:31'!
testIsNotRelative
	self deny: (filesystem / 'plonk') isRelative! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:49'!
testIsNotRoot
	self deny: (filesystem / 'plonk') isRoot! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:31'!
testIsRelative
	self assert: (filesystem * 'plonk') isRelative! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:49'!
testIsRoot
	self assert: (filesystem root) isRoot! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testMakeRelative
	"self run: #testMakeRelative"

	| parent child relative |
	parent := filesystem / 'griffle'.
	child := filesystem / 'griffle' / 'plonk' / 'nurb'.
	relative := parent makeRelative: child.
	self assert: relative = (Path * 'plonk' / 'nurb')! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:48'!
testParent
	| ref parent |
	ref := filesystem * 'plonk' / 'griffle'.
	parent := ref parent.
	self assert: parent class = ref class.
	self assert: (parent path at: 1) = 'plonk'! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 13:09'!
testParentResolutionWithAbsoluteReference
	| base relative absolute |
	base := (filesystem / '/plonk' / 'pinto').
	relative := (FileSystem memory / 'griffle' / 'zonk').
	absolute := base resolve: relative.
	self assert: absolute fileSystem == relative fileSystem.
	self assert: absolute isAbsolute.
	self assert: (absolute path at: 1) = 'griffle'.
	self assert: (absolute path at: 2) = 'zonk'.
	
	! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testParentResolutionWithPath
	| base relative absolute |
	base := filesystem / 'plonk' / 'pinto'.
	relative := Path parent / 'griffle' / 'zonk'.
	absolute := base resolve: relative.
	self assert: absolute isAbsolute.
	self assert: (absolute path at: 1) = 'plonk'.
	self assert: (absolute path at: 2) = 'griffle'.
	self assert: (absolute path at: 3) = 'zonk'.
	! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:48'!
testParentResolutionWithReference
	| base relative absolute |
	base := (filesystem / 'plonk' / 'pinto').
	relative := (filesystem referenceTo: '../griffle/zonk').
	absolute := base resolve: relative.
	self assert: absolute isAbsolute.
	self assert: (absolute path at: 1) = 'plonk'.
	self assert: (absolute path at: 2) = 'griffle'.
	self assert: (absolute path at: 3) = 'zonk'.
	
	! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
testParentResolutionWithRemoteReference
	| base relative absolute |
	base := (filesystem / 'plonk' / 'pinto').
	relative := (FileSystem memory referenceTo: '../griffle/zonk').
	absolute := base resolve: relative.
	self assert: absolute isAbsolute.
	self assert: (absolute path at: 1) = 'plonk'.
	self assert: (absolute path at: 2) = 'griffle'.
	self assert: (absolute path at: 3) = 'zonk'.
	
	! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testPathRelativeTo
	| parent childPath relative |
	parent := filesystem / 'griffle'.
	childPath := Path / 'griffle' / 'plonk' / 'nurb'.
	relative := childPath relativeTo: parent.
	self assert: relative = (Path * 'plonk' / 'nurb')! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testReadStream
	| ref stream path |
	path := Path * 'plonk'.
	filesystem store createFile: path.
	ref := filesystem referenceTo: path.
	[stream := ref readStream.
	self assert: stream class = FileSystemReadStream ] 
		ensure: [ stream ifNotNil: [ stream close ] ]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testReadStreamDo
	| ref path s |
	path := Path * 'plonk'.
	filesystem store createFile: path.
	ref := filesystem referenceTo: path.
	ref
		readStreamDo: [ :stream | 
			self assert: stream class = FileSystemReadStream.
			self assert: stream isOpen.
			s := stream ].
	self deny: s isOpen! !

!FileReferenceTest methodsFor: 'tests stream' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
testReadStreamDoNotFound
	| ref |
	ref := filesystem / 'plonk'.
	self
		should: [ref readStreamDo: [:s]]
		raise: FileDoesNotExist ! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testReadStreamDoifAbsent
	| ref path s |
	path := Path * 'plonk'.
	filesystem store createFile: path.
	ref := filesystem referenceTo: path.
	ref
		readStreamDo: [ :stream | 
			self assert: stream class = FileSystemReadStream.
			self assert: stream isOpen.
			s := stream ]
		ifAbsent: [ self signalFailure: 'The file exists!!' ].
	self deny: s isOpen! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:47'!
testReadStreamDoifAbsentNot
	| ref pass |
	pass := false.
	ref := filesystem * 'plonk'.
	ref 
		readStreamDo: [:stream]
		ifAbsent: [pass := true].
	self assert: pass! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testReadStreamIfAbsent
	| ref stream path |
	path := Path * 'plonk'.
	filesystem store createFile: path.
	ref := filesystem referenceTo: path.
	[ 
	stream := ref readStreamIfAbsent: [ self signalFailure: 'Should not reach here.' ].
	self assert: stream class = FileSystemReadStream ] 
		ensure: [ stream ifNotNil: [ stream close ] ]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
testReadStreamNotFound
	| ref  |
	ref := filesystem * 'plonk'.
	self
		should: [ref readStream]
		raise: FileDoesNotExist ! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRelativeToPath
	| parentPath child relative |
	parentPath := Path / 'griffle'.
	child := filesystem / 'griffle' / 'plonk' / 'nurb'.
	relative := child relativeTo: parentPath.
	self assert: relative = (Path * 'plonk' / 'nurb')! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRelativeToReference
	| parent child relative |
	parent := filesystem / 'griffle'.
	child := filesystem  / 'griffle' / 'plonk' / 'nurb'.
	relative := child relativeTo: parent.
	self assert: relative = (Path * 'plonk' / 'nurb')! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:47'!
testResolve
	| ref |
	ref := filesystem / 'griffle'.
	self assert: ref resolve == ref! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:47'!
testRootParent
	| root |
	root := filesystem root.
	self assert: root parent == root! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:46'!
testSiblingOfReference
	| griffle  nurb |
	griffle := filesystem / 'griffle'.
	nurb := filesystem / 'nurb'.
	self deny: (griffle isChildOf: nurb).
	self deny: (nurb isChildOf: griffle).! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testSimpleResolution
	| base relative absolute |
	base := filesystem / 'plonk'.
	relative := (Path * 'griffle') / 'zonk'.
	absolute := base resolve: relative.
	self assert: absolute isAbsolute.
	self assert: (absolute path at: 1) = 'plonk'.
	self assert: (absolute path at: 2) = 'griffle'.
	self assert: (absolute path at: 3) = 'zonk'.
	
	! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:31'!
testSlash
	| ref result |
	ref := filesystem * 'plonk'.
	result := ref / 'griffle'.
	self assert: result class = ref class.
	self assert: result  isRelative.
	self assert: (result path at: 1) = 'plonk'.
	self assert: (result path at: 2) = 'griffle'.
! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:46'!
testUnequalContent
	| a b |
	a := filesystem * 'plonk'.
	b := filesystem * 'griffle'.
	self deny: a = b.! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:46'!
testUnequalSize
	| a b |
	a := filesystem * 'plonk'.
	b := filesystem / 'plonk' / 'griffle'.
	self deny: a = b.! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:31'!
testWithExtentionAddsExtension
	| ref result |
	ref := filesystem * 'plonk'.
	result := ref withExtension: 'griffle'.
	self assert: result isRelative.
	self assert: result basename = 'plonk.griffle'! !

!FileReferenceTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:24'!
testWithExtentionReplacesExtension
	| ref result |
	ref := filesystem * 'plonk.griffle'.
	result := ref withExtension: 'nurp'.
	self assert: result isRelative.
	self assert: result basename = 'plonk.nurp'! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testWorkingDirectoryParent
	| wd |
	wd := filesystem referenceTo: Path workingDirectory.
	self assert: wd parent path size = 1.
	self assert: (wd parent path at: 1) = '..'.! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:36'!
testWriteStream
	| ref stream |
	ref := filesystem / 'plonk'.
	[stream := ref writeStream.
	self assert: (stream class = FileSystemWriteStream)]
		ensure: [stream ifNotNil: [stream close]]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:36'!
testWriteStreamDo
	| ref s |
	ref := filesystem / 'plonk'.
	ref writeStreamDo:
		[:stream |
		s := stream.
		self assert: stream class = FileSystemWriteStream.
		self assert: stream isOpen].
	self deny: s isOpen
		! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testWriteStreamDoExists
	| ref s path |
	path := Path * 'plonk'.
	filesystem store createFile: path.
	ref := filesystem referenceTo: path.
	ref
		writeStreamDo: [ :stream | 
			s := stream.
			self assert: stream class = FileSystemWriteStream.
			self assert: stream isOpen ].
	self deny: s isOpen! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:36'!
testWriteStreamDoifPresent
	| ref s |
	ref := filesystem / 'plonk'.
	ref 
		writeStreamDo:
			[:stream |
			s := stream.
			self assert: stream class = FileSystemWriteStream.
			self assert: stream isOpen]
		ifPresent: [self signalFailure: 'The file does not exist!!'].
	self deny: s isOpen
		! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testWriteStreamDoifPresentNot
	| ref pass path |
	pass := false.
	path := Path * 'plonk'.
	filesystem store createFile: path.
	ref := filesystem referenceTo: path.
	ref writeStreamDo: [ :stream |  ] ifPresent: [ pass := true ].
	self assert: pass! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testWriteStreamExists
	| ref stream path |
	path := Path * 'plonk'.
	filesystem store createFile: path.
	ref := filesystem referenceTo: path.
	[stream := ref writeStream.
	self assert: stream class = FileSystemWriteStream ] 
		ensure: [ stream ifNotNil: [ stream close ] ]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:36'!
testWriteStreamifPresent
	| ref stream |
	ref := filesystem / 'plonk'.
	[stream := ref writeStreamIfPresent: [self signalFailure: 'Should not reach here'].
	self assert: (stream class = FileSystemWriteStream)]
		ensure: [stream ifNotNil: [stream close]]! !

!FileReferenceTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testWriteStreamifPresentExists
	| ref pass path |
	pass := false.
	path := Path * 'plonk'.
	filesystem store createFile: path.
	ref := filesystem referenceTo: path.
	ref writeStreamIfPresent: [ pass := true ].
	self assert: pass! !

TestCase subclass: #FileSystemHandleTest
	instanceVariableNames: 'filesystem handle reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

FileSystemHandleTest subclass: #FileHandleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Disk'!

!FileHandleTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/3/2012 11:42'!
createFileSystem
	^ FileSystem store: DiskStore activeClass createDefault! !

!FileSystemHandleTest class methodsFor: 'testing' stamp: 'EstebanLorenzano 4/3/2012 09:38'!
isAbstract
	^ self name = #FileSystemHandleTest! !

!FileSystemHandleTest class methodsFor: 'testing' stamp: 'cwp 7/26/2009 12:46'!
shouldInheritSelectors 
	^ true! !

!FileSystemHandleTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/3/2012 11:43'!
createFileSystem
	self subclassResponsibility ! !

!FileSystemHandleTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/3/2012 11:42'!
setUp
	filesystem := self createFileSystem.
	reference := filesystem * 'plonk'.
	handle := reference openWritable: true! !

!FileSystemHandleTest methodsFor: 'running' stamp: 'cwp 7/26/2009 12:23'!
tearDown
	handle close.
	reference delete! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 22:06'!
testAt
	handle at: 1 write: (ByteArray with: 3) startingAt: 1 count: 1.
	self assert: (handle at: 1) = 3! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 22:03'!
testAtPut
	| in |
	handle at: 1 put: 3.
	in := ByteArray new: 1.
	handle at: 1 read: in startingAt: 1 count: 1.
	self assert: in first = 3! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'lr 4/13/2010 16:10'!
testAtPutBinaryAscii
	self shouldnt: [ handle at: 1 put: 32 ] raise: Error.
	self shouldnt: [ handle at: 1 put: Character space ] raise: Error! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'lr 4/13/2010 16:07'!
testAtWriteBinaryAscii
	self shouldnt: [ handle at: 1 write: #[32] startingAt: 1 count: 1 ] raise: Error.
	self shouldnt: [ handle at: 1 write: (String with: Character space) startingAt: 1 count: 1 ] raise: Error! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/26/2009 12:23'!
testClose
	handle close.
	self deny: handle isOpen
	! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'CamilloBruni 5/24/2012 15:21'!
testCreatedOpen
	
	
	self flag: 'TODO: activated once FileHandle is in use again!!'.
	"self assert: handle isOpen"! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/26/2009 14:15'!
testIO
	| out in |
	out := #(1 2 3) asByteArray.
	in := ByteArray new: 3.
	handle at: 1 write: out startingAt: 1 count: 3.
	handle at: 1 read: in startingAt: 1 count: 3.
	self assert: out = in.! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/31/2009 00:13'!
testReadBufferTooLarge
	| out in result |
	out := #(1 2 3) asByteArray.
	in := ByteArray new: 5.
	in atAllPut: 9.
	handle at: 1 write: out startingAt: 1 count: 3.
	result := handle at: 1 read: in startingAt: 2 count: 4.
	self assert: result = 3.
	self assert: in = #(9 1 2 3 9) asByteArray.! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/26/2009 14:16'!
testReadOnly
	handle close.
	handle := reference openWritable: false.
	self 
		should: 
			[ handle 
				at: 1
				write: #(1 2 3 )
				startingAt: 1
				count: 3 ]
		raise: Error! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/26/2009 12:45'!
testReference
	self assert: handle reference = reference asAbsolute! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 22:11'!
testSizeAfterGrow
	| out |
	out := #(1 2 3) asByteArray.
	handle at: 1 write: out startingAt: 1 count: 3.
	self assert: handle size = 3! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 22:13'!
testSizeNoGrow
	| bytes |
	bytes := #(1 2 3 4) asByteArray.
	handle at: 1 write: bytes startingAt: 1 count: 3.
	handle at: 4 write: bytes startingAt: 4 count: 1.
	self assert: handle size = 4! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/26/2009 14:16'!
testTruncate
	| out |
	out := #(1 2 3 4 5) asByteArray.
	handle at: 1 write: out startingAt: 1 count: 5.
	handle truncateTo: 3.
	self assert: handle size = 3! !

!FileSystemHandleTest methodsFor: 'tests' stamp: 'cwp 7/28/2009 22:40'!
testWriteStream
	| stream |
	stream := handle writeStream.
	self assert: (stream respondsTo: #nextPut:)! !

FileSystemHandleTest subclass: #MemoryHandleTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Memory'!

!MemoryHandleTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/3/2012 12:51'!
createFileSystem
	^ FileSystem memory! !

TestCase subclass: #FileSystemReadStreamTest
	instanceVariableNames: 'filesystem stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-AnsiStreams'!

!FileSystemReadStreamTest methodsFor: 'support' stamp: 'cwp 7/29/2009 22:43'!
contents: aByteArray
	stream := filesystem writeStreamOn: 'griffle'.
	stream nextPutAll: aByteArray.
	stream close.
	stream := filesystem readStreamOn: 'griffle'! !

!FileSystemReadStreamTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
setUp
	filesystem := FileSystem memory.
	! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 22:38'!
testAtEnd
	self contents: #().
	self assert: stream atEnd! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 22:57'!
testDo
	| contents |
	self contents: #(1 2 3).
	contents := Array streamContents:
		[:out |
		stream do: [:ea | out nextPut: ea]].
	self assert: contents = #(1 2 3)! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 23:00'!
testNext
	self contents: #(1 2 3).
	self assert: stream next = 1! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 23:07'!
testNextColon
	| result |
	self contents: #(1 2 3 4 5).
	result := stream next: 3.
	self assert: result = #(1 2 3) asByteArray.! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:54'!
testNextInto
	| result buffer |
	self contents: #(1 2 3 4 5).
	buffer := ByteArray new: 3.
	result := stream nextInto: buffer.
	self assert: result == buffer.
	self assert: result = #(1 2 3) asByteArray! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 23:30'!
testNextLineCR
	| result tail |
	self contents: #(97 97 97 13 98 98 98).
	result := stream nextLine.
	self assert: result = #(97 97 97) asByteArray.
	tail := stream next: 3.
	self assert: tail = #(98 98 98) asByteArray! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 23:31'!
testNextLineCRLF
	| line tail |
	self contents: #(97 97 97 13 10 98 98 98).
	line := stream nextLine.
	self assert: line = #(97 97 97) asByteArray.
	tail := stream next: 3.
	self assert: tail = #(98 98 98) asByteArray! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 23:31'!
testNextLineLF
	| result tail |
	self contents: #(97 97 97 10 98 98 98).
	result := stream nextLine.
	self assert: result = #(97 97 97) asByteArray.
	tail := stream next: 3.
	self assert: tail = #(98 98 98) asByteArray! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:04'!
testNextMatchFalse
	self contents: #(1 2 3).
	self deny: (stream nextMatchFor: 5)! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:01'!
testNextMatchTrue
	self contents: #(1 2 3).
	self assert: (stream nextMatchFor: 1)! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 1/9/2012 18:07'!
testNextPutAllThenNextPut

	| sourceArray loadedArray serializer materializer |

	stream := filesystem writeStreamOn: 'griffle'.
	stream nextPutAll: 'xxx' asByteArray.
	stream nextPut: 42.
	stream close. 

	stream := filesystem readStreamOn: 'griffle'.
	self assert: (stream next: 3) asString = 'xxx'.
	self assert: stream next  = 42.! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 22:46'!
testNotAtEnd
	self contents: #(1 2 3).
	self deny: stream atEnd! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:05'!
testPeek
	self contents: #(1 2 3).
	self assert: stream peek = 1.
	self assert: stream next = 1.! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:06'!
testPeekForFalse
	self contents: #(1 2 3).
	self deny: (stream peekFor: 5)
	! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:06'!
testPeekForTrue
	self contents: #(1 2 3).
	self assert: (stream peekFor: 1)
	! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:17'!
testSkip
	self contents: #(1 2 3 4 5).
	stream skip: 3.
	self assert: stream position = 4! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:18'!
testSkipPastEnd
	self contents: #(1 2 3 4 5).
	stream skip: 10.
	self assert: stream atEnd! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:20'!
testSkipTo
	self contents: #(1 2 3 4 5).
	stream skipTo: 3.
	self assert: stream position = 4! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:24'!
testSkipToEnd
	self contents: #(1 2 3 4 5).
	stream skipTo: 10.
	self assert: stream atEnd! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:27'!
testSkipToFalse
	self contents: #(1 2 3 4 5).
	self deny: (stream skipTo: 10).
	! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:25'!
testSkipToTrue
	self contents: #(1 2 3 4 5).
	self assert: (stream skipTo: 4).
	! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:38'!
testUpTo
	| result |
	self contents: #(1 2 3 4 5).
	result := stream upTo: 4.
	self assert: result = #(1 2 3) asByteArray! !

!FileSystemReadStreamTest methodsFor: 'tests' stamp: 'cwp 7/30/2009 23:38'!
testUpToEnd
	| result |
	self contents: #(1 2 3 4 5).
	result := stream upTo: 7.
	self assert: result = #(1 2 3 4 5) asByteArray! !

TestCase subclass: #FileSystemResolverTest
	instanceVariableNames: 'resolver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!FileSystemResolverTest class methodsFor: 'testing' stamp: 'EstebanLorenzano 4/3/2012 09:40'!
isAbstract
	^ self name = #FileSystemResolverTest! !

!FileSystemResolverTest methodsFor: 'asserting' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
assertOriginResolves: aSymbol
	| reference |
	reference := resolver resolve: aSymbol.
	self assert: (reference isKindOf: FileReference).
	self assert: reference isAbsolute.
	self assert: reference exists! !

!FileSystemResolverTest methodsFor: 'running' stamp: 'cwp 10/27/2009 11:12'!
createResolver
	self subclassResponsibility ! !

!FileSystemResolverTest methodsFor: 'running' stamp: 'cwp 10/26/2009 21:25'!
setUp
	resolver := self createResolver.! !

FileSystemResolverTest subclass: #InteractiveResolverTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!InteractiveResolverTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
createResolver
	^ InteractiveResolver new! !

!InteractiveResolverTest methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
home
	^ FileLocator imageDirectory resolve! !

!InteractiveResolverTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
testCached
	[resolver resolve: #home]
		on: ResolutionRequest
		do: [:req | req resume: self home].
	self
		shouldnt: [self assertOriginResolves: #home]
		raise: ResolutionRequest! !

!InteractiveResolverTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
testNew
	[self assertOriginResolves: #home]
		on: ResolutionRequest
		do: [:req | req resume: self home].
	! !

FileSystemResolverTest subclass: #PlatformResolverTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!PlatformResolverTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:39'!
createResolver
	^ PlatformResolver forCurrentPlatform! !

!PlatformResolverTest methodsFor: 'tests' stamp: 'cwp 10/27/2009 10:57'!
testHome
	self assertOriginResolves: #home! !

FileSystemResolverTest subclass: #SystemResolverTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!SystemResolverTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:39'!
createResolver
	^ SystemResolver new! !

!SystemResolverTest methodsFor: 'testing' stamp: 'lr 7/13/2010 13:32'!
testChanges
	self assertOriginResolves: #changes! !

!SystemResolverTest methodsFor: 'testing' stamp: 'cwp 10/26/2009 13:21'!
testImage
	self assertOriginResolves: #image! !

!SystemResolverTest methodsFor: 'testing' stamp: 'lr 7/13/2010 13:35'!
testImageDirectory
	self assertOriginResolves: #imageDirectory! !

!SystemResolverTest methodsFor: 'testing' stamp: 'cwp 10/26/2009 13:21'!
testVmBinary
	self assertOriginResolves: #vmBinary! !

!SystemResolverTest methodsFor: 'testing' stamp: 'cwp 10/26/2009 13:21'!
testVmDirectory
	self assertOriginResolves: #vmDirectory! !

TestCase subclass: #FileSystemTest
	instanceVariableNames: 'filesystem toDelete'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

FileSystemTest subclass: #DiskFileSystemTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Disk'!

!DiskFileSystemTest methodsFor: 'initialize-release' stamp: 'EstebanLorenzano 4/3/2012 13:17'!
createFileSystem
	^ FileSystem store: (DiskStore activeClass createDefault)! !

!DiskFileSystemTest methodsFor: 'tests' stamp: 'IgorStasenko 8/17/2011 17:01'!
testDefaultWorkingDirectory
	| ref |
	ref := filesystem workingDirectory.
	self assert: (Smalltalk vm imagePath beginsWith: ref asString)! !

!DiskFileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 11:42'!
testEqual
	| other |
	other := self createFileSystem.
	self assert: filesystem = other! !

!DiskFileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testIsDirectory
	self assert: (filesystem isDirectory: FileLocator imageDirectory resolve path)! !

!FileSystemTest class methodsFor: 'testing' stamp: 'EstebanLorenzano 4/3/2012 09:37'!
isAbstract
	^ self name = #FileSystemTest! !

!FileSystemTest class methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 12:53'!
packageNamesUnderTest
	^ #('FileSystem')! !

!FileSystemTest class methodsFor: 'testing' stamp: 'cwp 7/20/2009 08:56'!
shouldInheritSelectors
	^ true
	! !

!FileSystemTest methodsFor: 'initialize-release' stamp: 'EstebanLorenzano 4/3/2012 13:04'!
createFileSystem
	self subclassResponsibility ! !

!FileSystemTest methodsFor: 'initialize-release' stamp: 'cwp 10/10/2009 17:35'!
delete: anObject
	toDelete add: (filesystem resolve: anObject)! !

!FileSystemTest methodsFor: 'initialize-release' stamp: 'EstebanLorenzano 4/3/2012 11:42'!
setUp
	filesystem := self createFileSystem.
	toDelete := OrderedCollection new.! !

!FileSystemTest methodsFor: 'initialize-release' stamp: 'cwp 8/23/2009 23:09'!
tearDown
	toDelete do: [:path | filesystem delete: path]! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testChangeDirectory
	| cwd |
	filesystem workingDirectoryPath: Path / 'plonk'.
	filesystem changeDirectory: 'griffle'.
	cwd := filesystem workingDirectoryPath.
	self assert: cwd isAbsolute.
	self assert: (cwd at: 1) = 'plonk'.
	self assert: (cwd at: 2) = 'griffle'.
! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testChangeDirectoryString
	| cwd |
	filesystem workingDirectoryPath: (Path / 'plonk').
	filesystem changeDirectory: 'griffle'.
	cwd := filesystem workingDirectoryPath.
	self assert: cwd isAbsolute.
	self assert: (cwd at: 1) = 'plonk'.
	self assert: (cwd at: 2) = 'griffle'! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testChildrenAt
	| directory entries |
	directory := Path * 'plonk'.
	filesystem createDirectory: directory.
	filesystem createDirectory: directory / 'griffle'.
	filesystem createDirectory: directory / 'bint'.
	self delete: directory / 'griffle'.
	self delete: directory / 'bint'.
	self delete: directory.
	entries := filesystem childrenAt: directory.
	self assert: entries size = 2.
	entries do: 
		[ :ea | 
		self assert: (ea isKindOf: Path).
		self assert: ea parent = (filesystem resolve: directory).
		self assert: (#('griffle' 'bint' ) includes: ea basename) ]! !

!FileSystemTest methodsFor: 'tests' stamp: 'lr 7/13/2010 15:19'!
testCopy
	| out in contents |
	[ 
	out := filesystem writeStreamOn: 'gooly'.
	[ out nextPutAll: 'gooly' ] ensure: [ out close ].
	filesystem copy: 'gooly' to: 'plonk'.
	in := filesystem readStreamOn: 'plonk'.
	contents := [ in contents asString ] ensure: [ in close ].
	self assert: contents = 'gooly' ]
		ensure: [ 
			filesystem
				delete: 'gooly';
				delete: 'plonk' ]! !

!FileSystemTest methodsFor: 'tests' stamp: 'abc 5/11/2012 23:36'!
testCopyDestExists
	| out |
	
	[out := (filesystem open: 'gooly' writable: true) writeStream.
	[out nextPutAll: 'gooly'] ensure: [out close].

	(filesystem open: 'plonk' writable: true) writeStream close.
	
	self 
		should: [filesystem copy: 'gooly' to: 'plonk']
		raise: FileExists] 
			ensure: [filesystem delete: 'gooly'; delete: 'plonk']! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
testCopySourceDoesntExist
	self 
		should: [filesystem copy: 'plonk' to: 'griffle']
		raise: FileDoesNotExist! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testCreateDirectoryExists
	| path |
	path := Path * 'griffle'.
	self delete: path.
	filesystem createDirectory: path.
	self 
		should: [filesystem createDirectory: path]
		raise: DirectoryExists.
		

	
	
! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testCreateDirectoryNoParent
	| path |
	path := Path * 'griffle' / 'nurp'.
	self 
		should: [filesystem createDirectory: path]
		raise: DirectoryDoesNotExist.
	
	
! !

!FileSystemTest methodsFor: 'tests' stamp: 'cwp 7/20/2009 07:30'!
testDefaultWorkingDirectory
	self assert: filesystem workingDirectory isRoot! !

!FileSystemTest methodsFor: 'tests' stamp: 'cwp 7/20/2009 07:30'!
testDelimiter
	self assert: filesystem delimiter isCharacter! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testDirectory
	| path |
	path := Path * 'plonk'.
	filesystem createDirectory: path.
	self assert: (filesystem exists: path).
	self assert: (filesystem isDirectory: path).
	self deny: (filesystem isFile: path).
	filesystem delete: path.
	self deny: (filesystem exists: path)! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testEnsureDirectory
	| path |
	path := Path * 'plonk'.
	self delete: path.
	filesystem ensureDirectory: path.
	self assert: (filesystem isDirectory: path).! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testEnsureDirectoryCreatesParent
	| path |
	path := Path * 'plonk' / 'griffle'.
	self delete: path.
	self delete: path parent.
	self 
		shouldnt: [filesystem ensureDirectory: path]
		raise: FileSystemError.
	self assert: (filesystem isDirectory: (Path * 'plonk')).
	self assert: (filesystem isDirectory: path).
	! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testEnsureDirectoryExists
	| path |
	path := Path * 'plonk'.
	self delete: path.
	filesystem createDirectory: path.
	self 
		shouldnt: [filesystem ensureDirectory: path]
		raise: FileSystemError.
	! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
testEntriesAt
	| directory entries |
	directory := Path * 'plonk'.
	filesystem createDirectory: directory.
	filesystem createDirectory: directory / 'griffle'.
	filesystem createDirectory: directory / 'bint'.
	self delete: directory / 'griffle'.
	self delete: directory / 'bint'.
	self delete: directory.
	entries := filesystem entriesAt: directory.
	self assert: entries size = 2.
	entries do: 
		[ :ea | 
		self assert: (ea isKindOf: FileSystemDirectoryEntry).
		self assert: ea reference parent path = (filesystem resolve: directory).
		self assert: (#('griffle' 'bint' ) includes: ea reference basename).
		self assert: ea isDirectory ]! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testEntryAt
	| path entry |
	path := Path * 'plonk'.
	filesystem createDirectory: path.
	self delete: path.
	entry := filesystem entryAt: path.
	self assert: entry isDirectory.
	self assert: entry reference = (filesystem referenceTo: path) asAbsolute! !

!FileSystemTest methodsFor: 'tests' stamp: 'abc 5/11/2012 23:35'!
testFile
	| path |
	path := Path * 'gooly'.
	(filesystem open: path writable: true) writeStream close.
	self assert: (filesystem exists: path).
	self deny: (filesystem isDirectory: path).
	self assert: (filesystem isFile: path).
	filesystem delete: path.
	self deny: (filesystem exists: path)! !

!FileSystemTest methodsFor: 'tests' stamp: 'MarcusDenker 3/16/2012 08:51'!
testFileNames
	| reference |
	#('test one' 'test with two') do: [ :each |
		reference := filesystem workingDirectory / each.
		self assert: reference basename = each.
		self deny: reference exists.
		reference
			writeStreamDo: [ :stream | stream nextPutAll: 'gooly' ]
			ifPresent: [ self fail ].
		[	self assert: reference exists.
			self assert: (filesystem workingDirectory children
				anySatisfy: [ :ref | ref = reference ]) ]
			ensure: [ reference delete ] ]! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'sd 2/11/2011 18:21'!
testFileStreamDo
	| reference |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	self assert: (reference fileStreamDo: [ :stream |
		stream nextPutAll: 'griffle'.
		true ]).
	self assert: (filesystem workingDirectory / 'griffle') isFile.
	self assert: (reference fileStreamDo: [ :stream | true ])! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
testReadStream
	| reference stream |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	self
		should: [ reference readStream ]
		raise: FileDoesNotExist.
	reference writeStreamDo: [ :ws | ws nextPutAll: 'griffle' ].
	self
		shouldnt: [ stream := reference readStream ]
		raise: FileDoesNotExist.
	self assert: stream contents asString = 'griffle'.
	stream close! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
testReadStreamDo
	| reference |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	self 
		should: [ reference readStreamDo: [ :stream | self assert: false ] ] 
		raise: FileDoesNotExist.
	reference writeStreamDo: [ :ws | ws nextPutAll: 'griffle' ].
	self assert: (reference readStreamDo: [ :stream | stream contents asString ]) 
		= 'griffle'! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'sd 2/11/2011 18:21'!
testReadStreamDoIfAbsent
	| reference |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	self assert: (reference 
		readStreamDo: [ :stream | false ]
		ifAbsent: [ true ]).
	reference writeStreamDo: [ :ws | ws nextPutAll: 'griffle' ].
	self assert: (reference 
		readStreamDo: [ :stream | stream contents asString = 'griffle' ]
		ifAbsent: [ false ])! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'sd 2/11/2011 18:21'!
testReadStreamIfAbsent
	| reference stream |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	self assert: (reference readStreamIfAbsent: [ true ]).
	reference writeStreamDo: [ :ws | ws nextPutAll: 'griffle' ].
	stream := reference readStreamIfAbsent: [ false ].
	self assert: stream contents asString = 'griffle'.
	stream close! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testReferenceTo
	|path|
	path := Path / 'a' / 'b'.
	self assert: (filesystem referenceTo: '/') isRoot.
	self assert: (filesystem referenceTo: '/a/b') path = path.! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 13:06'!
testRoot
	self assert: filesystem root fileSystem = filesystem.
	self assert: filesystem root path = Path root.
	self assert: filesystem root isRoot.! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRootExists
	self assert: (filesystem exists: Path root)! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRootIsDirectory
	self assert: (filesystem isDirectory: Path root)! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRootIsNotAFile
	self deny: (filesystem isFile: Path root)! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testSetRelativeWorkingDirectory
	self 
		should: [filesystem workingDirectoryPath: (Path * 'plonk')]
		raise: Error
	! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testSetWorkingDirectory
	| cwd |
	filesystem workingDirectoryPath: (Path / 'plonk').
	cwd := filesystem workingDirectoryPath.
	self assert: cwd isAbsolute.
	self assert: (cwd at: 1) = 'plonk'! !

!FileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 13:06'!
testWorking
	self assert: filesystem workingDirectory fileSystem = filesystem.
	self assert: filesystem workingDirectory path = filesystem workingDirectoryPath! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'sd 2/11/2011 18:21'!
testWriteStream
	| reference stream |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	stream := reference writeStream.
	stream nextPutAll: 'griffle'.
	stream close.
	self assert: (filesystem workingDirectory / 'griffle') isFile.
	stream := reference writeStream.
	stream close! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'sd 2/11/2011 18:21'!
testWriteStreamDo
	| reference |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	self assert: (reference writeStreamDo: [ :stream |
		stream nextPutAll: 'griffle'.
		true ]).
	self assert: (filesystem workingDirectory / 'griffle') isFile.
	self assert: (reference writeStreamDo: [ :stream | true ])! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'sd 2/11/2011 18:21'!
testWriteStreamDoIfPresent
	| reference |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	self assert: (reference 
		writeStreamDo: [ :stream |
			stream nextPutAll: 'griffle'.
			true ]
		ifPresent: [ false ]).
	self assert: (filesystem workingDirectory / 'griffle') isFile.
	self assert: (reference 
		writeStreamDo: [ :stream | true ]
		ifPresent: [ true ])! !

!FileSystemTest methodsFor: 'tests-references' stamp: 'sd 2/11/2011 18:21'!
testWriteStreamIfPresent
	| reference stream |
	self delete: (reference := filesystem workingDirectory / 'griffle').
	stream := reference writeStreamIfPresent: [ false ].
	stream nextPutAll: 'griffle'.
	stream close.
	self assert: (filesystem workingDirectory / 'griffle') isFile.
	self assert: (reference writeStreamIfPresent: [ true ])! !

FileSystemTest subclass: #MemoryFileSystemTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Memory'!

!MemoryFileSystemTest methodsFor: 'initialize-release' stamp: 'EstebanLorenzano 4/3/2012 13:04'!
createFileSystem
	^ FileSystem memory! !

!MemoryFileSystemTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 11:42'!
testEqual
	| other |
	other := self createFileSystem.
	self deny: filesystem = other! !

TestCase subclass: #FileSystemTreeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

FileSystemTreeTest subclass: #CopyVisitorTest
	instanceVariableNames: 'source dest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!CopyVisitorTest methodsFor: 'running' stamp: 'cwp 2/18/2011 16:40'!
createDirectory: aString 
	source createDirectory: (source store pathFromString: aString)! !

!CopyVisitorTest methodsFor: 'running' stamp: 'cwp 2/18/2011 16:40'!
createFile: aString 
	source store createFile: (source store pathFromString: aString)! !

!CopyVisitorTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
setUp
	source := FileSystem memory.
	dest := FileSystem memory.
	
	! !

!CopyVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testAll
	self setUpGreek.
	CopyVisitor 
		copy: (source / 'alpha') 
		to: (dest / 'alpha').
	self assert: (dest isDirectory: '/alpha').
	self assert: (dest isFile: '/alpha/beta/gamma').! !

!FileSystemTreeTest class methodsFor: 'testing' stamp: 'EstebanLorenzano 4/3/2012 11:38'!
isAbstract
	^ self name = #FileSystemTreeTest! !

!FileSystemTreeTest methodsFor: 'running' stamp: 'lr 7/13/2010 15:26'!
createDirectory: aString
	self subclassResponsibility! !

!FileSystemTreeTest methodsFor: 'running' stamp: 'lr 7/13/2010 15:26'!
createFile: aString
	self subclassResponsibility! !

!FileSystemTreeTest methodsFor: 'running' stamp: 'cwp 10/30/2009 13:32'!
setUpGreek
	self 
		createDirectory: '/alpha';
		createDirectory: '/alpha/beta';
		createFile: '/alpha/beta/gamma';
		createFile: '/alpha/beta/delta';
		createDirectory: '/alpha/epsilon';
		createFile: '/alpha/epsilon/zeta'! !

FileSystemTreeTest subclass: #SingleTreeTest
	instanceVariableNames: 'filesystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

SingleTreeTest subclass: #AbstractEnumerationVisitorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!AbstractEnumerationVisitorTest class methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 4/3/2012 09:42'!
isAbstract
	^ self name = #AbstractEnumerationVisitorTest! !

!AbstractEnumerationVisitorTest methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
assertEntries: references are: expected
	| strings |
	self assert: references isArray.
	references do: [ :ea | self assert: ea class = FileSystemDirectoryEntry ].
	strings := references collect: [ :ea | ea reference pathString ].
	self assert: strings = expected! !

!AbstractEnumerationVisitorTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 15:45'!
root
	^ filesystem / 'alpha'! !

!AbstractEnumerationVisitorTest methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/9/2011 16:01'!
setUp
	super setUp.
	self setUpGreek.! !

AbstractEnumerationVisitorTest subclass: #CollectVisitorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!CollectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testBreadthFirst
	| entries |
	entries := CollectVisitor breadthFirst: self root.
	self 
		assertEntries: entries
		are: #(
				'/alpha'
				'/alpha/beta'
				'/alpha/epsilon'
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/epsilon/zeta'
			)! !

!CollectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testPostorder
	| entries |
	entries := CollectVisitor postorder: self root.
	self 
		assertEntries: entries
		are: #(
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/beta'
				'/alpha/epsilon/zeta'
				'/alpha/epsilon'
				'/alpha'
			)! !

!CollectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testPreorder
	| entries |
	entries := CollectVisitor preorder: self root.
	self 
		assertEntries: entries
		are: #(
				'/alpha'
				'/alpha/beta'
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/epsilon'
				'/alpha/epsilon/zeta'
			)! !

AbstractEnumerationVisitorTest subclass: #SelectVisitorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!SelectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testBreadthFirst
	| entries |
	entries := SelectVisitor breadthFirst: self root.
	self 
		assertEntries: entries
		are: #(
				'/alpha'
				'/alpha/beta'
				'/alpha/epsilon'
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/epsilon/zeta'
			)! !

!SelectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testBreadthFirstSelect
	| entries |
	entries := SelectVisitor breadthFirst: self root select: [ :node| node name endsWith: #a].
	self 
		assertEntries: entries
		are: #(
				'/alpha'
				'/alpha/beta'
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/epsilon/zeta'
			)! !

!SelectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testPostorder
	| entries |
	entries := SelectVisitor postorder: self root.
	self 
		assertEntries: entries
		are: #(
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/beta'
				'/alpha/epsilon/zeta'
				'/alpha/epsilon'
				'/alpha'
			)! !

!SelectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testPostorderSelect
	| entries |
	entries := SelectVisitor postorder: self root select: [ :node| node name endsWith: #a].
	self 
		assertEntries: entries
		are: #(
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/beta'
				'/alpha/epsilon/zeta'
				'/alpha'
			)! !

!SelectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testPreorder
	| entries |
	entries := SelectVisitor preorder: self root.
	self 
		assertEntries: entries
		are: #(
				'/alpha'
				'/alpha/beta'
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/epsilon'
				'/alpha/epsilon/zeta'
			)! !

!SelectVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testPreorderSelect
	| entries |
	entries := SelectVisitor preorder: self root select: [ :node| node name endsWith: #a].
	self 
		assertEntries: entries
		are: #(
				'/alpha'
				'/alpha/beta'
				'/alpha/beta/delta'
				'/alpha/beta/gamma'
				'/alpha/epsilon/zeta'
			)! !

SingleTreeTest subclass: #DeleteVisitorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!DeleteVisitorTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
testBeta
	self setUpGreek.
	DeleteVisitor delete: (filesystem / 'alpha' / 'beta').
	self assert: (filesystem isDirectory: '/alpha').
	self assert: (filesystem isDirectory: '/alpha/epsilon').
	self deny: (filesystem exists: '/alpha/beta').
	! !

SingleTreeTest subclass: #GuideTest
	instanceVariableNames: 'guide visited'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

GuideTest subclass: #BreadthFirstGuideTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!BreadthFirstGuideTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:37'!
testAll
	guide := BreadthFirstGuide for: self.
	guide show: (filesystem / 'alpha').
	self assertVisitedIs: #(
			'alpha'
			'beta'
			'epsilon'
			'delta'
			'gamma'
			'zeta'
		)! !

!GuideTest class methodsFor: 'testing' stamp: 'EstebanLorenzano 4/3/2012 09:43'!
isAbstract
	^ self name = #GuideTest! !

!GuideTest methodsFor: 'asserting' stamp: 'cwp 11/16/2009 10:46'!
assertVisitedIs: anArray
	visited with: anArray do:
		[:entry :basename | 
		self assert: entry reference basename = basename]! !

!GuideTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
setUp
	visited := OrderedCollection new.
	filesystem := FileSystem memory.
	self setUpGreek! !

!GuideTest methodsFor: 'visitor' stamp: 'cwp 10/29/2009 21:54'!
visitDirectory: aReference
	visited add: aReference.! !

!GuideTest methodsFor: 'visitor' stamp: 'cwp 10/29/2009 21:54'!
visitFile: aReference
	visited add: aReference.! !

GuideTest subclass: #PostorderGuideTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!PostorderGuideTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
testAll
	guide := PostorderGuide for: self.
	guide show: (filesystem / 'alpha').
	self assertVisitedIs: #(
			'delta'
			'gamma'
			'beta'
			'zeta'
			'epsilon'
			'alpha'
		)! !

GuideTest subclass: #PreorderGuideTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!PreorderGuideTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
testAll
	guide := PreorderGuide for: self.
	guide show: (filesystem / 'alpha').
	self assertVisitedIs: #(
			'alpha'
			'beta'
			'delta'
			'gamma'
			'epsilon'
			'zeta'
		)! !

!SingleTreeTest class methodsFor: 'testing' stamp: 'EstebanLorenzano 4/3/2012 09:42'!
isAbstract
	^ self name = #SingleTreeTest! !

!SingleTreeTest methodsFor: 'running' stamp: 'cwp 11/21/2009 11:30'!
createDirectory: aString 
	filesystem createDirectory: (filesystem pathFromString: aString)! !

!SingleTreeTest methodsFor: 'running' stamp: 'cwp 2/18/2011 16:17'!
createFile: aString 
	filesystem store createFile: (filesystem pathFromString: aString)! !

!SingleTreeTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
setUp
	super setUp.
	filesystem := FileSystem memory.
	! !

TestCase subclass: #FileSystemWriteStreamTest
	instanceVariableNames: 'filesystem stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-AnsiStreams'!

!FileSystemWriteStreamTest methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
contents
	^ (filesystem instVarNamed: 'store')
		nodeAt: Path / 'giffle'
		ifPresent: [:bytes | bytes]
		ifAbsent: [self signalFailure: 'No file!!']! !

!FileSystemWriteStreamTest methodsFor: 'running' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
setUp
	filesystem := FileSystem memory.
	stream := filesystem writeStreamOn: 'giffle'! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'cwp 7/28/2009 22:48'!
testClose
	self
		shouldnt: [stream close]
		raise: Error! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 21:55'!
testContents
	stream nextPutAll: #(42 43 44).
	self assert: stream contents = #(42 43 44) asByteArray! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 14:34'!
testCr
	stream cr.
	self assert: self contents first = Character cr asInteger! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 14:42'!
testCrLf
	stream crlf.
	self assert: self contents first = Character cr asInteger.
	self assert: self contents second = Character lf asInteger.! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 14:41'!
testCrTab
	stream crtab.
	self assert: self contents first = Character cr asInteger.
	self assert: self contents second = Character tab asInteger.! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'cwp 7/28/2009 22:21'!
testFlush
	
	self
		shouldnt: [stream flush]
		raise: Error! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 14:34'!
testLf
	stream lf.
	self assert: self contents first = Character lf asInteger! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 21:55'!
testNextPut
	stream nextPut: 42.
	self assert: self contents first = 42! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 22:23'!
testNextPutAll
	stream nextPutAll: #(42 43 44).
	stream flush.
	self assert: self contents = #(42 43 44) asByteArray! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 15:22'!
testOverriding
	stream 
		nextPutAll: 'foo bar';
		close.
	self assert: (filesystem root resolve: 'giffle') 
		readStream contents asString = 'foo bar'.
	
	(filesystem root resolve: 'giffle') writeStreamDo: [ :s | s nextPutAll: 'x' ].
	self assert: (filesystem / 'giffle') 
		readStream contents asString = 'x'! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 14:38'!
testPipeIn
	stream << #(42 43 44).
	stream flush.
	self assert: self contents = #(42 43 44) asByteArray! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 14:34'!
testSpace
	stream space.
	self assert: self contents first = Character space asInteger! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 14:40'!
testSpaceRepeat
	stream space: 3.
	self assert: self contents first = Character space asInteger.
	self assert: self contents second = Character space asInteger.
	self assert: self contents third = Character space asInteger.! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'cwp 7/29/2009 21:56'!
testTab
	stream tab.
	self assert: self contents first = 9! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'CamilloBruni 8/12/2011 14:40'!
testTabRepeat
	stream tab: 3.
	self assert: self contents first = Character tab asInteger.
	self assert: self contents second = Character tab asInteger.
	self assert: self contents third = Character tab asInteger.! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'cwp 10/15/2009 22:04'!
testTruncate
	stream 
		nextPutAll: #(42 43 44 45 46);
		flush;
		position: 4;
		truncate.
	self assert: self contents = #(42 43 44) asByteArray! !

!FileSystemWriteStreamTest methodsFor: 'tests' stamp: 'cwp 10/15/2009 21:50'!
testTruncateTo
	stream nextPutAll: #(42 43 44 45 46).
	stream flush.
	stream truncateTo: 3.
	self assert: self contents = #(42 43 44) asByteArray! !

TestCase variableSubclass: #PathTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Tests-Core'!

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 11:15'!
testAbsolutePrintString
	| path actual |
	path := Path / 'plonk' / 'griffle'.
	actual := path printString.
	self assert: actual = 'Path / ''plonk'' / ''griffle'''! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testAbsoluteWithParents
	| path allPaths |
	path := Path / 'plonk' / 'griffle' / 'nurb'.
	allPaths := path withParents.
	
	self assert: allPaths size = 4.
	self assert: allPaths first isRoot.
	self assert: allPaths second basename = 'plonk'.
	self assert: allPaths second size = 1.
	self assert: (allPaths second isChildOf: allPaths first).
	self assert: allPaths third basename = 'griffle'.
	self assert: allPaths third size = 2.
	self assert: (allPaths third isChildOf: allPaths second).
	self assert: allPaths fourth basename = 'nurb'.
	self assert: allPaths fourth size = 3.
	self assert: (allPaths fourth isChildOf: allPaths third).
	
	self assert: allPaths fourth = path.
	self assert: allPaths fourth == path! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
testAsReference
	| path reference |
	path := Path * 'plonk'.
	reference := path asFileReference.
	self assert: reference class = FileReference.
	self assert: reference path = path! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testBasename
	| path |
	path := Path * 'plonk' / 'griffle'.
	self assert: path basename = 'griffle'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testCommaAddsExtension
	| path result |
	path := Path * 'plonk' .
	result := path, 'griffle'.
	self assert: result basename = 'plonk.griffle'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testCommaAddsExtensionAgain
	| path result |
	path := Path * 'plonk.griffle'.
	result := path, 'nurp'.
	self assert: result basename = 'plonk.griffle.nurp'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testContains
	| ancestor descendent |
	ancestor := Path / 'plonk'.
	descendent := Path / 'plonk' / 'griffle' / 'bork'.
	self assert: (ancestor contains: descendent).
	self deny: (descendent contains: ancestor)! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/12/2012 14:26'!
testContainsLocator
	| ancestor descendent |
	ancestor := FileLocator imageDirectory resolve path.
	descendent := FileLocator image / 'griffle'.
	self deny: (ancestor contains: descendent).
	self deny: (descendent contains: ancestor)! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testEqual
	| a b |
	a := Path * 'plonk'.
	b := Path * 'plonk'.
	self deny: a == b.
	self assert: a = b.! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testGrandchildOfPath
	| griffle  nurb |
	griffle := Path / 'griffle'.
	nurb := griffle / 'plonk' / 'nurb'.
	self deny: (griffle isChildOf: nurb).
	self deny: (nurb isChildOf: griffle).! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testIsAbsolute
	self assert: (Path / 'plonk') isAbsolute! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testIsChildOfPath
	| parent child |
	parent := Path / 'griffle'.
	child := parent / 'nurb'.
	self assert: (child isChildOf: parent).
	self deny: (parent isChildOf: child)! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
testIsChildOfReference
	| parent child |
	parent := Path / 'griffle'.
	child := (FileSystem memory referenceTo: parent / 'nurb').
	self deny: (child isChildOf: parent).
	self deny: (parent isChildOf: child)! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testIsEmpty
	self assert: (Path workingDirectory) isEmpty! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testIsNotAbsolute
	self deny: (Path * 'plonk') isAbsolute! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testIsNotRelative
	self deny: (Path / 'plonk') isRelative! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testIsNotRoot
	self deny: (Path / 'plonk') isRoot! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testIsRelative
	self assert: (Path * 'plonk') isRelative! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testIsRoot
	self assert: Path root isRoot! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testMakeRelative
	"self run: #testMakeRelative"
	
	| parent child relative |
	parent := Path / 'griffle' / 'bibb'.
	child := Path / 'griffle' / 'plonk' / 'nurp'.
	relative := parent makeRelative: child.
	self assert: relative = (Path parent / 'plonk' / 'nurp')! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testParent
	| path parent |
	path := (Path * 'plonk') / 'griffle'.
	parent := path parent.
	self assert: parent isRelative.
	self assert: (parent at: 1) = 'plonk'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testParentParent
	| path  |
	path := (Path * '..') parent.
	self assert: path size = 2.
	self assert: (path at: 1) = '..'.
	self assert: (path at: 2) = '..'.! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testParentResolution
	| base relative absolute |
	base := Path / 'plonk' / 'pinto'.
	relative := Path parent / 'griffle' / 'zonk'.
	absolute := base resolve: relative.
	self assert: absolute isAbsolute.
	self assert: (absolute at: 1) = 'plonk'.
	self assert: (absolute at: 2) = 'griffle'.
	self assert: (absolute at: 3) = 'zonk'.
! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testParse
	"self run: #testParse"
	
	| path |
	path := Path from: 'parent/child/grandChild' delimiter: $/.
	self assert: path size = 3.
	self assert: (path at: 1) = 'parent'.
	self assert: (path at: 2) = 'child'.
	self assert: (path at: 3) = 'grandChild'.
	
! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testParseBogus
	"self run: #testParseBogus"
	
	| path |
	path := Path from: 'parent?<>~ \child/grandChild' delimiter: $/.
	self assert: path size = 2.
	self assert: (path at: 1) = 'parent?<>~ \child'.
	self assert: (path at: 2) = 'grandChild'.
	
! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testParseTrailingSlash
	| path |
	path := Path from: 'griffle/' delimiter: $/.
	self assert: path size = 1.
	self assert: (path at: 1) = 'griffle'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 5/4/2012 15:50'!
testParseWindowsPathWithUnixDelimiters
	"self run: #testParse"
	
	| path |
	path := WindowsStore new pathFromString: 'C:\a/b/c'.
	self assert: path segments size = 4.
	self assert: path segments = #('C:' 'a' 'b' 'c')
! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testPrintRelativeWithParent
	| path |
	path := Path parent / 'foo'.
	self assert: (path printWithDelimiter: $/) = '../foo'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testPrintWithDelimiter
	| path |
	path := (Path * 'plonk') / 'griffle'.
	self assert: (path printWithDelimiter: $%) = 'plonk%griffle'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 11:15'!
testRelativePrintString
	| path actual |
	path := Path * 'plonk' / 'griffle'.
	actual := path printString.
	self assert: actual = 'Path * ''plonk'' / ''griffle'''! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRelativeTo
	"self run: #testRelativeTo"
	"aPath relativeTo: aParent returns a new path relative to the parent"
	
	| parent child relative |
	parent := Path / 'griffle'.
	child := Path / 'griffle' / 'plonk' / 'nurp'.
	relative := child relativeTo: parent.
	self assert: relative = (Path * 'plonk' / 'nurp')! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRelativeToBranch
	| parent child relative |
	parent := Path / 'griffle' / 'bibb'.
	child := Path / 'griffle' / 'plonk' / 'nurp'.
	relative := child relativeTo: parent.
	self assert: relative = (Path parent / 'plonk' / 'nurp')! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRelativeWithParents
	| path allPaths |
	path := Path * 'plonk' / 'griffle' / 'nurb'.
	allPaths := path withParents.
	
	self assert: allPaths size = 3.
	self assert: allPaths first basename = 'plonk'.
	self assert: allPaths first size = 1.
	self assert: allPaths second basename = 'griffle'.
	self assert: allPaths second size = 2.
	self assert: (allPaths second isChildOf: allPaths first).
	self assert: allPaths third basename = 'nurb'.
	self assert: allPaths third size = 3.
	self assert: (allPaths third isChildOf: allPaths second).
	self assert: allPaths third == path! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testResolveAbsolute
	| path |
	path := Path / 'griffle'.
	self assert: path resolve == path.
	self assert: path isAbsolute! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testResolvePath
	"self debug: #testResolvePath"
	
	| path |
	path := Path / 'grandfather' / 'father' / 'child'.
	self assert: (path resolvePath: Path / 'grandfather') = (Path / 'grandfather').
	self assert: (path resolvePath: Path / 'child') = (Path / 'child').
	self assert: (path resolvePath: Path * 'grandfather') = (Path / 'grandfather' / 'father' / 'child' / 'grandfather').
	self assert: (path resolvePath: Path * 'child') = (Path / 'grandfather' / 'father' / 'child' / 'child').
	self assert: (path resolvePath: Path * 'grandfather') = (Path / 'grandfather' / 'father' / 'child' / 'grandfather').
	self assert: (path resolvePath: Path * 'child') = (Path / 'grandfather' / 'father' / 'child' / 'child').
	self assert:  (path resolvePath: (Path parent) / '..') =  (Path / 'grandfather')

! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testResolveRelative
	| path |
	path := Path * 'griffle'.
	self assert: path resolve == path.
	self assert: path isRelative! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testResolveString
	"self debug: #testResolveString"

	| path result |
	path := Path * 'plonk'.
	result := path resolve: 'griffle'.
	self assert: result class = path class.
	self assert: result size = 2.
	self assert: (result at: 1) = 'plonk'.
	self assert: (result at: 2) = 'griffle'.! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testRootParent
	| root |
	root := Path root.
	self assert: root parent == root! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 11:16'!
testRootPrintString
	| path actual |
	path := Path root.
	actual := path printString.
	self assert: actual = 'Path root'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testSiblingOfPath
	| griffle  nurb |
	griffle := Path / 'griffle'.
	nurb := Path / 'nurb'.
	self deny: (griffle isChildOf: nurb).
	self deny: (nurb isChildOf: griffle).! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testSimpleResolution
	| base relative absolute |
	base := Path / 'plonk'.
	relative := (Path * 'griffle') / 'zonk'.
	absolute := base resolve: relative.
	self assert: absolute isAbsolute.
	self assert: (absolute at: 1) = 'plonk'.
	self assert: (absolute at: 2) = 'griffle'.
	self assert: (absolute at: 3) = 'zonk'.
! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testSlash
	| path actual |
	path := Path * 'plonk'.
	actual := path / 'griffle'.
	self assert: actual class = path class.
	self assert: (actual printWithDelimiter: $/)  = 'plonk/griffle'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testUnequalContent
	| a b |
	a := Path * 'plonk'.
	b := Path * 'griffle'.
	self deny: a = b.! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testUnequalSize
	| a b |
	a := Path * 'plonk'.
	b := (Path * 'plonk') / 'griffle'.
	self deny: a = b.! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testWithExtentionAddsExtension
	| path result |
	path := Path * 'plonk'.
	result := path withExtension: 'griffle'.
	self assert: result basename = 'plonk.griffle'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testWithExtentionReplacesExtension
	| path result |
	path := Path * 'plonk.griffle'.
	result := path withExtension: 'griffle'.
	self assert: result basename = 'plonk.griffle'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/3/2012 11:16'!
testWorkingDirPrintString
	| path actual |
	path := Path workingDirectory. 
	actual := path printString.
	self assert: actual = 'Path workingDirectory'! !

!PathTest methodsFor: 'tests' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
testWorkingDirectoryParent
	| path |
	path := Path workingDirectory parent.
	self assert: path size = 1.
	self assert: (path at: 1) = '..'! !

Object subclass: #FileLocator
	instanceVariableNames: 'origin path'
	classVariableNames: 'Resolver'
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileLocator commentStamp: '<historical>' prior: 0!
I am a late-bound reference. I refer to a file or directory in relation to a well-known location on the filesystem, called an origin. When asked to perform concrete operation, I look up the current location of my origin, and resolve my path against it. 

Usage
----------
Locator vmDirectory parent pathString
	> '/Applications'


Locator desktop.
Locator desktop basename.

Locator home basename.
Locator image.
Locator vmBinary asAbsolute pathString 
	> '/Applications/Squeak'
Locator vmBinary pathString 
	> '/Applications/Squeak'
		




Implementation
------------------------
origin 
	A symbolic name for base reference I use to resolve myself.

path
	A relative path that is resolved against my origin"
	
	!

!FileLocator class methodsFor: 'class initialization' stamp: 'cwp 10/26/2009 20:54'!
addResolver: aResolver
	Resolver addResolver: aResolver! !

!FileLocator class methodsFor: 'origins' stamp: 'lr 7/13/2010 13:29'!
changes
	^ self origin: #changes ! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/27/2009 10:24'!
desktop
	^ self origin: #desktop! !

!FileLocator class methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:07'!
documents
	^ self origin: #documents! !

!FileLocator class methodsFor: 'class initialization' stamp: 'cwp 10/27/2009 10:28'!
flushCaches
	Resolver flushCaches! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/27/2009 09:34'!
home
	^ self origin: #home! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/25/2009 09:54'!
image
	^ self origin: #image ! !

!FileLocator class methodsFor: 'origins' stamp: 'lr 7/13/2010 13:35'!
imageDirectory
	^ self origin: #imageDirectory ! !

!FileLocator class methodsFor: 'class initialization' stamp: 'cwp 11/20/2009 15:01'!
initialize
	Smalltalk addToStartUpList: self.
	self startUp: true! !

!FileLocator class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
origin: aSymbol 
	^ self origin: aSymbol path: Path workingDirectory! !

!FileLocator class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/24/2012 14:49'!
origin: aSymbol path: aPath
	^ self basicNew 
			initializeWithOrigin: aSymbol path: aPath! !

!FileLocator class methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:07'!
preferences
	^ self origin: #preferences! !

!FileLocator class methodsFor: 'class initialization' stamp: 'EstebanLorenzano 4/2/2012 11:39'!
startUp: resuming
	resuming
		ifFalse: [ ^ self ].
	Resolver := InteractiveResolver new.
	Resolver addResolver: SystemResolver new.
	Resolver addResolver: PlatformResolver forCurrentPlatform! !

!FileLocator class methodsFor: 'accessing' stamp: 'cwp 10/27/2009 11:25'!
supportedOrigins
	| origins current |
	origins := IdentitySet new.
	current := Resolver.
	[current notNil] whileTrue:
		[origins addAll: current supportedOrigins.
		current := current next].
	^ origins! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/26/2009 11:37'!
vmBinary
	^ self origin: #vmBinary! !

!FileLocator class methodsFor: 'origins' stamp: 'cwp 10/26/2009 13:49'!
vmDirectory
	^ self origin: #vmDirectory! !

!FileLocator methodsFor: 'navigating' stamp: 'cwp 10/25/2009 11:12'!
, extension
	^ self withPath: path, extension! !

!FileLocator methodsFor: 'navigating' stamp: 'cwp 10/25/2009 11:03'!
/ aString
	^ self withPath: (path / aString)! !

!FileLocator methodsFor: 'comparing' stamp: 'cwp 10/26/2009 10:28'!
= other
	^ self species = other species
		and: [origin = other origin
			and: [path = other path]]! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 11/15/2009 21:23'!
allChildren
	^ self resolve allChildren! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 11/16/2009 21:19'!
allEntries
	^ self resolve allEntries! !

!FileLocator methodsFor: 'converting' stamp: 'cwp 10/25/2009 10:30'!
asAbsolute
	^ self ! !

!FileLocator methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
asFileReference
	^ self resolve! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:09'!
asPathWith: anObject
	^ self resolve asPathWith: anObject! !

!FileLocator methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:30'!
asReference
	self deprecated: 'Use #asFileReference instead'  on: '12 April 2012' in:  'Pharo 1.4' .
	^ self asFileReference! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/25/2009 11:09'!
basename
	^ self resolve basename! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 11/15/2009 21:24'!
children
	^ self resolve children! !

!FileLocator methodsFor: 'comparing' stamp: 'cwp 10/25/2009 22:26'!
contains: anObject
	^ self resolve contains: anObject! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 11/15/2009 08:10'!
copyAllTo: aReference
	^ self resolve copyAllTo: aReference! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:02'!
copyTo: aReference
	^ self resolve copyTo: aReference resolve! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:03'!
delete
	^ self resolve delete! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 11/15/2009 08:11'!
deleteAll
	^ self resolve deleteAll! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:27'!
directories
	^ self resolve directories! !

!FileLocator methodsFor: 'error handling' stamp: 'CamilloBruni 5/24/2012 12:35'!
doesNotUnderstand: aMessage
	"Redirect message to the resolved version of this FileLocator.
	If FileReference won't understand the message send a normal DNU."

	| resolved |
	
	resolved := self resolve.
	(resolved respondsTo: aMessage selector)
		ifTrue: [ ^ resolved perform: aMessage selector withArguments: aMessage arguments ].
	
	^ super doesNotUnderstand: aMessage.! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:03'!
ensureDirectory
	^ self resolve ensureDirectory! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:26'!
entries
	^ self resolve entries! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:26'!
entry
	^ self resolve entry! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:03'!
exists
	^ self resolve exists! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:27'!
extension
	^ self resolve extension! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:04'!
fileStreamDo: aBlock
	^ self resolve fileStreamDo: aBlock! !

!FileLocator methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 12:54'!
fileSystem
	^ self resolve fileSystem! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:27'!
files
	^ self resolve files! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:28'!
filesMatching: patterns
	^ self resolve filesMatching: patterns! !

!FileLocator methodsFor: 'delegated' stamp: 'EstebanLorenzano 4/3/2012 12:55'!
filesystem
	self deprecated: 'Use fileSystem' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self fileSystem! !

!FileLocator methodsFor: 'delegated' stamp: 'sd 2/11/2011 20:36'!
fullName

	^ self resolve fullName! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:28'!
glob: aBlock
	^ self resolve glob: aBlock! !

!FileLocator methodsFor: 'comparing' stamp: 'cwp 10/25/2009 11:05'!
hash
	^ origin hash bitXor: path hash! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:28'!
ifFile: fBlock ifDirectory: dBlock ifAbsent: aBlock
	^ self resolve 
		ifFile: fBlock ifDirectory: dBlock ifAbsent: aBlock! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:28'!
indicator
	^ self resolve indicator! !

!FileLocator methodsFor: 'initialize-release' stamp: 'cwp 10/25/2009 09:56'!
initializeWithOrigin: aSymbol path: aPath
	self initialize.
	origin := aSymbol.
	path := aPath.! !

!FileLocator methodsFor: 'testing' stamp: 'cwp 10/25/2009 10:30'!
isAbsolute
	^ true! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:29'!
isChildOf: anObject
	^ self resolve isChildOf: anObject! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/25/2009 23:05'!
isContainedBy: anObject
	^ self resolve isContainedBy: anObject! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:04'!
isDirectory
	^ self resolve isDirectory! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:04'!
isFile
	^ self resolve isFile! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:29'!
isReadable
	^ self resolve isReadable! !

!FileLocator methodsFor: 'testing' stamp: 'cwp 10/25/2009 11:15'!
isRelative
	^ false! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/25/2009 11:22'!
isRoot
	^ self resolve isRoot! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:29'!
isWriteable
	^ self resolve isWriteable! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:29'!
macTypeAndCreator
	^ self resolve macTypeAndCreator! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:29'!
mimeTypes
	^ self resolve mimeTypes! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:30'!
moveTo: aReference
	^ self resolve moveTo: aReference! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:30'!
nextVersion
	^ self resolve nextVersion! !

!FileLocator methodsFor: 'accessing' stamp: 'cwp 10/25/2009 21:31'!
origin
	^ origin! !

!FileLocator methodsFor: 'delegated' stamp: 'CamilloBruni 5/24/2012 12:30'!
parent
	^ self withPath: self path parent! !

!FileLocator methodsFor: 'accessing' stamp: 'cwp 10/25/2009 21:31'!
path
	^ path! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:05'!
pathString
	^ self resolve pathString! !

!FileLocator methodsFor: 'printing' stamp: 'EstebanLorenzano 4/3/2012 12:55'!
printOn: aStream
	| fs |
	aStream
		nextPut: ${;
		nextPutAll: origin;
		nextPut: $}.
	path isWorkingDirectory
		ifTrue: [ ^ self ].
	fs := self fileSystem.
	aStream nextPut: fs delimiter.
	fs printPath: path on: aStream! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:05'!
readStream
	^ self resolve readStream! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:05'!
readStreamDo: aBlock
	^ self resolve readStreamDo: aBlock! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/29/2009 11:23'!
readStreamDo: doBlock ifAbsent: aBlock
	^ self resolve readStreamDo: doBlock ifAbsent: aBlock! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/29/2009 11:24'!
readStreamIfAbsent: aBlock
	^ self resolve readStream readStreamIfAbsent: aBlock! !

!FileLocator methodsFor: 'navigating' stamp: 'cwp 10/25/2009 09:59'!
resolve
	^ (Resolver resolve: origin) resolve: path! !

!FileLocator methodsFor: 'navigating' stamp: 'cwp 10/26/2009 01:03'!
resolve: anObject
	^ anObject asResolvedBy: self! !

!FileLocator methodsFor: 'navigating' stamp: 'cwp 10/25/2009 21:36'!
resolvePath: aPath
	^ self withPath: (path resolvePath: aPath)! !

!FileLocator methodsFor: 'navigating' stamp: 'cwp 10/25/2009 21:53'!
resolveReference: aReference
	^ aReference isAbsolute
		ifTrue: [aReference]
		ifFalse: [self withPath: aReference path]! !

!FileLocator methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/3/2012 12:55'!
resolveString: aString 
	| filesystem thePath |
	filesystem := (Resolver resolve: origin) fileSystem.
	thePath := filesystem pathFromString: aString.
	^ self withPath: (path resolvePath: thePath)! !

!FileLocator methodsFor: 'navigating' stamp: 'CamilloBruni 5/24/2012 11:19'!
withExtension: aString
	^ self withPath: (self path withExtension: aString)! !

!FileLocator methodsFor: 'navigating' stamp: 'cwp 10/25/2009 11:03'!
withPath: newPath
	^ path == newPath
		ifTrue: [self]
		ifFalse: [self class origin: origin path: newPath]! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:05'!
writeStream
	^ self resolve writeStream! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/26/2009 02:06'!
writeStreamDo: aBlock
	^ self resolve writeStreamDo: aBlock! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/29/2009 11:24'!
writeStreamDo: doBlock ifPresent: pBlock
	^ self resolve writeStreamDo: doBlock ifPresent: pBlock! !

!FileLocator methodsFor: 'delegated' stamp: 'cwp 10/29/2009 11:23'!
writeStreamIfPresent: aBlock
	^ self resolve writeStreamIfPresent: aBlock! !

Object subclass: #FilePluginPrims
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!FilePluginPrims commentStamp: 'cwp 11/18/2009 13:02' prior: 0!
I provide an interface to the primitives in the FilePlugin.
!

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:26'!
atEnd: id
	"Answer true if the file position is at the end of the file."

	<primitive: 'primitiveFileAtEnd' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:27'!
close: id
	"Close this file."

	<primitive: 'primitiveFileClose' module: 'FilePlugin'>
	! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 11/17/2009 16:26'!
createDirectory: fullPath
	"Create a directory named by the given path. 
	Fail if the path is bad or if a file or directory by that name already exists."

 	<primitive: 'primitiveDirectoryCreate' module: 'FilePlugin'>
	^ nil
! !

!FilePluginPrims methodsFor: 'encoding primitives' stamp: 'lr 7/13/2010 14:10'!
decode: aString
	^ aString convertFromWithConverter: LanguageEnvironment defaultFileNameConverter! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:21'!
deleteDirectory: fullPath
	"Delete the directory named by the given path. 
	Fail if the path is bad or if a directory by that name does not exist."

 	<primitive: 'primitiveDirectoryDelete' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:22'!
deleteFile: aFileName
	"Delete the file of the given name. 
	Return self if the primitive succeeds, nil otherwise."

	<primitive: 'primitiveFileDelete' module: 'FilePlugin'>
	^ nil
! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 10/11/2009 11:02'!
delimiter
	"Return the path delimiter for the underlying platform's file system."

 	<primitive: 'primitiveDirectoryDelimitor' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'encoding primitives' stamp: 'lr 7/13/2010 14:11'!
encode: aString
	^ aString convertToWithConverter: LanguageEnvironment defaultFileNameConverter! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'lr 3/21/2010 12:10'!
flush: id
	"Flush pending changes to the disk"
	
	| pos |
	<primitive: 'primitiveFileFlush' module: 'FilePlugin'>
	"In some OS's seeking to 0 and back will do a flush"
	pos := self getPosition: id.
	self setPosition: id to: 0; setPosition: id to: pos! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:23'!
getMacFile: fileName type: typeString creator: creatorString
	"Get the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4. This primitive is Mac specific; it is a noop on other platforms."

 	<primitive: 'primitiveDirectoryGetMacTypeAndCreator' module: 'FilePlugin'>

! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:27'!
getPosition: id
	"Get this files current position."

	<primitive: 'primitiveFileGetPosition' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 10/11/2009 11:02'!
imageFile
	"Answer the full path name for the current image."

	<primitive: 121>
	self primitiveFailed! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'CamilloBruni 1/17/2012 17:09'!
lookupDirectory: fullPath filename: fileName
 	<primitive: 'primitiveDirectoryEntry' module: 'FilePlugin'>
	^ #badDirectoryPath

! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:24'!
lookupEntryIn: fullPath index: index
	"Look up the index-th entry of the directory with the given fully-qualified path 
	(i.e., starting from the root of the file hierarchy) and return an array containing:

	<name> <creationTime> <modificationTime> <dirFlag> <fileSize>

	The empty string enumerates the top-level files or drives. (For example, on Unix, the empty 
	path enumerates the contents of '/'. On Macs and PCs, it enumerates the mounted volumes/drives.)

	The creation and modification times are in seconds since the start of the Smalltalk time epoch. 
	DirFlag is true if the entry is a directory. FileSize the file size in bytes or zero for directories. 
	The primitive returns nil when index is past the end of the directory. It fails if the given path 
	is bad."

 	<primitive: 'primitiveDirectoryLookup' module: 'FilePlugin'>
	^ #badDirectoryPath

! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:27'!
open: fileName writable: writableFlag
	"Open a file of the given name, and return the file ID obtained.
	If writableFlag is true, then
		if there is none with this name, then create one
		else prepare to overwrite the existing from the beginning
	otherwise
		if the file exists, open it read-only
		else return nil"

	<primitive: 'primitiveFileOpen' module: 'FilePlugin'>
	^ nil
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:28'!
read: id into: byteArray startingAt: startIndex count: count
	"Read up to count bytes of data from this file into the given string or byte array starting at the given index. Answer the number of bytes actually read."

	<primitive: 'primitiveFileRead' module: 'FilePlugin'>
	self primitiveFailed! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:24'!
rename: oldFileFullName to: newFileFullName 
	"Rename the file of the given name to the new name. Fail if there is no file of the old name 
	or if there is an existing file with the new name."

	<primitive: 'primitiveFileRename' module: 'FilePlugin'>
	^nil! !

!FilePluginPrims methodsFor: 'path primitives' stamp: 'cwp 7/20/2009 17:25'!
setMacFileNamed: fileName type: typeString creator: creatorString
	"Set the Macintosh file type and creator info for the file with the given name. Fails if the 
	file does not exist or if the type and creator type arguments are not strings of length 4. This
	primitive is Mac specific; it is a noop on other platforms."

 	<primitive: 'primitiveDirectorySetMacTypeAndCreator' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:30'!
setPosition: id to: anInteger
	"Set this file to the given position."

	<primitive: 'primitiveFileSetPosition' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:30'!
size: id
	"Answer the size of this file."

	<primitive: 'primitiveFileSize' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/22/2009 07:10'!
sizeOrNil: id
	"Answer the size of this file."

	<primitive: 'primitiveFileSize' module: 'FilePlugin'>
	^ nil! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:33'!
truncate: id to: anInteger
	"Truncate this file to the given position."

	<primitive: 'primitiveFileTruncate' module: 'FilePlugin'>
	self primitiveFailed
! !

!FilePluginPrims methodsFor: 'file primitives' stamp: 'cwp 7/20/2009 17:33'!
write: id from: stringOrByteArray startingAt: startIndex count: count
	"Write count bytes onto this file from the given string or byte array starting at the given index. 	Answer the number of bytes written."

	<primitive: 'primitiveFileWrite' module: 'FilePlugin'>
	self primitiveFailed! !

Object subclass: #FileReference
	instanceVariableNames: 'filesystem path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileReference commentStamp: '<historical>' prior: 0!
I combine a filesystem and path, which is sufficient to refer to a concrete file or directory. I provide methods for navigating my filesystem, performing filesystem operations and opening and closing files.  I am the primary mechanism for working with files and directories. 

| working |
working := FileSystem disk workingDirectory.
working files 

| disk |
disk := FileSystem disk.
disk root.                               	"a reference to the root directory"
disk working.                         	"a reference to the working directory"!

!FileReference class methodsFor: 'cross platform' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
/ aString
	"Answer a reference to the argument resolved against the root of the current disk filesystem."
	
	^ FileSystem disk / aString! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:13'!
A
	^ self driveNamed: #A 
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
B
	^ self driveNamed: #B
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
C
	^ self driveNamed: #C
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
D
	^ self driveNamed: #D
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
E
	^ self driveNamed: #E
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
F
	^ self driveNamed: #F
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
G
	^ self driveNamed: #G
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
H
	^ self driveNamed: #H
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
I
	^ self driveNamed: #I
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
J
	^ self driveNamed: #J
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
K
	^ self driveNamed: #K
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
L
	^ self driveNamed: #L
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
M
	^ self driveNamed: #M
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
N
	^ self driveNamed: #N
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:16'!
O
	^ self driveNamed: #O
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
P
	^ self driveNamed: #P
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
Q
	^ self driveNamed: #Q
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
R
	^ self driveNamed: #R
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
S
	^ self driveNamed: #S
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
T
	^ self driveNamed: #T
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
U
	^ self driveNamed: #U
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
V
	^ self driveNamed: #V
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
W
	^ self driveNamed: #W! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
X
	^ self driveNamed: #X
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
Y
	^ self driveNamed: #Y
! !

!FileReference class methodsFor: 'windows' stamp: 'CamilloBruni 8/12/2011 15:15'!
Z
	^ self driveNamed: #Z
! !

!FileReference class methodsFor: 'windows' stamp: 'EstebanLorenzano 4/3/2012 13:23'!
driveNamed: driveName
	^ self fileSystem: (FileSystem disk) path: Path / (driveName, ':')! !

!FileReference class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/3/2012 13:21'!
fileSystem: aFilesystem path: aPath
	^ self new setFileSystem: aFilesystem path: aPath! !

!FileReference class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/3/2012 13:22'!
filesystem: aFilesystem path: aPath
	self deprecated: 'Use #fileSystem:path:' on: '3 April 2012' in: 'Pharo 1.4'. 
	^ self fileSystem: aFilesystem path: aPath! !

!FileReference methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 12:45'!
, aString
	^ self withPath: self path, aString! !

!FileReference methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 12:45'!
/ anObject
	^ self withPath: self path / anObject! !

!FileReference methodsFor: 'comparing' stamp: 'MaxLeske 7/30/2010 17:28'!
<= other
	^ path asString <= other path asString! !

!FileReference methodsFor: 'comparing' stamp: 'EstebanLorenzano 4/3/2012 13:02'!
= other
	^ self species = other species
		and: [self path = other path
			and: [self fileSystem = other fileSystem]]! !

!FileReference methodsFor: 'enumerating' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
allChildren
	"Return all the files and folders recursively nested in the receiver"
	
	^ CollectVisitor breadthFirst: self collect: [:ea | ea reference]! !

!FileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 1/19/2012 13:46'!
allDirectories
	"Return all the directories recursively nested in the receiver."
	^ self allChildren reject: [:each | each isFile]! !

!FileReference methodsFor: 'enumerating' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
allEntries
	^ CollectVisitor breadthFirst: self! !

!FileReference methodsFor: 'enumerating' stamp: 'CamilloBruni 1/19/2012 13:46'!
allFiles
	"Return all the files (not directories) recursively nested in the receiver."
	^ self allChildren select: [:each | each isFile]! !

!FileReference methodsFor: 'converting' stamp: 'StephaneDucasse 2/3/2011 08:10'!
asAbsolute
	"Return the receiver as an absolute file reference."
	
	^ self isAbsolute
		ifTrue: [self]
		ifFalse: [filesystem referenceTo: (filesystem resolve: path)]! !

!FileReference methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
asFileReference
	^ self! !

!FileReference methodsFor: 'converting' stamp: 'CamilloBruni 1/19/2012 01:13'!
asPathWith: anObject
	^ self path! !

!FileReference methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:30'!
asReference
	self deprecated: 'Use #asFileReference instead'  on: '12 April 2012' in:  'Pharo 1.4' .
	^ self asFileReference! !

!FileReference methodsFor: 'resolving' stamp: 'cwp 9/22/2009 09:03'!
asResolvedBy: anObject
	^ anObject resolveReference: self! !

!FileReference methodsFor: 'converting' stamp: 'CamilloBruni 5/4/2012 21:54'!
asUrl
	"Convert my path into a file:// type url - a FileUrl."
	
	^ self path asUrl! !

!FileReference methodsFor: 'accessing' stamp: 'StephaneDucasse 2/15/2010 17:57'!
base
	"Returns the base of the basename, i.e. /foo/gloops.taz base is 'gloops'"
	^ path base! !

!FileReference methodsFor: 'accessing' stamp: 'StephaneDucasse 2/2/2011 22:48'!
basename
	"Returns the basename, i.e. /foo/gloops.taz basename is 'gloops.taz'"
	^ path basename! !

!FileReference methodsFor: 'accessing' stamp: 'DamienCassou 4/17/2011 14:36'!
basenameWithIndicator
	"Returns the basename with the indicator appended, i.e. /foo/gloops.taz basenameWithIndicator is 'gloops.taz', whereras /foo basenameWithIndicator is 'foo/'" 
	^ self basename, self indicator   ! !

!FileReference methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
children
	"Answer an array containing references to the direct children
	of this reference."

	^ (self fileSystem childrenAt: self path) collect: [:ea |
		FileReference
			fileSystem: self fileSystem
			path: ea]! !

!FileReference methodsFor: 'comparing' stamp: 'cwp 10/26/2009 01:03'!
contains: anObject
	^ anObject isContainedBy: self! !

!FileReference methodsFor: 'comparing' stamp: 'cwp 10/26/2009 00:54'!
containsPath: aPath
	^ self path containsPath: aPath! !

!FileReference methodsFor: 'comparing' stamp: 'EstebanLorenzano 4/3/2012 13:05'!
containsReference: aReference
	^  aReference fileSystem = filesystem and: [path contains: aReference path]! !

!FileReference methodsFor: 'operations' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
copyAllTo: aResolvable
	
	CopyVisitor copy: self asAbsolute to: aResolvable resolve! !

!FileReference methodsFor: 'operations' stamp: 'EstebanLorenzano 4/3/2012 13:05'!
copyTo: aReference
	self isDirectory
		ifTrue: [ aReference ensureDirectory ]
		ifFalse: [ 
			filesystem = aReference fileSystem
				ifTrue: [ filesystem copy: path to: aReference path ]
				ifFalse: [ filesystem copy: path toReference: aReference ] ]! !

!FileReference methodsFor: 'operations' stamp: 'cwp 11/17/2009 21:05'!
createDirectory
	filesystem createDirectory: path! !

!FileReference methodsFor: 'operations' stamp: 'cwp 7/22/2009 07:42'!
delete
	filesystem delete: path! !

!FileReference methodsFor: 'operations' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
deleteAll
	"delete this directory and all children of it"
	DeleteVisitor delete: self! !

!FileReference methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
directories
	"Return all the directories (by opposition to files) contained in the receiver"

	^ (self fileSystem directoriesAt: self path) collect: [:ea |
		FileReference
			fileSystem: self fileSystem
			path: ea]! !

!FileReference methodsFor: 'operations' stamp: 'sd 2/11/2011 20:16'!
ensureDirectory
	"Create if necessary a directory for the receiver."
	filesystem ensureDirectory: path
	! !

!FileReference methodsFor: 'operations' stamp: 'CamilloBruni 8/12/2011 16:51'!
ensureFile
	"Create if necessary a file for the receiver."
	self writeStream close.
	! !

!FileReference methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/3/2012 13:07'!
entries
	"Return the entries (meta data - file description) of the direct children of the receiver"
	^ self fileSystem entriesAt: self path ! !

!FileReference methodsFor: 'accessing' stamp: 'sd 2/11/2011 19:58'!
entry
	"Return the entry (meta data) describing the receiver."
	^ filesystem entryAt: path! !

!FileReference methodsFor: 'testing' stamp: 'cwp 1/13/2009 20:52'!
exists
	^ filesystem exists: path! !

!FileReference methodsFor: 'accessing' stamp: 'cwp 3/29/2011 16:56'!
extension
	"Returns the extension of the basename, i.e. /foo/gloops.taz extension is 'taz'. 
	Note that compound extensions are returned completely: 
	/foo/gloops.taz.txt extension is 'taz.txt'"
	
	^ path extension! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 5/8/2012 17:50'!
fileNames
	^ self files collect: #basename! !

!FileReference methodsFor: 'streams' stamp: 'cwp 3/11/2011 23:26'!
fileStreamDo: aBlock
	^ self fileStreamWritable: true do: aBlock! !

!FileReference methodsFor: 'streams' stamp: 'cwp 3/11/2011 23:27'!
fileStreamWritable: aBoolean do: aBlock
	| stream |
	stream := filesystem openFileStream: path writable: aBoolean.
	^ [ aBlock value: stream ] ensure: [ stream close ]
	! !

!FileReference methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 13:03'!
fileSystem
	"Return the filesystem to which the receiver belong."
	^ filesystem! !

!FileReference methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
files
	"Return all the files (as opposed to folders) contained in the receiver"
	
	^ (self fileSystem filesAt: self path) collect: [:ea |
		FileReference
			fileSystem: self fileSystem
			path: ea]! !

!FileReference methodsFor: 'utility' stamp: 'CamilloBruni 5/9/2012 11:51'!
filesMatching: patterns
	"
	FileDirectory default fileNamesMatching: '*'
	FileDirectory default fileNamesMatching: '*.image;*.changes'
	"
	
	| files |
	files := Set new.
	
	(patterns findTokens: ';', String crlf) do: [ :pattern | 
		files addAll: (self files select: [:file | pattern match: file basename])].
	
	^files asOrderedCollection
! !

!FileReference methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 13:03'!
filesystem
	"Return the filesystem to which the receiver belong."
	self deprecated: 'Use fileSystem' on: '3 April 2012' in: 'Pharo 1.4'.	
	^ self fileSystem! !

!FileReference methodsFor: 'accessing' stamp: 'sd 2/11/2011 20:34'!
fullName
	"Return the full path name of the receiver."
	^ filesystem stringFromPath: (filesystem resolve: path)! !

!FileReference methodsFor: 'accessing' stamp: 'DamienCassou 4/17/2011 14:36'!
fullNameWithIndicator
	"Returns the basename with the indicator appended, i.e. /foo/gloops.taz basenameWithIndicator is '/foo/gloops.taz', whereras /foo basenameWithIndicator is '/foo/'" 
	^ self fullName, self indicator   ! !

!FileReference methodsFor: 'enumerating' stamp: 'EstebanLorenzano 4/2/2012 11:40'!
glob: aBlock
	^ SelectVisitor breadthFirst: self select: aBlock! !

!FileReference methodsFor: 'comparing' stamp: 'cwp 9/16/2009 23:54'!
hash
	^ path hash bitXor: filesystem hash! !

!FileReference methodsFor: 'testing' stamp: 'lr 7/13/2010 15:36'!
ifFile: fBlock ifDirectory: dBlock ifAbsent: aBlock
	^ self isFile
		ifTrue: [ fBlock value ]
		ifFalse: [
			self isDirectory
				ifTrue: [ dBlock value ]
				ifFalse: [ aBlock value ] ]! !

!FileReference methodsFor: 'printing' stamp: 'DamienCassou 4/17/2011 14:36'!
indicator
	"Returns a string indicating the type of reference:
	- '?' for a non existing reference',
	- '/' for a directory,
	- the empty string for a file."
	"When this framework gets more complete, it is possible to extend this behavior with the following indicators (taken from unix ls utility):
	- '*' for a regular file that is executable
	- '@' for a symbolic link
	- '|' for FIFOs
	- '=' for sockets
	- '>' for doors"
	^ self exists
		ifTrue: [self isDirectory ifTrue: ['/'] ifFalse: ['']  ]
		ifFalse: ['?']! !

!FileReference methodsFor: 'testing' stamp: 'cwp 7/20/2009 09:24'!
isAbsolute
	^ path isAbsolute! !

!FileReference methodsFor: 'testing' stamp: 'cwp 11/16/2009 09:06'!
isChildOf: anObject
	^ self parent = anObject! !

!FileReference methodsFor: 'comparing' stamp: 'cwp 10/25/2009 23:03'!
isContainedBy: anObject
	^ anObject containsReference: self! !

!FileReference methodsFor: 'testing' stamp: 'cwp 1/13/2009 21:39'!
isDirectory
	^ filesystem isDirectory: path! !

!FileReference methodsFor: 'testing' stamp: 'cwp 1/13/2009 21:57'!
isFile
	^ filesystem isFile: path! !

!FileReference methodsFor: 'testing' stamp: 'CamilloBruni 5/9/2012 00:37'!
isReadable
	^ filesystem isReadable: path! !

!FileReference methodsFor: 'testing' stamp: 'cwp 7/20/2009 09:25'!
isRelative
	^ path isRelative! !

!FileReference methodsFor: 'testing' stamp: 'cwp 7/20/2009 09:26'!
isRoot
	^ path isRoot! !

!FileReference methodsFor: 'testing' stamp: 'CamilloBruni 5/9/2012 00:37'!
isWritable
	^ filesystem isWritable: path! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 11:37'!
macTypeAndCreator
	| results typeString creatorString |
	"get the Macintosh file type and creator info for the file with the given name. Fails if the file does not exist or if the type and creator type arguments are not strings of length 4. Does nothing on other platforms (where the underlying primitive is a noop)."
	"FileDirectory default getMacFileNamed: 'foo'"

	self flag: 'TODO'.
	"
	typeString := ByteArray new: 4 withAll: ($? asInteger). #[63 63 63 63]
	creatorString := ByteArray new: 4 withAll: ($? asInteger).
	[self primGetMacFileNamed: (self fullNameFor: fileName) asVmPathName
		type: typeString
		creator: creatorString.] ensure: 
		[typeString := typeString asString. 
		creatorString := creatorString asString].
	results := Array with: typeString convertFromSystemString with: creatorString convertFromSystemString.
	^results"
! !

!FileReference methodsFor: 'navigating' stamp: 'cwp 11/15/2009 00:26'!
makeRelative: anObject
	^ anObject relativeToReference: self! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 5/9/2012 13:04'!
mimeTypes
	"Return the possible mime types for the given path."
	^ filesystem mimeTypesAt: path! !

!FileReference methodsFor: 'operations' stamp: 'EstebanLorenzano 4/3/2012 13:08'!
moveTo: aReference
	
	| result |
	result := self fileSystem rename: self path to: aReference path.
	result ifNotNil: [
		self setFileSystem: filesystem path: aReference path ].		! !

!FileReference methodsFor: 'utility' stamp: 'CamilloBruni 5/7/2012 03:06'!
nextVersion
	"Assumes a file name includes a version number encoded as '.' followed by digits 
	preceding the file extension.  Increment the version number and answer the new file name.
	If a version number is not found, set the version to 1 and answer a new file name"

	| parent version versionNumbers nameWithoutExtension |
	
	self exists
		ifFalse: [ ^ self ].
	
	parent := self parent.
	nameWithoutExtension := self basename copyUpTo: $..
	
	versionNumbers := parent files 
				select: [ :f| 
					(f basename beginsWith: nameWithoutExtension) ]
				thenCollect: [ :f| 
					Number squeezeNumberOutOfString: f basename ifFail: [ 0 ]].
				
	version := versionNumbers max + 1.
	^ parent / (nameWithoutExtension , '.', version asString) , self extension! !

!FileReference methodsFor: 'private' stamp: 'CamilloBruni 1/20/2012 13:24'!
openWritable: aBoolean 
	^ filesystem open: path writable: aBoolean! !

!FileReference methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 12:45'!
parent
	^ self withPath: self path parent! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 1/19/2012 14:16'!
path
	"Return the path internal representation that denotes the receiver in the context of its filesystem. "
	^ path! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 8/12/2011 17:07'!
pathSegments
	^ path segments! !

!FileReference methodsFor: 'printing' stamp: 'sd 2/11/2011 20:34'!
pathString
	"Return the full path name of the receiver."
	
	^ filesystem stringFromPath: (filesystem resolve: path)! !

!FileReference methodsFor: 'printing' stamp: 'cwp 10/11/2009 22:32'!
printOn: aStream
	filesystem forReferencePrintOn: aStream.
	filesystem printPath: path on: aStream! !

!FileReference methodsFor: 'streams' stamp: 'abc 5/11/2012 23:21'!
readStream
	^ filesystem readStreamOn: path! !

!FileReference methodsFor: 'streams' stamp: 'lr 7/19/2010 19:32'!
readStreamDo: aBlock
	| stream |
	stream := self readStream.
	^ [ aBlock value: stream ] ensure: [ stream close ]! !

!FileReference methodsFor: 'streams' stamp: 'lr 7/19/2010 19:35'!
readStreamDo: doBlock ifAbsent: absentBlock
	^ self isFile 
		ifTrue: [ self readStreamDo: doBlock ]
		ifFalse: [ absentBlock value ]! !

!FileReference methodsFor: 'streams' stamp: 'lr 7/19/2010 19:35'!
readStreamIfAbsent: absentBlock
	^ self isFile
		ifTrue: [ self readStream ]
		ifFalse: [ absentBlock value ]! !

!FileReference methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 14:02'!
relativeTo: landmark
	"Answer a new path relative to landmark."
	
	"parent/child/grandChild relativeTo: parent returns child/grandChild
	(FSFilesystem disk / 'griffle' / 'plonk' / 'nurp') relativeTo: (FSFilesystem disk / 'griffle') 
	returns plonk/nurp"

	^ landmark makeRelative: self! !

!FileReference methodsFor: 'navigating' stamp: 'DamienPollet 3/1/2011 19:05'!
relativeToPath: landmarkPath 
	
	^ path relativeTo: landmarkPath! !

!FileReference methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 01:12'!
relativeToReference: landmarkReference
	"Return the path of the receiver relative to landmarkReference."
	
	^ self path relativeTo: landmarkReference path! !

!FileReference methodsFor: 'operations' stamp: 'EstebanLorenzano 4/3/2012 13:08'!
renameTo: aStringOrPath
	
	| destinationPath |
	
	destinationPath := self fileSystem rename: self to: aStringOrPath.
	
	destinationPath ifNotNil: [
		self 
			setFileSystem: filesystem 
			path: destinationPath ].
	^ self
		! !

!FileReference methodsFor: 'resolving' stamp: 'cwp 10/26/2009 02:02'!
resolve
	^ self! !

!FileReference methodsFor: 'resolving' stamp: 'cwp 10/26/2009 01:03'!
resolve: anObject
	^ anObject asResolvedBy: self! !

!FileReference methodsFor: 'resolving' stamp: 'CamilloBruni 1/19/2012 12:45'!
resolvePath: anObject
	^ self withPath: (path resolve: anObject)! !

!FileReference methodsFor: 'resolving' stamp: 'EstebanLorenzano 4/3/2012 13:09'!
resolveReference: aReference
	
	^ (filesystem = aReference fileSystem or: [aReference isRelative])
		ifTrue: [filesystem referenceTo: (path resolvePath: aReference path)]
		ifFalse: [aReference]! !

!FileReference methodsFor: 'resolving' stamp: 'cwp 11/21/2009 11:30'!
resolveString: aString 
	| thePath |
	thePath := filesystem pathFromString: aString.
	^ filesystem referenceTo: (path resolve: thePath)! !

!FileReference methodsFor: 'initialize-release' stamp: 'EstebanLorenzano 4/3/2012 13:07'!
setFileSystem: aFilesystem path: aPath
	filesystem := aFilesystem.
	path := aPath! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 02:01'!
setMacType: aTypeString creator: aCreatorString
	self flag: 'TODO'.
	
" 	self primSetMacFileNamed: (self fullNameFor: fileName) asVmPathName
		type: typeString convertToSystemString
		creator: creatorString convertToSystemString.
"! !

!FileReference methodsFor: 'accessing' stamp: 'CamilloBruni 5/4/2012 19:53'!
uri
	"Convert my path into a file:// type url. For odd characters use %20 notation."

	self flag: 'TODO: return a real URI object instead of a string'.
	^(String streamContents: [:strm |
		strm nextPutAll: 'file:'.
		self pathSegments do: [:each | strm nextPut: $/; nextPutAll: each encodeForHTTP].
		strm nextPut: $/]) asURI! !

!FileReference methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 12:46'!
withExtension: aString 
	^ self withPath: (self path withExtension: aString)! !

!FileReference methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 12:45'!
withPath: newPath
	^ self path == newPath
		ifTrue: [self]
		ifFalse: [filesystem referenceTo: newPath]! !

!FileReference methodsFor: 'streams' stamp: 'abc 5/11/2012 23:20'!
writeStream
	^ filesystem writeStreamOn: path! !

!FileReference methodsFor: 'streams' stamp: 'lr 7/19/2010 19:34'!
writeStreamDo: aBlock
	| stream |
	stream := self writeStream.
	^ [ aBlock value: stream ]
		ensure: [ stream close ]! !

!FileReference methodsFor: 'streams' stamp: 'lr 7/19/2010 19:33'!
writeStreamDo: doBlock ifPresent: presentBlock
	^ self isFile
		ifTrue: [ presentBlock value ]
		ifFalse: [ self writeStreamDo: doBlock ]! !

!FileReference methodsFor: 'streams' stamp: 'lr 7/19/2010 19:35'!
writeStreamIfPresent: presentBlock
	^ self isFile 
		ifTrue: [ presentBlock value ]
		ifFalse: [ self writeStream ]! !

Object subclass: #FileSystem
	instanceVariableNames: 'workingDirectory store'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileSystem commentStamp: '<historical>' prior: 0!
I present a low-level protocol for interacting with filesystems. I hold a reference to
a store (a subinstance of FileSystemStore) which takes care of the details of performing 
file and directory operations on the filesystem I represent. 

I keep track of the current directory, and am responsible for resolving all paths that
I pass into my store. My store acts as a factory and offers platform specific actions.


FileSystem instances know two methods that return a Reference object: workingDirectory and root.

FileSystem disk workingDirectory
FileSystem disk root

!

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/4/2012 19:19'!
* aFileOrDirectoryName
	^ self disk * aFileOrDirectoryName! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/4/2012 19:19'!
/ aFileOrDirectoryName
	^ self disk / aFileOrDirectoryName! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'EstebanLorenzano 4/3/2012 11:30'!
disk
	"Answer a filesystem that represents the 'on-disk' filesystem used by the host operating system."

	^ DiskStore currentFileSystem! !

!FileSystem class methodsFor: '*filesystem-memory' stamp: 'EstebanLorenzano 4/3/2012 13:13'!
inMemory
	"This method provides compatibility with the original Filesystem interface"
	self deprecated: 'Use #memory' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self memory! !

!FileSystem class methodsFor: '*filesystem-zip' stamp: 'EstebanLorenzano 4/3/2012 17:21'!
inZip: aReference
	self deprecated: 'Use #zip:' on: '3 April 2012' in: 'Pharo 1.4'. 
	^ self zip: aReference ! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/7/2012 11:57'!
lookInUsualPlaces: fileName
	"Check the default directory, the imagePath, and the vmPath (and the vmPath's owner) for this file."

	| file |
	 (file := fileName asFileReference) exists
		ifTrue: [ ^ file ].

	(file := Smalltalk imageFile resolve: fileName) exists
		ifTrue: [ ^ file ].

	(file :=  Smalltalk vmDirectory resolve: fileName) exists
		ifTrue: [ ^ file ].

	(file := Smalltalk vmDirectory parent resolve: fileName) exists
		ifTrue: [ ^ file ].

	^ nil! !

!FileSystem class methodsFor: '*filesystem-memory' stamp: 'EstebanLorenzano 4/3/2012 09:36'!
memory
	^ self store: (MemoryStore new)! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'EstebanLorenzano 4/3/2012 13:13'!
onDisk
	"This method provides compatibility with the original Filesystem interface"
	self deprecated: 'Use #disk' on: '3 April 2012' in: 'Pharo 1.4'. 
	^ self disk! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/4/2012 20:54'!
root
	^ self disk root! !

!FileSystem class methodsFor: 'initializing' stamp: 'EstebanLorenzano 4/3/2012 13:13'!
startUp: aBoolean
	"This is only here to deal with migration from older versions of
	FileSystem that wanted to receive startup notifcations."
	
	Smalltalk removeFromStartUpList: self! !

!FileSystem class methodsFor: 'instance creation' stamp: 'cwp 2/18/2011 20:34'!
store: aStore
	^ self basicNew
		initializeWithStore: aStore;
		yourself! !

!FileSystem class methodsFor: '*filesystem-disk' stamp: 'CamilloBruni 5/4/2012 20:53'!
workingDirectory
	^ self disk workingDirectory! !

!FileSystem class methodsFor: '*filesystem-zip' stamp: 'EstebanLorenzano 4/3/2012 17:20'!
zip: aReference
	^ self store: (ZipStore reference: aReference)! !

!FileSystem methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
* anObject
	"Return a relative reference."
	
	^ self referenceTo:( Path * anObject)! !

!FileSystem methodsFor: 'navigating' stamp: 'cwp 3/25/2011 13:04'!
/ anObject
	"Return the absolute reference obtained by resolving anObject against the
	root of this filesystem."
	
	^ self root / anObject! !

!FileSystem methodsFor: 'comparing' stamp: 'cwp 2/18/2011 16:08'!
= other
	^ self species = other species and: [self store = other store]! !

!FileSystem methodsFor: 'accessing' stamp: 'cwp 3/29/2011 15:58'!
changeDirectory: aPath
	self workingDirectoryPath: (self resolve: aPath)! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:01'!
checkName: aString fixErrors: fixErrors
	^ store checkName: aString fixErrors: fixErrors! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/19/2012 00:31'!
childrenAt: aResolvable
	^ Array streamContents: [ :out | 
		self childrenAt: aResolvable do: [ :path|
			out nextPut: path ]].! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/19/2012 00:30'!
childrenAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		nodesDo: [ :entry | 
			aBlock value: path / (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 2/19/2011 01:39'!
close
	store close! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:29'!
copy: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock
	"Copy the file referenced as sourcePath to the destination referred as destPath. Perform associate actions in case of problems."

	store
		copy: (self resolve: sourcePath)
		ifAbsent: absentBlock
		to: (self resolve: destinationPath)
		ifPresent: presentBlock
		fileSystem: self! !

!FileSystem methodsFor: 'public' stamp: 'cwp 4/3/2011 22:17'!
copy: sourcePath to: destPath
	"Copy the file referenced as sourcePath to the destination referred as destPath.  
	If there is no file at sourcePath, raise FileDoesNotExist.
	If destPath is a file, raise FileExists."
	
	self
		copy: sourcePath
		ifAbsent: [ store signalFileDoesNotExist: sourcePath ]
		to: destPath
		ifPresent: [ store signalFileExists: destPath ]! !

!FileSystem methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:06'!
copy: aPath toReference: destRef
	| inputStream path |
	
	path := self resolve: aPath.
	
	[
	inputStream := self readStreamOn: path.
	inputStream ifNil: [ store signalFileDoesNotExist: path ].
	destRef fileSystem copyFrom: inputStream to: destRef path 
	
		] ensure: [ inputStream ifNotNil: [ inputStream close ]]! !

!FileSystem methodsFor: 'private' stamp: 'CamilloBruni 1/20/2012 12:33'!
copyFrom: inputStream to: destPath
	| buffer out |
	out := nil.
	(self exists: destPath)
		ifTrue: [ store signalFileExists: destPath ].
	^ [ 
	out := self writeStreamOn: destPath.
	buffer := ByteArray new: 1024.
	[ inputStream atEnd ]
		whileFalse: [ 
			buffer := inputStream nextInto: buffer.
			out nextPutAll: buffer ]] ensure: [ out ifNotNil: [ out close ]]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 13:15'!
createDirectory: aResolvable
	"Resolve aResolvable into an absolute path, then as the store to create a directory there. 
	The store is expected to raise an exception if it cannot do so."

	^ store createDirectory: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 13:14'!
delete: aResolvable
	store delete: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:14'!
delimiter
	"Return path delimiter used by this filesystem."

	^ store delimiter! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/19/2012 00:32'!
directoriesAt: aResolvable
	^ Array streamContents: [ :out | 
		self directoriesAt: aResolvable do: [ :path|
			out nextPut: path ]].! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/19/2012 00:30'!
directoriesAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		directoryNodesDo: [ :entry | 
			aBlock value: path / (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:16'!
ensureDirectory: aResolvable
	"Resolve the argument to an absolute path, then ask the store to make
	sure that all the directories contained in the argument path exist or are created."
	
	store ensureDirectory: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'cwp 4/3/2011 22:16'!
entriesAt: aResolvable
	| path entry aFilesystem |
	path := self resolve: aResolvable.
	aFilesystem := self.
	^ Array
		streamContents: [ :out | 
			store
				directoryAt: path
				ifAbsent: [ store signalDirectoryDoesNotExist: path ]
				nodesDo: [ :node | 
					entry := store entryFromNode: node path: path for: aFilesystem.
					out nextPut: entry ] ]! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:27'!
entryAt: aResolvable
	| path |
	path := self resolve: aResolvable.
	^ store
		nodeAt: path
		ifPresent: [ :node | store entryFromNode: node fileSystem: self path: path ]
		ifAbsent: [ store signalFileDoesNotExist: path ]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:19'!
exists: aResolvable
	"Resolve the argument, and answer true if the there is
	a file or directory at that path, false if there is not."
	
	^ store exists: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:15'!
extensionDelimiter
	^ $.! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/19/2012 00:32'!
filesAt: aResolvable
	^ Array streamContents: [ :out | 
		self filesAt: aResolvable do: [ :path|
			out nextPut: path ]].! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/19/2012 00:30'!
filesAt: aResolvable do: aBlock
	| path |
	path := self resolve: aResolvable.
	store
		directoryAt: path
		ifAbsent: [ store signalDirectoryDoesNotExist: path ]
		fileNodesDo: [ :entry | 
			aBlock value: path / (store basenameFromEntry: entry) ]! !

!FileSystem methodsFor: 'printing' stamp: 'cwp 2/18/2011 16:34'!
forReferencePrintOn: aStream
	store forReferencePrintOn: aStream! !

!FileSystem methodsFor: 'comparing' stamp: 'cwp 2/18/2011 16:08'!
hash
	^ store hash! !

!FileSystem methodsFor: 'initialize-release' stamp: 'cwp 2/18/2011 20:33'!
initializeWithStore: aStore
	store := aStore.
	workingDirectory := store defaultWorkingDirectory! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:19'!
isCaseSensitive
	^ self store isCaseSensitive! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:21'!
isDirectory: aResolvable
	"Resolve the argument, and answer true if the result refers
	to a directory, false if it refers to a file or doesn't exist."

	^ store isDirectory: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:23'!
isFile: aResolvable
	"Resolve the argument, and answer true if the result refers
	to a file, false if it refers to a directory or doesn't exist."

	| path |
	path := self resolve: aResolvable.
	^ store isFile: path! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 00:39'!
isReadable: aResolvable
	"Resolve the argument, and answer true if the there is
	a file or directory that can be read from."
	
	^ store isReadable: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 00:39'!
isWritable: aResolvable
	"Resolve the argument, and answer true if the there is
	a file that can be written to or directory that can be changed."
	
	^ store isWritable: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 13:03'!
mimeTypesAt: aResolvable
	"Return the possible mime types for the given path."
	^ store mimeTypesAt: (self resolve: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/25/2011 19:23'!
open
	"Some kinds of filesystems need to open connections to external resources. Does nothing by default."
	
	store open! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
open: aResolvable writable: aBoolean 
	"Resolve aResolvable into an absolute path, then ask the store to open the file at
	that path using the specified access mode."
	
	| path |
	path := self resolve: aResolvable.
	^ store handleClass 
		open: (FileReference fileSystem: self path: path) 
		writable: aBoolean
		! !

!FileSystem methodsFor: 'delegated' stamp: 'cwp 3/25/2011 13:16'!
openFileStream: aResolvable writable: aBoolean
	^ store 
		openFileStream: (self resolve: aResolvable) 
		writable: aBoolean! !

!FileSystem methodsFor: 'private' stamp: 'CamilloBruni 1/20/2012 13:34'!
openStreamDescription: aResolvable writable: aBoolean
	"I am  a helper method to delegate basicOpen:writable: to the store.
	 I am called from FSHandle implementations."
	
	| path |
	
	path := self resolve: aResolvable.
	^ store basicOpen: path writable: aBoolean! !

!FileSystem methodsFor: 'converting' stamp: 'cwp 11/21/2009 11:30'!
pathFromObject: anObject 
	^ anObject asPathWith: self! !

!FileSystem methodsFor: 'converting' stamp: 'cwp 2/18/2011 16:39'!
pathFromString: aString
	^ store pathFromString: aString! !

!FileSystem methodsFor: 'printing' stamp: 'cwp 2/28/2011 12:29'!
printPath: aPath on: aStream
	store printPath: aPath on: aStream! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/20/2012 13:23'!
readStreamOn: aResolvable
	"Resolve the argument into an absolute path and open a file handle on the file
	at that path. Ask the handle to give us a read stream for reading the file."

	^ (self open: aResolvable writable: false) readStream.! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
referenceTo: aResolvable 
	"Answer a reference to the argument from the context of the receiver filesystem.  	
		Example: FSFilesystem disk referenceTo: 'plonk.taz'"

	^ FileReference 
		fileSystem: self
		path: (self pathFromObject: aResolvable)! !

!FileSystem methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:24'!
rename: sourcePath ifAbsent: aBlock to: destPath ifPresent: pBlock
	"Rename the file referenced as sourcePath to the destination referred as destPath. 
	Perform associate actions in case of problems."
	
	| source destination |
	source := self resolve: sourcePath.
	destination := self resolve: destPath.
	store
		rename: source
		ifAbsent: aBlock
		to: destination
		ifPresent: pBlock
		fileSystem: self.
	^ destination! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/23/2012 15:49'!
rename: sourcePath to: destName
	"Rename the file referenced as sourcePath to destPath.  Raise exceptions 
	FileExists or FileDoesNotExist  if the operation fails"
	
	^ self
		rename: sourcePath
		ifAbsent: [store signalFileDoesNotExist: sourcePath]
		to: destName
		ifPresent: [self signalFileExists: destName]! !

!FileSystem methodsFor: 'public' stamp: 'cwp 3/29/2011 15:54'!
resolve: aResolvable
	^ aResolvable asResolvedBy: self! !

!FileSystem methodsFor: 'navigating' stamp: 'CamilloBruni 2/4/2012 19:22'!
resolvePath: aPath
	"Return a path where the argument is resolved in the context of the
	receiver. The behavior is similar to the one of a command line.
		> cd /a/b/c
		> cd b
		The shell will attempt to make /a/b/c/b the current directory. "

	^ workingDirectory resolve: aPath! !

!FileSystem methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/3/2012 13:06'!
resolveReference: aReference
	^ aReference fileSystem = self ifTrue:
		[workingDirectory resolvePath: aReference path]! !

!FileSystem methodsFor: 'navigating' stamp: 'cwp 3/29/2011 15:56'!
resolveString: aString
	"Returns the root of the receiver filesystem, i.e. / on unix"
	
	^ workingDirectory resolvePath: (store pathFromString: aString)! !

!FileSystem methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
root
	"Returns the root of the receiver filesystem, i.e. / on unix"
	^ self referenceTo: Path root! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:12'!
separator
	"Return path separator used by this filesystem."

	^ store separator! !

!FileSystem methodsFor: 'accessing' stamp: 'cwp 2/18/2011 16:08'!
store
	^ store! !

!FileSystem methodsFor: 'converting' stamp: 'cwp 2/18/2011 12:09'!
stringFromPath: aPath
	^ store stringFromPath: aPath! !

!FileSystem methodsFor: 'accessing' stamp: 'StephaneDucasse 7/3/2011 16:07'!
workingDirectory
	"Returns a reference to the directory from where the image was launched"

	^ self referenceTo: self workingDirectoryPath! !

!FileSystem methodsFor: 'accessing' stamp: 'cwp 3/29/2011 15:57'!
workingDirectoryPath
	^ workingDirectory! !

!FileSystem methodsFor: 'accessing' stamp: 'CamilloBruni 5/4/2012 19:14'!
workingDirectoryPath: aPath

	aPath isAbsolute 
		ifFalse: [ self error: 'Cannot set the working directory to a relative path' ].
	workingDirectory := aPath
! !

!FileSystem methodsFor: 'public' stamp: 'CamilloBruni 1/20/2012 13:23'!
writeStreamOn: aResolvable
	"Open a write stream on the file referred by the argument. It can be a string or a path"

	^ (self open: aResolvable writable: true) writeStream.! !

Object subclass: #FileSystemDirectoryEntry
	instanceVariableNames: 'reference creation modification isDirectory size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Public'!

!FileSystemDirectoryEntry commentStamp: 'cwp 11/18/2009 11:09' prior: 0!
I am a cache for metadata about a file or directory. The information I hold is as follows:

reference
	A reference to the file or directory to which my data pertains.
	
creation
	The creation date and time, stored as number seconds since the 
	Smalltalk epoch.
	
modification
	The modification date and time, number seconds since the Smalltalk epoch.
	
isDirectory 
	True if my data pertains to a directory, false if a file.
	
size
	Size in bytes for a file, 0 for a directory.
!

!FileSystemDirectoryEntry class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/3/2012 13:20'!
fileSystem: aFilesystem path: aPath creation: cTime modification: mTime isDir: aBoolean size: anInteger
	"Create a directory entry given a filesystem and a path in such filesystem. In addition, the creation and modification time are required as well as a boolean that indicates whether the entry is a folder or a file and its size."

	^ self 
		reference: (aFilesystem referenceTo: aPath)
		creation: cTime
		modification: mTime
		isDir: aBoolean
		size: anInteger! !

!FileSystemDirectoryEntry class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/3/2012 13:21'!
filesystem: aFilesystem path: aPath creation: cTime modification: mTime isDir: aBoolean size: anInteger
	"Create a directory entry given a filesystem and a path in such filesystem. In addition, the creation and modification time are required as well as a boolean that indicates whether the entry is a folder or a file and its size."

	self deprecated: 'Use #fileSystem:path:creation:modification:isDir:size:' on: '3 April 2012' in: 'Pharo 1.4'.

	^ self 
		fileSystem: aFilesystem 
		path: aPath 
		creation: cTime 
		modification: mTime 
		isDir: aBoolean 
		size: anInteger
! !

!FileSystemDirectoryEntry class methodsFor: 'instance creation' stamp: 'StephaneDucasse 1/27/2011 22:19'!
reference: ref creation: cTime modification: mTime isDir: aBoolean size: anInteger
	"Create a directory entry for the file reference ref, with the creation time, cTime, the modification time, mTime. aBoolean indicates if the entry represents a directory or a file of size given by anInteger"
	
	^ self basicNew
		initializeWithRef: ref
		creation: cTime
		modification: mTime
		isDir: aBoolean
		size: anInteger! !

!FileSystemDirectoryEntry methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
asFileReference
	^ reference! !

!FileSystemDirectoryEntry methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:30'!
asReference
	self deprecated: 'Use #asFileReference instead'  on: '12 April 2012' in:  'Pharo 1.4' .
	^ self asFileReference! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 2/15/2010 17:59'!
basename

	^ reference basename! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'abc 5/11/2012 23:31'!
creation
	^ self creationTime ! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 1/27/2011 22:16'!
creationSeconds
	"Return the creation date and time of the entry receiver in seconds."
		
	^ creation! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'abc 5/11/2012 23:31'!
creationTime
	"Return the creation date and time of the entry receiver."
	
	^ DateAndTime fromSeconds: creation! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/15/2011 17:02'!
extension
	^ reference extension! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/12/2011 20:33'!
fullName
	^ reference fullName! !

!FileSystemDirectoryEntry methodsFor: 'initialize-release' stamp: 'cwp 11/15/2009 21:52'!
initializeWithRef: ref creation: cTime modification: mTime isDir: bool size: bytes
	
	reference := ref.
	creation := cTime.
	modification := mTime.
	isDirectory := bool.
	size := bytes! !

!FileSystemDirectoryEntry methodsFor: 'testing' stamp: 'StephaneDucasse 1/27/2011 22:16'!
isDirectory
	"Return whether the receiver is a directory"
	
	^ isDirectory! !

!FileSystemDirectoryEntry methodsFor: 'testing' stamp: 'StephaneDucasse 1/27/2011 22:16'!
isFile
	"Return whether the receiver is a file"
	
	^ isDirectory not! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'abc 5/11/2012 23:30'!
modification
	^ self modificationTime ! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 1/27/2011 22:15'!
modificationSeconds
	"Return the modification date and time of the entry receiver in seconds."
	^ modification! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'abc 5/11/2012 23:30'!
modificationTime
	"Return the modification date and time of the entry receiver."

	^ DateAndTime fromSeconds: modification! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/12/2011 20:56'!
pathSegments
	^ reference pathSegments! !

!FileSystemDirectoryEntry methodsFor: 'printing' stamp: 'sd 2/11/2011 19:40'!
printOn: aStream

	aStream nextPutAll: 'DirectoryEntry: '.
	reference ifNotNilDo: [:ref | aStream nextPutAll: reference printString].! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/12/2011 20:32'!
readStream
	^ reference readStream! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'cwp 11/15/2009 21:54'!
reference
	^ reference! !

!FileSystemDirectoryEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 1/27/2011 22:15'!
size
	"Returns the receiver size"
	^ size! !

!FileSystemDirectoryEntry methodsFor: 'delegate' stamp: 'CamilloBruni 8/12/2011 20:33'!
writeStream
	^ reference writeStream! !

Object subclass: #FileSystemGuide
	instanceVariableNames: 'visitor work'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!FileSystemGuide commentStamp: 'cwp 11/18/2009 12:09' prior: 0!
I am an abstract superclass for objects that fulfill the Guide role in the Guide/Visitor pattern. My subclasses know how to traverse a filesystem in a specific order, "showing" the files and directories they encounter to a visitor.

visitor
	An object that fulfills the Visitor role and implements the visitor protocol.
	
work
	An OrderedCollection, used to keep track of filesystem nodes that have not yet been visited!

FileSystemGuide subclass: #BreadthFirstGuide
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!BreadthFirstGuide commentStamp: 'cwp 11/18/2009 12:13' prior: 0!
I traverse the filesystem in breadth-first order. Given this hierarchy:

alpha
	beta
	gamma
delta
	epsilon

I would visit the nodes in the following order:  alpha, delta, beta, gamma, epsilon.

I use my work instance variable as a queue, adding nodes to be visited to the end and retrieving them from the beginning.
!

!BreadthFirstGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:23'!
show: aReference 
	self push: aReference entry.
	self whileNotDoneDo: [ self visitNextEntry: self top ]! !

!BreadthFirstGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:19'!
visitNextEntry: entry
	entry isFile 
		ifTrue: [ ^ visitor visitFile: entry ].
	
	visitor visitDirectory: entry.
	self pushAll: entry reference entries.! !

!FileSystemGuide class methodsFor: 'instance creation' stamp: 'cwp 10/29/2009 19:27'!
for: aVisitor
	^ self basicNew initializeWithVisitor: aVisitor! !

!FileSystemGuide class methodsFor: 'instance creation' stamp: 'cwp 11/17/2009 11:58'!
show: aReference to: aVisitor
	^ (self for: aVisitor) show: aReference! !

!FileSystemGuide methodsFor: 'initialize-release' stamp: 'StephaneDucasse 1/27/2011 10:37'!
initialize

	super initialize.
	work := OrderedCollection new! !

!FileSystemGuide methodsFor: 'initialize-release' stamp: 'cwp 10/29/2009 23:48'!
initializeWithVisitor: aVisitor
	self initialize.
	visitor := aVisitor.
	! !

!FileSystemGuide methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 18:18'!
pop
	^ work removeLast! !

!FileSystemGuide methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 18:18'!
push: anObject
	work add: anObject! !

!FileSystemGuide methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 18:20'!
pushAll: aCollection 
	aCollection do: [ :ea | self push: ea ]! !

!FileSystemGuide methodsFor: 'showing' stamp: 'cwp 10/29/2009 23:21'!
show: aReference
	self subclassResponsibility! !

!FileSystemGuide methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/12/2011 18:22'!
top
	^ work removeFirst! !

!FileSystemGuide methodsFor: 'showing' stamp: 'lr 7/13/2010 15:36'!
whileNotDoneDo: aBlock
	[ work isEmpty ] whileFalse: [ aBlock value ]! !

FileSystemGuide subclass: #PostorderGuide
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!PostorderGuide commentStamp: 'cwp 11/18/2009 12:16' prior: 0!
I traverse the filesystem in depth-first post order. Given this hierarchy:

alpha
	beta
	gamma
delta
	epsilon

I would visit the nodes in the following order:  beta, gamma, alpha, epsilon, delta.

I use my work instance variable as a stack. I push messages that cause nodes to be traversed or visited, and execute them in reverse order.!

!PostorderGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:18'!
pushTraverse: aReference 
	self push: (Message 
				selector: #traverse:
				argument: aReference)! !

!PostorderGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:18'!
pushVisit: aReference 
	self push: (Message 
				selector: #visit:
				argument: aReference)! !

!PostorderGuide methodsFor: 'showing' stamp: 'cwp 11/16/2009 10:50'!
show: aReference 
	self pushTraverse: aReference entry.
	self whileNotDoneDo: [self pop sendTo: self ]! !

!PostorderGuide methodsFor: 'showing' stamp: 'cwp 11/16/2009 21:57'!
traverse: anEntry 
	self pushVisit: anEntry.
	anEntry isDirectory ifTrue: 
		[anEntry reference entries reverseDo: 
			[:ea | self pushTraverse: ea]]! !

!PostorderGuide methodsFor: 'showing' stamp: 'cwp 11/16/2009 10:50'!
visit: anEntry
	anEntry isDirectory
		ifTrue: [visitor visitDirectory: anEntry]
		ifFalse: [visitor visitFile: anEntry]
	! !

FileSystemGuide subclass: #PreorderGuide
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!PreorderGuide commentStamp: 'cwp 11/18/2009 12:18' prior: 0!
I traverse the filesystem in depth-first pre order. Given this hierarchy:

alpha
	beta
	gamma
delta
	epsilon

I would visit the nodes in the following order: alpha, beta, gamma, delta, epsilon.

I use my work instance variable as a stack. I push nodes to be visited and visit them in reverse order.!

!PreorderGuide methodsFor: 'showing' stamp: 'CamilloBruni 8/12/2011 18:19'!
pushAll: aCollection 
	aCollection reverseDo: [ :ea | work add: ea ]! !

!PreorderGuide methodsFor: 'showing' stamp: 'cwp 11/16/2009 10:47'!
show: aReference 
	self push: aReference entry.
	self whileNotDoneDo: 
		[| entry |
		entry := self pop.
		entry isFile 
			ifTrue: [visitor visitFile: entry]
			ifFalse: 
				[visitor visitDirectory: entry.
				self pushAll: entry reference entries]]! !

Object subclass: #FileSystemHandle
	instanceVariableNames: 'reference writable'
	classVariableNames: 'Primitives'
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileSystemHandle commentStamp: 'cwp 11/18/2009 11:11' prior: 0!
I am an abstract superclass for file handle implementations. I provide a uniform interface that streams can use for read and write operations on a file regardless of the filesystem. I encapsulate the actual IO primitives.!

FileSystemHandle subclass: #FileHandle
	instanceVariableNames: 'id'
	classVariableNames: 'Registry'
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!FileHandle commentStamp: 'cwp 11/18/2009 13:02' prior: 0!
I provide an interface for doing IO on an open file. I keep an id, which as an opaque identifier used by the FilePlugin primitives. I translate positions from the 1-based indexes used in Smalltalk to the 0-based offsets used by the primitives.

I do not implement the primitives myself, instead delegating those to an instance of FilePluginPrimitives.!

!FileHandle class methodsFor: 'class initialization' stamp: 'cwp 11/20/2009 16:16'!
initialize
	self useFilePlugin.
! !

!FileHandle class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 5/24/2012 15:06'!
register: aFileHandle
	"properly register the given FileHandle for being closed on garbage collection"
	^self registry add: aFileHandle! !

!FileHandle class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 5/24/2012 15:06'!
registry
	^Registry ifNil: [Registry := WeakRegistry new]
! !

!FileHandle class methodsFor: 'class initialization' stamp: 'cwp 11/20/2009 16:51'!
startUp: resuming
	"This functionality is disabled for now, to avoid doing a lot of processing at
	image start up. To reenable, add this class to the start up list."
	
	resuming ifTrue: [self allInstancesDo: [:ea | ea startUp]]! !

!FileHandle class methodsFor: 'class initialization' stamp: 'EstebanLorenzano 4/2/2012 11:46'!
useFilePlugin
	Primitives := FilePluginPrims new! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:19'!
assureOpen
	"compatibility method to make the FileHandle Tests pass"
	self isOpen ifFalse: [ id := self basicOpen ].! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:20'!
at: index read: buffer startingAt: start count: count
	
	self flag: 'TODO: remove once FileHandles are really used!!'.
	self assureOpen.
	
	^ Primitives 
		setPosition: id to: index - 1;
		read: id into: buffer startingAt: start count: count
! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:20'!
at: index write: buffer startingAt: start count: count
	
	self flag: 'TODO: remove once FileHandles are really used!!'.
	self assureOpen.
	
	Primitives 
		setPosition: id to: index - 1;
		write: id from: buffer startingAt: start count: count
! !

!FileHandle methodsFor: 'public' stamp: 'cwp 11/20/2009 14:59'!
close
	Primitives close: id.
	id := nil! !

!FileHandle methodsFor: 'finalization' stamp: 'CamilloBruni 5/24/2012 15:07'!
finalize
	self primCloseNoError: id.! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:20'!
flush
	
	self flag: 'TODO: remove once FileHandles are really used!!'.
	self assureOpen.
	
	Primitives flush: id! !

!FileHandle methodsFor: 'testing' stamp: 'cwp 7/22/2009 07:10'!
isOpen
	^ (Primitives sizeOrNil: id) notNil! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:18'!
open
	self flag: 'TODO: for now we solely rely on the old FileStreams'
	"id := self basicOpen.
	id ifNil: 	[
		reference exists ifFalse: [FileDoesNotExist signalWith: reference].
		self error: 'Unable to open file ' , reference printString]"! !

!FileHandle methodsFor: 'public' stamp: 'abc 5/11/2012 23:22'!
readStream
	^ (FileStream onHandle: self)
		ifNil: [ self streamError ]
	! !

!FileHandle methodsFor: 'finalization' stamp: 'CamilloBruni 5/24/2012 15:05'!
register
	"register the instance for proper clreanup on garbage collection"
	^self class register: self! !

!FileHandle methodsFor: 'public' stamp: 'CamilloBruni 5/24/2012 15:20'!
size
	
	self flag: 'TODO: remove once FileHandles are really used!!'.
	self assureOpen.
	
	^ Primitives size: id! !

!FileHandle methodsFor: 'private' stamp: 'cwp 11/20/2009 16:48'!
startUp
	"This functionality is disabled for now, to avoid doing lots of processing
	on start up."
	
	"We're starting up in a new OS process, so the file id will be invalid.
	Try to reopen the file, but fail silently: just leave the id as nil. #isOpen will
	answer false, and we'll raise an error if anyone tries to do IO."
	
	self basicOpen! !

!FileHandle methodsFor: 'public' stamp: 'abc 5/11/2012 23:22'!
streamError
	reference exists
		ifFalse: [FileDoesNotExist signalWith: reference].
	self error: 'Unable to open file ' , reference printString! !

!FileHandle methodsFor: 'public' stamp: 'cwp 7/22/2009 08:17'!
truncateTo: anInteger
	Primitives truncate: id to: anInteger.
	self reopen! !

!FileHandle methodsFor: 'public' stamp: 'abc 5/11/2012 23:22'!
writeStream
	^( FileStream onHandle: self)
		 ifNil: [ self streamError ]! !

!FileSystemHandle class methodsFor: 'instance creation' stamp: 'cwp 7/26/2009 12:52'!
on: aReference writable: aBoolean
	^ self new setReference: aReference writable: aBoolean! !

!FileSystemHandle class methodsFor: 'instance creation' stamp: 'cwp 7/26/2009 12:52'!
open: aReference writable: aBoolean
	^ (self on: aReference writable: aBoolean) open! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/29/2009 22:19'!
at: index
	| buffer |
	buffer := ByteArray new: 1.
	self at: index read: buffer startingAt: 1 count: 1.
	^ buffer at: 1! !

!FileSystemHandle methodsFor: 'public' stamp: 'lr 4/13/2010 16:10'!
at: index put: anObject
	| buffer |
	buffer := ByteArray with: (anObject isCharacter
		ifTrue: [ anObject codePoint ]
		ifFalse: [ anObject ]).
	self at: index write: buffer startingAt: 1 count: 1.
	! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
at: offset read: buffer startingAt: start count: count
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
at: offset write: buffer startingAt: start count: count
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:06'!
basicOpen
	"get the raw stream description from the filesystem's store"
	^ self fileSystem 
		openStreamDescription: reference path
		writable: writable! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
close
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 13:02'!
fileSystem
	^ reference fileSystem ! !

!FileSystemHandle methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 13:03'!
filesystem
	self deprecated: 'Use fileSystem' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self fileSystem ! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
flush
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'accessing' stamp: 'abc 5/11/2012 23:24'!
fullName
	^ reference fullName! !

!FileSystemHandle methodsFor: 'testing' stamp: 'cwp 7/26/2009 12:50'!
isOpen
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'testing' stamp: 'abc 5/11/2012 23:24'!
isWritable
	^ writable! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
open
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'public' stamp: 'EstebanLorenzano 4/2/2012 11:36'!
readStream
	^ self isOpen ifTrue: [FileSystemReadStream onHandle: self]! !

!FileSystemHandle methodsFor: 'accessing' stamp: 'cwp 7/26/2009 12:51'!
reference
	^ reference! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:51'!
reopen
	self close.
	self open! !

!FileSystemHandle methodsFor: 'initialize-release' stamp: 'cwp 11/20/2009 14:56'!
setReference: aReference writable: aBoolean
	reference := aReference resolve.
	writable := aBoolean! !

!FileSystemHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 12:50'!
truncateTo: anInteger
	self subclassResponsibility! !

!FileSystemHandle methodsFor: 'public' stamp: 'EstebanLorenzano 4/2/2012 11:36'!
writeStream
	^ FileSystemWriteStream onHandle: self! !

FileSystemHandle subclass: #MemoryHandle
	instanceVariableNames: 'bytes size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!MemoryHandle commentStamp: '<historical>' prior: 0!
I provide "primitives" for doing IO on files in a MemoryFileSystem. I buffer writes until the stream is flushed or closed. !

!MemoryHandle methodsFor: 'public' stamp: 'cwp 7/29/2009 22:07'!
at: index
	^ bytes at: index! !

!MemoryHandle methodsFor: 'public' stamp: 'lr 4/13/2010 16:12'!
at: index put: anObject
	index > bytes size ifTrue: [self grow].
	bytes at: index put: (anObject isCharacter
		ifTrue: [ anObject codePoint ]
		ifFalse: [ anObject ]).
	size := size max: index! !

!MemoryHandle methodsFor: 'public' stamp: 'lr 2/15/2010 20:54'!
at: index read: aCollection startingAt: start count: count 
	| max stop |
	max := size - index + 1 min: count.
	stop := start + max - 1.
	aCollection 
		replaceFrom: start
		to: stop
		with: bytes
		startingAt: index.
	^ stop - start + 1! !

!MemoryHandle methodsFor: 'public' stamp: 'cwp 1/31/2010 22:39'!
at: first write: aCollection startingAt: start count: count 
	| last |
	writable ifFalse: [ self primitiveFailed ].
	last := first + count - 1.
	last > bytes size ifTrue: [self growTo: last].
	bytes 
		replaceFrom: first
		to: last
		with: aCollection
		startingAt: start.
	size := last! !

!MemoryHandle methodsFor: 'public' stamp: 'cwp 7/28/2009 23:10'!
close
	self truncateTo: size.
	bytes := nil! !

!MemoryHandle methodsFor: 'public' stamp: 'cwp 7/29/2009 22:23'!
flush
	self truncateTo: size! !

!MemoryHandle methodsFor: 'private' stamp: 'cwp 1/31/2010 22:45'!
grow
	^ self growTo: bytes size + 1! !

!MemoryHandle methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:01'!
growTo: anInteger
	| path |
	path := reference fileSystem resolve: reference path.
	bytes := reference fileSystem store growFile: path to: anInteger + self sizeIncrement! !

!MemoryHandle methodsFor: 'testing' stamp: 'cwp 7/26/2009 14:08'!
isOpen
	^ bytes notNil! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 1/20/2012 13:22'!
open

	bytes := self basicOpen.
	size := bytes size.! !

!MemoryHandle methodsFor: 'public' stamp: 'cwp 7/26/2009 14:27'!
size
	^ size! !

!MemoryHandle methodsFor: 'private' stamp: 'cwp 1/31/2010 22:40'!
sizeIncrement
	^ (bytes size min: 20) max: 1024! !

!MemoryHandle methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:01'!
truncateTo: anInteger
	| path |
	path := reference fileSystem resolve: reference path.
	bytes := reference fileSystem store replaceFile: path in: [ :all  | all first: anInteger ].
	size := anInteger! !

Object subclass: #FileSystemResolver
	instanceVariableNames: 'next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!FileSystemResolver commentStamp: 'cwp 3/29/2011 17:04' prior: 0!
I am an abstract superclass for objects that can resolve origins into references. Such objects use the Chain of Responsibility pattern, and when unable to resolve a particular origin, delegate that resolution request to the next resolver in the list.

next
	The next resolver in the list, or nil
!

!FileSystemResolver methodsFor: 'accessing' stamp: 'cwp 10/26/2009 20:53'!
addResolver: aResolver
	next
		ifNil: [next := aResolver]
		ifNotNil: [next addResolver: aResolver]! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'cwp 10/27/2009 11:18'!
canResolve: aSymbol
	^ self supportedOrigins includes: aSymbol! !

!FileSystemResolver methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:28'!
flushCaches
	self flushLocalCache.
	next ifNotNil: [next flushCaches]! !

!FileSystemResolver methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:28'!
flushLocalCache! !

!FileSystemResolver methodsFor: 'accessing' stamp: 'cwp 10/27/2009 11:25'!
next
	^ next! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'cwp 10/27/2009 11:18'!
resolve: aSymbol
	^ (self canResolve: aSymbol)
		ifTrue: [self perform: aSymbol]
		ifFalse: [self unknownOrigin: aSymbol]! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'EstebanLorenzano 4/12/2012 14:25'!
resolveString: aString 
	| decoded fs |
	decoded := (FilePath pathName: aString isEncoded: true) asSqueakPathName.
	fs := FileSystem disk.
	^ FileReference
		fileSystem: fs
		path: (fs pathFromString: decoded)! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'cwp 10/26/2009 20:06'!
supportedOrigins
	^ #()! !

!FileSystemResolver methodsFor: 'resolving' stamp: 'cwp 10/27/2009 09:26'!
unknownOrigin: aSymbol
	^ next ifNotNil: [next resolve: aSymbol]! !

FileSystemResolver subclass: #InteractiveResolver
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!InteractiveResolver commentStamp: 'cwp 11/18/2009 11:56' prior: 0!
I resolve origins by consulting the user. I maintain a cache of the user's responses.!

!InteractiveResolver methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:29'!
flushLocalCache
	cache := IdentityDictionary new! !

!InteractiveResolver methodsFor: 'initialize-release' stamp: 'cwp 10/27/2009 10:29'!
initialize
	self flushLocalCache! !

!InteractiveResolver methodsFor: 'resolving' stamp: 'cwp 10/27/2009 10:12'!
resolve: origin
	^ cache at: origin ifAbsent: [self unknownOrigin: origin]
	
! !

!InteractiveResolver methodsFor: 'resolving' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
unknownOrigin: origin
	| reference |
	^ (next ifNotNil: [next resolve: origin]) ifNil: 
			[reference := ResolutionRequest for: origin.
			reference ifNotNil: [cache at: origin put: reference]]! !

FileSystemResolver subclass: #PlatformResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!PlatformResolver commentStamp: 'cwp 11/18/2009 11:56' prior: 0!
I am an abstract superclass for platform-specific resolvers.!

PlatformResolver subclass: #MacOSResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!MacOSResolver commentStamp: 'cwp 11/18/2009 11:57' prior: 0!
I am an expert on Mac OS X filesystem conventions. I resolve origins according to these conventions.!

!MacOSResolver class methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:59'!
platformName
	^  'Mac OS'! !

!MacOSResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:45'!
desktop
	^ self home / 'Desktop'! !

!MacOSResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:50'!
documents
	^ self home / 'Documents'! !

!MacOSResolver methodsFor: 'origins' stamp: 'StephaneDucasse 7/5/2011 16:38'!
home
	^ (self resolveString: self class primitiveGetUntrustedUserDirectory)
		parent parent parent parent parent! !

!MacOSResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:11'!
library
	^ self home / 'Library' ! !

!MacOSResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:46'!
preferences 
	^ self library / 'Preferences'! !

!MacOSResolver methodsFor: 'resolving' stamp: 'CamilloBruni 5/24/2012 14:51'!
supportedOrigins
	^ super supportedOrigins , #( library )! !

!PlatformResolver class methodsFor: 'instance creation' stamp: 'tg 11/8/2010 19:05'!
forCurrentPlatform
	| platformName |
	platformName :=  Smalltalk os platformName.
	^ (self allSubclasses detect: [:ea | ea platformName = platformName]) new! !

!PlatformResolver class methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:58'!
platformName
	^ nil! !

!PlatformResolver class methodsFor: 'private' stamp: 'CamilloBruni 8/12/2011 17:36'!
primitiveGetUntrustedUserDirectory
	<primitive: 'primitiveGetUntrustedUserDirectory' module: 'SecurityPlugin'>
	self primitiveFailed.
	self flag: 'use a more decent way to get a path in the users home directory'.! !

!PlatformResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 15:35'!
desktop
	^ self subclassResponsibility! !

!PlatformResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 15:35'!
documents
	^ self subclassResponsibility! !

!PlatformResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:01'!
home
	^ self subclassResponsibility! !

!PlatformResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:07'!
preferences
	^ self subclassResponsibility! !

!PlatformResolver methodsFor: 'resolving' stamp: 'CamilloBruni 5/24/2012 12:46'!
supportedOrigins
	^ #(home desktop documents preferences)! !

PlatformResolver subclass: #UnixResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!UnixResolver commentStamp: '<historical>' prior: 0!
I am an expert on Unix filesystem conventions. I resolve origins according to these conventions.!

!UnixResolver class methodsFor: 'accessing' stamp: 'pls
12/18/2009 04:53'!
platformName
	^  'unix'! !

!UnixResolver methodsFor: 'origins' stamp: 'pls 12/18/2009
04:52'!
desktop
	^ self home / 'Desktop'! !

!UnixResolver methodsFor: 'origins' stamp: 'pls 12/18/2009
04:53'!
documents
	^ self home / 'Documents'! !

!UnixResolver methodsFor: 'origins' stamp: 'CamilloBruni 1/19/2012 00:55'!
home
	"FIXME: hackish solution. should use env vars"
	^ (self resolveString: self class primitiveGetUntrustedUserDirectory)
		parent parent parent ! !

!UnixResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:12'!
preferences
	^ self home / '.config' ! !

PlatformResolver subclass: #WindowsResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!WindowsResolver commentStamp: 'cwp 11/18/2009 11:57' prior: 0!
I am an expert on Windows filesystem conventions. I resolve origins according to these conventions.!

!WindowsResolver class methodsFor: 'accessing' stamp: 'cwp 10/27/2009 10:59'!
platformName
	^ 'Win32'! !

!WindowsResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:45'!
desktop
	^ self home / 'Desktop'! !

!WindowsResolver methodsFor: 'origins' stamp: 'cwp 10/27/2009 21:50'!
documents
	^ self home / 'My Documents'! !

!WindowsResolver methodsFor: 'origins' stamp: 'CamilloBruni 8/12/2011 17:17'!
home
	| pathString |
	pathString := self class primitiveGetUntrustedUserDirectory.
	^ (self resolveString: pathString) parent parent! !

!WindowsResolver methodsFor: 'origins' stamp: 'CamilloBruni 5/24/2012 12:24'!
preferences
	"For further details refer to http://code.google.com/p/pharo/issues/detail?id=5255" 

	|  osMajorVer osMinorVer rootPath exeFolderList  |
	
	"First try to adhere to hardcoded Microsoft conventions - though we really should be reading environment variables since %SYSTEMDRIVE% might not be C: "
	
       osMajorVer := Smalltalk os osVersion first.   
       osMinorVer := Smalltalk os osVersion third.

       osMajorVer >= $6 
		ifTrue: [ rootPath := 'C:\ProgramData\Pharo\' ].   "Vista, 2008, 7 and later"  
       osMajorVer = $5 
		ifTrue: [ rootPath := 'C:\Documents and Settings\All Users\Application Data\Pharo\' ]. "2000, XP, 2003"
       osMajorVer = $4 
		ifTrue: [  rootPath := 'C:\Windows\All Users\Application Data\Pharo\' . "98, 98SE, Me"
           osMinorVer = $0 
			ifTrue: [ rootPath := 'C:\WinNT\Profiles\All Users\Application Data\Pharo\' ] .    "NT 4"
     	].
       osMajorVer = $3 
		ifTrue: [ rootPath := 'C:\WinNT\Profiles\All Users\Application Data\Pharo\' ] . "NT 3.1 3.5 3.51"	

	"Now work out which rootPath user is using.  Perhaps we should inform users of their specific options"	
	rootPath ifNotNil: [ 
		rootPath := rootPath asFileReference.
		rootPath exists 
			 ifTrue: [ ^ rootPath ]].
		
	"An alternative fall back folder in case system locked down such that user cannot follow Microsoft conventions above, or is using a portable drive on a new system"	
	exeFolderList := (self default fullName subStrings: self slash) copyFrom: 1 to: 2.
	rootPath := (exeFolderList joinUsing: self slash) , '\_config\Pharo\' .
	
	rootPath ifNotNil: [ 
		rootPath := rootPath asFileReference.
		rootPath exists 
			ifTrue: [ ^ rootPath ]].
		
	^ FileSystem disk root. "Avoids later MNU"! !

FileSystemResolver subclass: #SystemResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!SystemResolver commentStamp: 'cwp 11/18/2009 11:58' prior: 0!
I resolve origins that are related to the currently running Smalltalk system, using primitives provided by the VM. !

!SystemResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 13:31'!
changes
	^ self image withExtension: 'changes'! !

!SystemResolver methodsFor: 'origins' stamp: 'cwp 10/26/2009 20:04'!
image
	^ self resolveString: self primImagePath! !

!SystemResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 13:34'!
imageDirectory
	^ self image parent! !

!SystemResolver methodsFor: 'primitives' stamp: 'cwp 10/26/2009 20:05'!
primImagePath
	"Answer the full path name for the current image."

	<primitive: 121>
	self primitiveFailed! !

!SystemResolver methodsFor: 'primitives' stamp: 'lr 7/13/2010 13:27'!
primVmDirectoryPath
	"Answer the full path name for the current virtual machine."

	<primitive: 142>
	self primitiveFailed! !

!SystemResolver methodsFor: 'resolving' stamp: 'lr 7/13/2010 13:35'!
supportedOrigins
	^ #(image imageDirectory changes vmBinary vmDirectory)! !

!SystemResolver methodsFor: 'origins' stamp: 'StephaneDucasse 7/3/2011 16:28'!
vmBinary
	^ self resolveString: (SmalltalkImage current vmFullPath)! !

!SystemResolver methodsFor: 'origins' stamp: 'lr 7/13/2010 13:33'!
vmDirectory
	^ self resolveString: self primVmDirectoryPath! !

Object subclass: #FileSystemStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!FileSystemStore commentStamp: '<historical>' prior: 0!
I am an abstract superclass for store implementations. My subclasses provide access to the actual data storage of a particular kind of filesystem. 

The file system can be accessed via
	FileSystem disk 
	FileSystem memory
	
My associated filesystem can be accessed as follows:
      DiskStore currentFileSystem!

FileSystemStore subclass: #DiskStore
	instanceVariableNames: 'maxFileNameLength'
	classVariableNames: 'CurrentFS Primitives'
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!DiskStore commentStamp: '<historical>' prior: 0!
I am an abstract superclass for disk store implementations. My subclasses provide access to the actual data storage of a particular kind of filesystem. 
!

!DiskStore class methodsFor: 'current' stamp: 'CamilloBruni 5/10/2012 15:41'!
activeClass
	self allSubclassesDo: [:ea | 
		ea isActiveClass ifTrue: [^ ea]].
	^ self! !

!DiskStore class methodsFor: 'current' stamp: 'cwp 2/27/2011 10:02'!
createDefault
	^ self new! !

!DiskStore class methodsFor: 'current' stamp: 'EstebanLorenzano 4/3/2012 11:30'!
current
	^ self currentFileSystem store! !

!DiskStore class methodsFor: 'current' stamp: 'CamilloBruni 5/13/2012 19:17'!
currentFileSystem
	^ CurrentFS ifNil: [
		CurrentFS := FileSystem store: self activeClass createDefault]! !

!DiskStore class methodsFor: 'current' stamp: 'EstebanLorenzano 4/3/2012 13:11'!
currentFilesystem
	self 
		deprecated: 'Use #currentFileSystem' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self currentFileSystem! !

!DiskStore class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/25/2012 15:47'!
initialize
	self useFilePlugin.
	Smalltalk 
		addToStartUpList: self;
		addToShutDownList: self.! !

!DiskStore class methodsFor: 'current' stamp: 'cwp 2/18/2011 17:20'!
isActiveClass
	^ self delimiter = Primitives delimiter! !

!DiskStore class methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:03'!
maxFileNameLength
	self subclassResponsibility ! !

!DiskStore class methodsFor: 'current' stamp: 'cwp 4/4/2011 19:04'!
reset
	CurrentFS := nil! !

!DiskStore class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/25/2012 15:47'!
shutDown: quitting
	"nothing to do so far"! !

!DiskStore class methodsFor: 'class initialization' stamp: 'cwp 2/27/2011 10:19'!
startUp: resuming
	resuming ifTrue: [self reset]! !

!DiskStore class methodsFor: 'class initialization' stamp: 'EstebanLorenzano 4/2/2012 11:46'!
useFilePlugin
	Primitives := FilePluginPrims new! !

!DiskStore methodsFor: 'comparing' stamp: 'cwp 2/27/2011 09:50'!
= other
	^ self species = other species! !

!DiskStore methodsFor: 'public' stamp: 'cwp 2/18/2011 13:27'!
basenameFromEntry: entry
	^ entry at: 1! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/18/2011 11:32'!
basicIsDirectory: anEntry
	^ anEntry at: 4! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/18/2011 11:33'!
basicIsFile: anEntry
	^ (anEntry at: 4) not! !

!DiskStore methodsFor: 'private' stamp: 'CamilloBruni 5/9/2012 00:48'!
basicIsReadable: anEntry
	^ true! !

!DiskStore methodsFor: 'private' stamp: 'CamilloBruni 5/9/2012 00:43'!
basicIsWritable: anEntry
	^ true! !

!DiskStore methodsFor: 'public' stamp: 'cwp 2/28/2011 12:35'!
basicOpen: aPath writable: aBoolean
	| string encoded |
	string := self stringFromPath: aPath.
	encoded := Primitives encode: string.
	^ Primitives open: encoded writable: aBoolean! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 5/13/2012 19:17'!
checkName: aFileName fixErrors: fixErrors
	"Check a string aFileName for validity as a file name. Answer the original file name if it is valid. If the name is not valid (e.g., it is too long or contains illegal characters) and fixing is false, raise an error. If fixing is true, fix the name (usually by truncating and/or tranforming characters), and answer the corrected name. The default behavior is just to truncate the name to the maximum length for this platform. Subclasses can do any kind of checking and correction appropriate for their platform."
	
	| maxLength |
	aFileName size = 0 ifTrue: [self error: 'zero length file name'].
	maxLength := self maxFileNameLength.
	aFileName size > maxLength ifTrue: [
		fixErrors
			ifTrue: [^ aFileName contractTo: maxLength]
			ifFalse: [self error: 'file name is too long']].
	^ aFileName! !

!DiskStore methodsFor: 'public' stamp: 'cwp 4/3/2011 22:17'!
createDirectory: path
	"Create a directory for the argument path. 
	If the path refers to an existing file, raise FileExists.
	If the path refers to an existing directory, raise DirectoryExists.
	If the parent directory of the path does not exist, raise DirectoryDoesNotExist"

	| parent encodedPathString pathString result |
	pathString := self stringFromPath: path.
	encodedPathString := Primitives encode: pathString.
	result := Primitives createDirectory: encodedPathString.
	result
		ifNil: [ 
			parent := path parent.
			(self exists: path)
				ifTrue: [ 
					(self isFile: path)
						ifTrue: [ self signalFileExists: path ]
						ifFalse: [ self signalDirectoryExists: path ] ].
			(self isDirectory: parent)
				ifFalse: [ ^ self signalDirectoryDoesNotExist: parent ].
			self primitiveFailed ].
	^ self! !

!DiskStore methodsFor: 'accessing' stamp: 'cwp 2/27/2011 10:03'!
defaultWorkingDirectory
	| pathString |
	pathString := Primitives decode: Primitives imageFile.
	^ (self pathFromString: pathString) parent! !

!DiskStore methodsFor: 'public' stamp: 'cwp 2/27/2011 10:03'!
delete: path
	| pathString encodedPathString |
	pathString := self stringFromPath: path.
	encodedPathString := Primitives encode: pathString.
	(self isDirectory: path)
		ifTrue: [ Primitives deleteDirectory: encodedPathString ]
		ifFalse: [ 
			StandardFileStream
				retryWithGC: [ Primitives deleteFile: encodedPathString ]
				until: [ :result | result notNil ]
				forFileNamed: pathString ]! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/27/2011 10:03'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock
	| encodedPathString index entry pathString |
	index := 1.
	pathString := self stringFromPath: aPath.
	encodedPathString := Primitives encode: pathString.
	entry := Primitives lookupEntryIn: encodedPathString index: index.
	entry = #badDirectoryPath
		ifTrue: [ ^ absentBlock value ].
	[ entry isNil ]
		whileFalse: [ 
			entry at: 1 put: (Primitives decode: entry first).
			aBlock value: entry.
			index := index + 1.
			entry := Primitives lookupEntryIn: encodedPathString index: index ].
	^ self! !

!DiskStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:26'!
entryFromNode: anArray fileSystem: aFilesystem path: aPath
	^ FileSystemDirectoryEntry
		fileSystem: aFilesystem
		path: aPath
		creation: (anArray at: 2)
		modification: (anArray at: 3)
		isDir: (anArray at: 4)
		size: (anArray at: 5)! !

!DiskStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:26'!
entryFromNode: anArray filesystem: aFilesystem path: aPath
	self deprecated: 'Use entryFromNode:fileSystem:path:' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self 
		entryFromNode: anArray 
		fileSystem: aFilesystem 
		path: aPath! !

!DiskStore methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:27'!
entryFromNode: node path: path for: aFileystem
	| entryPath |
	entryPath := path / (self basenameFromEntry: node).
	^ self entryFromNode: node fileSystem: aFileystem path: entryPath! !

!DiskStore methodsFor: 'printing' stamp: 'cwp 2/27/2011 08:32'!
forReferencePrintOn: aStream
! !

!DiskStore methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/2/2012 11:46'!
handleClass
	^ FileHandle! !

!DiskStore methodsFor: 'comparing' stamp: 'cwp 2/27/2011 09:51'!
hash
	^ self species hash! !

!DiskStore methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/13/2012 19:18'!
initialize
	super initialize.
	maxFileNameLength := Smalltalk vm maxFilenameLength ifNil: [ 255 ].! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 00:56'!
isReadable: aPath
	(self exists: aPath) ifFalse: [ ^ false ].
	self flag: 'TODO: we need a decent primitive for this...'.
	(self basicOpen: aPath writable: false) 
		ifNotNilDo: [ :id|
			Primitives close: id.
			^ true].
	^ false! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 01:01'!
isWritable: aPath
	(self exists: aPath) ifFalse: [ ^ false ].
	self flag: 'TODO: we need a decent primitive for this...'.
	(self basicOpen: aPath writable: true) 
		ifNotNilDo: [ :id|
			Primitives close: id.
			^ true].
	^ false! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 5/13/2012 19:17'!
maxFileNameLength
	^ maxFileNameLength! !

!DiskStore methodsFor: 'private' stamp: 'CamilloBruni 1/19/2012 00:35'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	
	| name|
	
	aPath isRoot ifTrue: [ ^ presentBlock value: self rootNode ].
	
	"| encodedPath encodedBasename entry |
	encodedPath := Primitives encode: (self stringFromPath: aPath parent).
	encodedBasename := Primitives encode: aPath basename.
	entry := Primitives lookupDirectory: encodedPath filename: encodedBasename.
	
	^ entry == #badDirectoryPath 
		ifTrue: absentBlock
		ifFalse: [
			entry at: 1 put: aPath basename.
			presentBlock value: entry ]."
		
	name := aPath basename.
	self
		directoryAt: aPath parent
		ifAbsent: absentBlock
		nodesDo: 
			[ :entry | 
			(self filename: (entry at: 1) matches: name)
				ifTrue: [ ^ presentBlock value: entry ] ].
	^ absentBlock value! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 2/20/2012 21:41'!
openFileStream: path writable: writable
	| fullPath |
	fullPath := self stringFromPath: path.
	"redirect over the default implementation"
	^ writable 
		ifFalse: [ FileStream readOnlyFileNamed: fullPath ]
		ifTrue: [ FileStream fileNamed: fullPath ]! !

!DiskStore methodsFor: 'public' stamp: 'CamilloBruni 1/23/2012 15:52'!
rename: sourcePath to: destinationPath

	| sourcePathString encodedSourcePathString targetPathString encodedTargetPathString result |
	sourcePathString := self stringFromPath: sourcePath.
	encodedSourcePathString := Primitives encode: sourcePathString.
	targetPathString := self stringFromPath: destinationPath.
	encodedTargetPathString := Primitives encode: targetPathString.
	^ Primitives rename: encodedSourcePathString to: encodedTargetPathString.! !

!DiskStore methodsFor: 'private' stamp: 'cwp 2/18/2011 11:37'!
rootNode
	^ #('' 0 0 true 0)! !

DiskStore subclass: #UnixStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!UnixStore commentStamp: '<historical>' prior: 0!
I'm a specific store for Unix file systems!

UnixStore subclass: #MacStore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!MacStore commentStamp: '<historical>' prior: 0!
I'm a specific store for OSX file systems!

!MacStore class methodsFor: 'current' stamp: 'CamilloBruni 5/10/2012 15:43'!
isActiveClass
	^ Smalltalk os isMacOS! !

!MacStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 13:11'!
mimeTypesAt: aPath
	"Return a list of MIME types applicable to the receiver. This default implementation uses the file name extension to figure out what we're looking at but specific subclasses may use other means of figuring out what the type of some file is. Some systems like the macintosh use meta data on the file to indicate data type"
	"| typeCreator type | 
	typeCreator := self getMacFileTypeAndCreator: ((self fullNameFor: fileName)).
	type := (typeCreator at: 1) asLowercase.
	^TypeToMimeMappings at: type ifAbsent:[super mimeTypesFor: fileName]"
	self flag: 'TODO: properly implement the mac os version'.
	^ super mimeTypesAt: aPath! !

!UnixStore class methodsFor: 'public' stamp: 'cwp 2/18/2011 17:19'!
delimiter
	^ $/! !

!UnixStore class methodsFor: 'current' stamp: 'CamilloBruni 5/10/2012 15:43'!
isActiveClass
	^ Smalltalk os isUnix and: [ super isActiveClass ]! !

!UnixStore class methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:21'!
isCaseSensitive
	"except for OSX the default is case sensitive"
	^ OSPlatform isMacOSX not! !

!UnixStore class methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:04'!
maxFileNameLength

	^ 255! !

!UnixStore class methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:11'!
separator 
	^ $:! !

!UnixStore methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:08'!
checkName: aFileName fixErrors: fixing
	"Check if the file name contains any invalid characters"
	| fName |
	fName := super checkName: aFileName fixErrors: fixing.
	
	(fName includes: self delimiter) ifFalse:
		[^fName].
	
	fixing ifFalse: [self error:'Invalid file name'].
	
	^ fName copyReplaceAll: self delimiter asString with: '#'! !

DiskStore subclass: #WindowsStore
	instanceVariableNames: 'disk'
	classVariableNames: 'Disks'
	poolDictionaries: ''
	category: 'FileSystem-Disk'!

!WindowsStore commentStamp: '<historical>' prior: 0!
I'm a specific store for Windows file systems!

!WindowsStore class methodsFor: 'accessing' stamp: 'cwp 2/18/2011 17:19'!
delimiter
	^ $\! !

!WindowsStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:20'!
isCaseSensitive
	^ false! !

!WindowsStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/10/2012 16:05'!
maxFileNameLength
	self flag: 'TODO: more tests needed here!!'.
	^ 255! !

!WindowsStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:11'!
separator 
	^ $\! !

!WindowsStore methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:07'!
checkName: aFileName fixErrors: fixing
	"Check if the file name contains any invalid characters"
	
	| fName badChars hasBadChars |
	fName := super checkName: aFileName fixErrors: fixing.
	badChars := (#( $: $< $> $| $/ $\ $? $* $"), ((0 to: 31) collect: [:n | n asCharacter])) asSet.
	
	(fName includesAnyOf: badChars)
		ifFalse: [^ fName].
	
	fixing ifFalse: [^self error: 'filename contains invalid characters'].
	
	^ fName collect:
		[:char | (badChars includes: char) 
				ifTrue: [$#] 
				ifFalse: [char]]! !

!WindowsStore methodsFor: 'converting' stamp: 'EstebanLorenzano 5/4/2012 15:42'!
pathFromString: aString
	| normalized |
	normalized := aString replaceAll: UnixStore delimiter with: self delimiter.
	^ ((aString first isLetter and: [aString second = $:])
		ifTrue: [ AbsolutePath ]
		ifFalse: [ RelativePath ])
			from: normalized delimiter: self delimiter! !

!WindowsStore methodsFor: 'converting' stamp: 'cwp 2/28/2011 12:32'!
printPath: aPath on: aStream
	aPath printOn: aStream delimiter: self delimiter! !

!FileSystemStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:10'!
delimiter
	self shouldBeImplemented ! !

!FileSystemStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:20'!
isCaseSensitive
	^ self shouldBeImplemented! !

!FileSystemStore class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:11'!
separator
	self shouldBeImplemented! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 1/20/2012 11:34'!
basicIsDirectory: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is a directory.
	This private message should only be called form within the store."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 1/20/2012 11:34'!
basicIsFile: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is a file.
	This private message should only be called form within the store."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 5/9/2012 00:42'!
basicIsReadable: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is a readable
	file or a directory whose contents can be listed."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 5/9/2012 00:41'!
basicIsWritable: aNode
	"Used to decide whether the low-level representation (node / entry) from the store is 
	a writable file or can be changed."
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'CamilloBruni 1/20/2012 13:34'!
basicOpen: aPath writable: aBoolean
	"open the file at the given path and return an identifier"
	self subclassResponsibility! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:01'!
checkName: aString fixErrors: fixErrors
	^ self subclassResponsibility! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/19/2011 01:39'!
close
	"Some kinds of filesystems need to open connections to external resources"! !

!FileSystemStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:27'!
copy: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock fileSystem: aFilesystem

	| buffer out in |
	
	in := nil.
	out := nil.
	buffer := nil.
	[ 
	in := aFilesystem readStreamOn: sourcePath.
	in ifNil: [ ^ absentBlock value ].
	
	(self exists: destinationPath)
		ifTrue: [ "cannot overwrite destination"
			^ presentBlock value ].
		
	out := aFilesystem writeStreamOn: destinationPath.
	buffer := ByteArray new: 1024.
	
	[ in atEnd ]
		whileFalse: [ 
			buffer := in nextInto: buffer.
			out nextPutAll: buffer ]]
		ensure: [ 
			in ifNotNil: [ in close ].
			out ifNotNil: [ out close ]]! !

!FileSystemStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:28'!
copy: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock filesystem: aFilesystem
	self deprecated: 'Use #copy:ifAbsent:to:ifPresent:fileSystem:' on: '3 April 2012' in: 'Pharo 1.4'.
	^self 
		copy: sourcePath 
		ifAbsent: absentBlock 
		to: destinationPath 
		ifPresent: presentBlock 
		fileSystem: aFilesystem! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/18/2011 23:59'!
createDirectory: aPath
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
defaultWorkingDirectory
	^ Path root! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/18/2011 13:25'!
delete: aPath
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'accessing' stamp: 'cwp 2/18/2011 17:19'!
delimiter
	^ self class delimiter! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 1/19/2012 00:15'!
directoryAt: aPath ifAbsent: absentBlock directoryNodesDo: aBlock
	^ self 
		directoryAt: aPath 
		ifAbsent: absentBlock 
		nodesDo: [ :entry|
			(self basicIsDirectory: entry) 
				ifTrue: [ aBlock value: entry ]].! !

!FileSystemStore methodsFor: 'abstract' stamp: 'CamilloBruni 1/19/2012 00:15'!
directoryAt: aPath ifAbsent: absentBlock fileNodesDo: aBlock
	^ self 
		directoryAt: aPath 
		ifAbsent: absentBlock 
		nodesDo: [ :entry|
			(self basicIsFile: entry) 
				ifTrue: [ aBlock value: entry ]].! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/18/2011 13:07'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'public' stamp: 'cwp 2/19/2011 00:00'!
ensureDirectory: aPath
	(self isDirectory: aPath) ifTrue: [ ^ self ].
	self ensureDirectory: aPath parent.
	self createDirectory: aPath! !

!FileSystemStore methodsFor: 'public' stamp: 'cwp 2/18/2011 13:11'!
exists: aPath
	self nodeAt: aPath 
		ifPresent: [ :entry | ^ true ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'private' stamp: 'cwp 2/18/2011 12:28'!
filename: aByteString matches: aByteString2
	^ aByteString = aByteString2! !

!FileSystemStore methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:20'!
isCaseSensitive
	^ self class isCaseSensitive! !

!FileSystemStore methodsFor: 'public' stamp: 'cwp 2/18/2011 13:22'!
isDirectory: aPath
	aPath isRoot ifTrue: [ ^ true ].
	self 
		nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicIsDirectory: entry ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 1/20/2012 13:13'!
isFile: aPath
	"slow solution for big directories!! "
	^ self 
		nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicIsFile: entry ] 
		ifAbsent: [ ^ false ]
	! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 00:40'!
isReadable: aPath
	self nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicIsReadable: entry ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 00:40'!
isWritable: aPath
	self nodeAt: aPath 
		ifPresent: [ :entry | ^ self basicIsWritable: entry ] 
		ifAbsent: [ ^ false ].
	! !

!FileSystemStore methodsFor: 'public' stamp: 'CamilloBruni 5/9/2012 13:06'!
mimeTypesAt: aPath
	^ MIMEType forExtensionReturnMimeTypesOrNil: aPath extension! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/18/2011 12:50'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	self subclassResponsibility ! !

!FileSystemStore methodsFor: 'abstract' stamp: 'cwp 2/19/2011 01:38'!
open
	"Some kinds of filesystems need to open connections to external resources"! !

!FileSystemStore methodsFor: 'converting' stamp: 'CamilloBruni 5/7/2012 02:20'!
pathFromString: aString
	"Use the unix convention by default, since many filesystems are based on it."
	
	^ Path from: aString delimiter: self delimiter! !

!FileSystemStore methodsFor: 'converting' stamp: 'cwp 2/28/2011 12:28'!
printPath: aPath on: out
	"Use the unix convention by default, since it's the most common."
	
	aPath isAbsolute ifTrue: [ out nextPut: $/ ].
	^ aPath printOn: out delimiter: self delimiter! !

!FileSystemStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:19'!
rename: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock fileSystem: anFSFilesystem

	| result |
	result := self rename: sourcePath to: destinationPath.
	result	ifNil: [
		(self exists: destinationPath) 
			ifTrue: [ ^ presentBlock value ].
		(self exists: sourcePath) 
			ifFalse: [ ^ absentBlock value ].
		self primitiveFailed ].
	^ self.! !

!FileSystemStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:20'!
rename: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock filesystem: anFSFilesystem

	self deprecated: 'Use #rename:ifAbsent:to:ifPresent:filesystem:' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self 
		rename: sourcePath 
		ifAbsent: absentBlock 
		to: destinationPath 
		ifPresent: presentBlock 
		fileSystem: anFSFilesystem.! !

!FileSystemStore methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 01:13'!
separator
	^ self class separator! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
signalDirectoryDoesNotExist: aPath
	"Raise FSDirectoryDoesNotExist exception"
		
	^ DirectoryDoesNotExist signalWith: aPath! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
signalDirectoryExists: aPath
	"Raise FSDirectoryExists exception"
		
	^ DirectoryExists signalWith: aPath! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
signalFileDoesNotExist: aPath
	"Raise FSFileDoesNotExist exception"
		
	^ FileDoesNotExist signalWith: aPath! !

!FileSystemStore methodsFor: 'error signalling' stamp: 'EstebanLorenzano 4/2/2012 11:41'!
signalFileExists: aPath
	"Raise FSFileExist exception"
	
	^ FileExists signalWith: aPath! !

!FileSystemStore methodsFor: 'converting' stamp: 'CamilloBruni 1/19/2012 00:24'!
stringFromPath: aPath
	^ String streamContents: [ :out | 
		self printPath: aPath on: out ]! !

FileSystemStore subclass: #MemoryStore
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!MemoryStore commentStamp: '<historical>' prior: 0!
I'm a specific store for memory file system!

!MemoryStore class methodsFor: 'public' stamp: 'cwp 2/18/2011 17:19'!
delimiter
	^ $/! !

!MemoryStore class methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:20'!
isCaseSensitive
	^ true! !

!MemoryStore class methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:11'!
separator 
	^ $:! !

!MemoryStore methodsFor: 'public' stamp: 'cwp 2/18/2011 13:08'!
basenameFromEntry: association
	^ association key! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 1/19/2012 00:47'!
basicIsDirectory: association
	^ association value isDictionary! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 1/19/2012 00:47'!
basicIsFile: association
	^ association value isDictionary not! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 5/9/2012 00:42'!
basicIsReadable: association
	^ true! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 5/9/2012 00:42'!
basicIsWritable: association
	^ true! !

!MemoryStore methodsFor: 'private' stamp: 'cwp 4/3/2011 22:17'!
basicOpen: path writable: aBoolean
	^ self
		nodeAt: path
		ifPresent: [ :bytes | bytes ]
		ifAbsent: [ 
			aBoolean
				ifFalse: [ self signalFileDoesNotExist: path ]
				ifTrue: [ self createFile: path ] ]! !

!MemoryStore methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:02'!
checkName: aString fixErrors: fixErrors
	aString ifEmpty: [ self error: 'zero length file name' ].
	^ aString! !

!MemoryStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:29'!
copy: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock fileSystem: aFilesystem
	self
		nodeAt: sourcePath
		ifPresent: [ :bytes | 
			(self basicIsDirectory: bytes)
				ifTrue: [ ^ absentBlock value ].
			self
				nodeAt: destinationPath parent
				ifPresent: [ :dict | 
					(self basicIsFile: dict)
						ifTrue: [ self signalDirectoryDoesNotExist: destinationPath parent ].
					(dict includesKey: destinationPath basename)
						ifTrue: [ "cannot overwrite existing file"^ presentBlock value ].
					dict at: destinationPath basename put: bytes copy ]
				ifAbsent: [ self signalDirectoryDoesNotExist: destinationPath parent ] ]
		ifAbsent: absentBlock! !

!MemoryStore methodsFor: 'public' stamp: 'cwp 4/3/2011 22:17'!
createDirectory: path
	| parent |
	parent := path parent.
	^ self
		nodeAt: parent
		ifPresent: [ :dict | 
			dict
				at: path basename
				ifPresent: [ :node | 
					node isDictionary
						ifTrue: [ self signalDirectoryExists: path ]
						ifFalse: [ self signalFileExists: path ] ].
			dict at: path basename put: Dictionary new ]
		ifAbsent: [ self signalDirectoryDoesNotExist: parent ]! !

!MemoryStore methodsFor: 'private' stamp: 'cwp 4/3/2011 22:16'!
createFile: aPath
	^ self
		nodeAt: aPath parent
		ifPresent: [ :dict | 
			(self basicIsDirectory: dict)
				ifTrue: [ dict at: aPath basename put: ByteArray new ] ]
		ifAbsent: [ self signalDirectoryDoesNotExist: aPath parent ]! !

!MemoryStore methodsFor: 'public' stamp: 'cwp 2/18/2011 13:24'!
delete: path
	self nodeAt: path parent ifPresent: [ :dict | dict removeKey: path basename ] ifAbsent: [  ]! !

!MemoryStore methodsFor: 'private' stamp: 'cwp 2/18/2011 13:06'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock
	^ self
		nodeAt: aPath
		ifPresent: [ :directory | 
			directory isDictionary
				ifFalse: [ ^ absentBlock value ].
			directory associations sorted do: aBlock ]
		ifAbsent: absentBlock! !

!MemoryStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:24'!
entryFromNode: anObject fileSystem: aFilesystem path: aPath
	^ FileSystemDirectoryEntry
		fileSystem: aFilesystem
		path: aPath
		creation: 0
		modification: 0
		isDir: anObject isDictionary
		size:
			(anObject isDictionary
				ifTrue: [ 0 ]
				ifFalse: [ anObject size ])! !

!MemoryStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:25'!
entryFromNode: anObject filesystem: aFilesystem path: aPath
	self deprecated: 'Use entryFromNode:fileSystem:path:' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self 
		entryFromNode: anObject 
		fileSystem: aFilesystem 
		path: aPath! !

!MemoryStore methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 13:27'!
entryFromNode: association path: path for: aFilesystem
	| entryPath |
	entryPath := path / (self basenameFromEntry: association).
	^ self entryFromNode: association value fileSystem: aFilesystem path: entryPath! !

!MemoryStore methodsFor: 'printing' stamp: 'cwp 2/18/2011 16:32'!
forReferencePrintOn: aStream
	aStream nextPutAll: 'memory:'! !

!MemoryStore methodsFor: 'private' stamp: 'cwp 2/19/2011 01:02'!
growFile: path to: anInteger
	^ self
		replaceFile: path
		in: [ :bytes | 
			(bytes class new: anInteger)
				replaceFrom: 1
					to: bytes size
					with: bytes
					startingAt: 1;
				yourself ]! !

!MemoryStore methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 4/3/2012 09:36'!
handleClass
	^ MemoryHandle ! !

!MemoryStore methodsFor: 'as yet unclassified' stamp: 'cwp 2/18/2011 12:46'!
initialize 
	root := Dictionary new! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 1/19/2012 00:42'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	| current |
	current := self root.
	aPath do: [ :segment | 
		current isDictionary
			ifTrue: [ current := current at: segment ifAbsent: [ ^ absentBlock value ]]
			ifFalse: [ ^ absentBlock value ]].
	^ presentBlock value: current! !

!MemoryStore methodsFor: 'public' stamp: 'EstebanLorenzano 4/3/2012 09:36'!
openFileStream: path writable: anObject
	| bytes |
	bytes := self nodeAt: path ifPresent: [ :array | array ] ifAbsent: [ self createFile: path ].
	^ MemoryFileStream on: bytes store: self path: path! !

!MemoryStore methodsFor: 'private' stamp: 'cwp 4/3/2011 22:17'!
replaceFile: path in: aBlock
	^ self
		nodeAt: path parent
		ifPresent: [ :dict | 
			| old new |
			(self basicIsDirectory: dict)
				ifFalse: [ self signalFileDoesNotExist: path ].
			old := dict at: path basename ifAbsent: [ self signalFileDoesNotExist: path ].
			new := aBlock value: old.
			dict at: path basename put: new ]
		ifAbsent: [ self signalFileDoesNotExist: path ]! !

!MemoryStore methodsFor: 'as yet unclassified' stamp: 'cwp 2/18/2011 12:46'!
root
	^ root! !

MemoryStore subclass: #ZipStore
	instanceVariableNames: 'reference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Zip'!

!ZipStore commentStamp: '<historical>' prior: 0!
I'm a specific store for Zip file systems!

!ZipStore class methodsFor: 'as yet unclassified' stamp: 'cwp 2/18/2011 23:46'!
reference: aReference
	^ self basicNew
		initializeWithReference: aReference
		yourself! !

!ZipStore methodsFor: 'public' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
close
	| archive fs stream |
	archive := ZipArchive new.
	fs := FileSystem store: self.
	fs root allChildren do: 
		[ :each | 
		each isFile ifTrue: 
			[ each readStreamDo: 
				[ :output | 
				archive addString: output contents as: each path printString ] ] ].
	archive
		writeTo: (stream := WriteStream on: ByteArray new);
		close.
	self reference writeStreamDo: 
		[ :output | output nextPutAll: stream contents ]! !

!ZipStore methodsFor: 'as yet unclassified' stamp: 'cwp 2/18/2011 23:49'!
initializeWithReference: aReference
	self initialize.
	reference := aReference! !

!ZipStore methodsFor: 'public' stamp: 'cwp 2/19/2011 01:41'!
open
	| archive output |
	self reference exists
		ifFalse: [ ^ self ].
	archive := ZipArchive new.
	self reference
		readStreamDo: [ :input | 
			archive readFrom: input contents readStream.
			archive members
				do: [ :member | 
					| path |
					path := self pathFromMember: member.
					member isDirectory
						ifTrue: [ self ensureDirectory: path ]
						ifFalse: [ 
							self ensureDirectory: path parent.
							self createFile: path.
							self
								replaceFile: path
								in: [ :bytes | 
									output := bytes writeStream.
									member rewindData.
									member copyRawDataTo: output.
									output contents ] ] ] ].
	^ self! !

!ZipStore methodsFor: 'private' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
pathFromMember: anArchiveMember
	| path |
	path := Path root resolve: anArchiveMember fileName.
	^ path basename isEmpty
		ifTrue: [ path parent ]
		ifFalse: [ path ]! !

!ZipStore methodsFor: 'as yet unclassified' stamp: 'cwp 2/18/2011 23:49'!
reference
	^ reference! !

Object subclass: #FileSystemStream
	instanceVariableNames: 'handle position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-AnsiStreams'!

!FileSystemStream commentStamp: '<historical>' prior: 0!
I am an abstract superclass for read- and write-streams that perform IO via a handle rather than by calling primitives directly. My subclasses' provide a cursor on a collection, so that sequences of IO messages can be position independent.

handle
	A subclass of FileSystemHandle. All IO goes through this object.
	
position
	An integer describing the next index to be accessed.!

FileSystemStream subclass: #FileSystemReadStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-AnsiStreams'!

!FileSystemReadStream commentStamp: 'cwp 11/18/2009 11:36' prior: 0!
I implement (more or less) the ANSI <FileReadStream> protocol. I provide methods for reading data from a file. !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 22:45'!
atEnd
	^ position - 1 = handle size! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 22:58'!
do: aBlock
	[self atEnd] whileFalse: [aBlock value: self next]! !

!FileSystemReadStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isRead
	^ true! !

!FileSystemReadStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isWrite
	^ false! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 22:59'!
next
	| result |
	result := handle at: position.
	position := position + 1.
 	^ result! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'MarianoMartinezPeck 1/9/2012 18:02'!
next: count
	^ self nextInto: (ByteArray new: count)
! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'MarianoMartinezPeck 1/9/2012 18:02'!
next: number into: aCollection
    | count |
    count := handle at: position read: aCollection startingAt: 1 count: number.

    position := position + count.
     ^ count < aCollection size
        ifTrue: [aCollection first: count]
        ifFalse: [aCollection]! !

!FileSystemReadStream methodsFor: 'squeak' stamp: 'cwp 7/31/2009 00:24'!
nextInto: aCollection
	| count |
	count := handle at: position read: aCollection startingAt: 1 count: aCollection size.
	position := position + count.
 	^ count < aCollection size
		ifTrue: [aCollection first: count]
		ifFalse: [aCollection]! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 23:31'!
nextLine
	| char |
	^ ByteArray streamContents:
		[:out |
		[self atEnd or: [#(13 10) includes: (char := self next)]] whileFalse: [out nextPut: char].
		(char = 13 and: [self peek = 10]) ifTrue: [position := position + 1]].
	! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:03'!
nextMatchFor: anObject
	^ self next = anObject! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 23:27'!
peek
	^ handle at: position! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:06'!
peekFor: anObject
	^ self peek = anObject! !

!FileSystemReadStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/12/2011 20:49'!
size
	^ handle size! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:19'!
skip: anInteger
	position := position + anInteger min: handle size + 1! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:27'!
skipTo: anObject
	
	| result |
	[self atEnd or: [result := (self next = anObject)]] whileFalse.
	^ result! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:37'!
upTo: anObject
	
	| byte |
	^ ByteArray streamContents:
		[:out |
		[self atEnd or: [(byte := self next) = anObject]] whileFalse:
			[out nextPut: byte]]
		! !

!FileSystemStream class methodsFor: 'instance creation' stamp: 'cwp 7/29/2009 22:30'!
onHandle: aHandle
	^ self new setHandle: aHandle
	! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 7/29/2009 22:29'!
close
	handle close! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 7/29/2009 22:29'!
contents
	| size contents |
	size := handle size.
	contents := ByteArray new: size.
	handle at: 1 read: contents startingAt: 1 count: size.
	^ contents! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 10/29/2009 09:47'!
isOpen
	^ handle isOpen! !

!FileSystemStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isRead
	self subclassResponsibility! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 2/5/2011 22:54'!
isStream
	^ true! !

!FileSystemStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isWrite
	self subclassResponsibility! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 10/29/2009 09:47'!
open
	handle open! !

!FileSystemStream methodsFor: 'public' stamp: 'Camillobruni 8/18/2011 16:43'!
position
	^ position! !

!FileSystemStream methodsFor: 'public' stamp: 'Camillobruni 8/18/2011 16:49'!
position: anInteger
	self assert: anInteger > 0.
	position := anInteger.! !

!FileSystemStream methodsFor: 'initialize-release' stamp: 'cwp 7/29/2009 22:30'!
setHandle: aHandle
	handle := aHandle.
	position := 1! !

FileSystemStream subclass: #FileSystemWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-AnsiStreams'!

!FileSystemWriteStream commentStamp: 'cwp 11/18/2009 11:36' prior: 0!
I implement (more or less) the ANSI <FileWriteStream> protocol. I provide methods for writing data to a file.!

!FileSystemWriteStream methodsFor: 'writing' stamp: 'CamilloBruni 8/12/2011 14:26'!
<< aCollection
	^ self nextPutAll: aCollection! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'cwp 7/28/2009 22:25'!
cr
	self nextPut: 13! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:35'!
crlf
	"Append a line feed character to the receiver."

	self nextPut: 13; nextPut: 10! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 1/24/2012 13:54'!
crtab
	"Append a return character, followed by a single tab character, to the 
	receiver."

	self nextPut: 13.
	self nextPut: Character tab! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:36'!
crtab: anInteger 
	"Append a return character, followed by anInteger tab characters, to the 
	receiver."

	self nextPut: 13.
	anInteger timesRepeat: [self tab]! !

!FileSystemWriteStream methodsFor: 'ansi puttable' stamp: 'cwp 7/28/2009 22:22'!
flush
	handle flush! !

!FileSystemWriteStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isRead
	^ false! !

!FileSystemWriteStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:29'!
isWrite
	^ true! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:35'!
lf
	"Append a line feed character to the receiver."

	self nextPut: 10! !

!FileSystemWriteStream methodsFor: 'writing' stamp: 'cwp 7/23/2009 17:03'!
nextPut: anObject
	handle at: position put: anObject.
	position := position + 1.
	^ anObject! !

!FileSystemWriteStream methodsFor: 'writing' stamp: 'cwp 7/23/2009 22:51'!
nextPutAll: aCollection
	handle at: position write: aCollection startingAt: 1 count: aCollection size.
	position := position + aCollection size.
	^ aCollection! !

!FileSystemWriteStream methodsFor: 'writing' stamp: 'CamilloBruni 1/24/2012 13:54'!
print: anObject
	"Have anObject print itself on the receiver."

	anObject printOn: self! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:36'!
space
	"Append a space character to the receiver."

	self nextPut: 32! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:24'!
space: anInteger 
	"Append anInteger space characters to the receiver."

	anInteger timesRepeat: [self space]! !

!FileSystemWriteStream methodsFor: 'writing' stamp: 'CamilloBruni 1/24/2012 13:42'!
store: anObject 
	"Have anObject print on the receiver for purposes of rereading."

	anObject storeOn: self! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'cwp 7/28/2009 22:28'!
tab
	self nextPut: 9! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:24'!
tab: anInteger 
	"Append anInteger tab characters to the receiver."

	anInteger timesRepeat: [self tab]! !

!FileSystemWriteStream methodsFor: 'file' stamp: 'cwp 10/15/2009 22:05'!
truncate
	handle truncateTo: position - 1! !

!FileSystemWriteStream methodsFor: 'file' stamp: 'cwp 10/15/2009 21:49'!
truncateTo: anInteger
	handle truncateTo: anInteger! !

Object subclass: #FileSystemVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!FileSystemVisitor commentStamp: 'cwp 11/18/2009 12:25' prior: 0!
I am an abstract superclass for objects that can perform operations on directory trees. My subclasses implement the visitor protocol, and process filesystem nodes shown to them by guides.!

FileSystemVisitor subclass: #AbstractEnumerationVisitor
	instanceVariableNames: 'out block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!AbstractEnumerationVisitor commentStamp: '<historical>' prior: 0!
I'm an abstract superclass for enumeration operations on directory entries. !

!AbstractEnumerationVisitor methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 4/2/2012 11:37'!
breadthFirst: aReference
	^ self visit: aReference with: (BreadthFirstGuide for: self)! !

!AbstractEnumerationVisitor methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/9/2011 15:46'!
initializeWithBlock: aBlock
	self initialize.
	block := aBlock! !

!AbstractEnumerationVisitor methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
postorder: aReference
	^ self visit: aReference with: (PostorderGuide for: self)! !

!AbstractEnumerationVisitor methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
preorder: aReference
	^ self visit: aReference with: (PreorderGuide for: self)! !

!AbstractEnumerationVisitor methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/9/2011 15:46'!
visit: aReference with: aGuide
	out := (Array new: 10) writeStream.
	aGuide show: aReference.
	^ out contents! !

!AbstractEnumerationVisitor methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/9/2011 15:48'!
visitReference: anEntry
	self subclassResponsibility! !

AbstractEnumerationVisitor subclass: #CollectVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!CollectVisitor commentStamp: 'cwp 11/18/2009 12:32' prior: 0!
I am a visitor that collects objects from the nodes I visit. I take a block similar to those passed to Collection>>collect:. I evaluate the block with DirectoryEntries for the nodes I visit, and collect the objects answered into an array.

I can use any guide, and the objects in the array I produce will reflect the order imposed by the guide.!

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:33'!
breadthFirst: aReference
	^ self breadthFirst: aReference collect: [:entry | entry]! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:32'!
breadthFirst: aReference collect: aBlock
	^ (self collect: aBlock) breadthFirst: aReference! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:39'!
collect: aBlock
	^ self basicNew initializeWithBlock: aBlock! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:36'!
postorder: aReference
	^ self postorder: aReference collect: [:entry | entry]! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:35'!
postorder: aReference collect: aBlock
	^ (self collect: aBlock) postorder: aReference! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:35'!
preorder: aReference
	^ self preorder: aReference collect: [:entry | entry]! !

!CollectVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/16/2009 10:36'!
preorder: aReference collect: aBlock
	^ (self collect: aBlock) preorder: aReference! !

!CollectVisitor methodsFor: 'visiting' stamp: 'cwp 11/16/2009 10:38'!
visitReference: anEntry
	out nextPut: (block value: anEntry)! !

AbstractEnumerationVisitor subclass: #SelectVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!SelectVisitor commentStamp: '<historical>' prior: 0!
I am a visitor that selects objects from the nodes I visit. I take a block similar to those passed to Collection>>select:. I evaluate the block with DirectoryEntries for the nodes I visit.

I can use any guide, and the objects in the array I produce will reflect the order imposed by the guide.!

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
breadthFirst: aReference
	^ self breadthFirst: aReference select: [:entry | true]! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
breadthFirst: aReference select: aBlock
	^ (self select: aBlock) breadthFirst: aReference! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
postorder: aReference
	^ self postorder: aReference select: [:entry | true]! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
postorder: aReference select: aBlock
	^ (self select: aBlock) postorder: aReference! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
preorder: aReference
	^ self preorder: aReference select: [:entry | true]! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
preorder: aReference select: aBlock
	^ (self select: aBlock) preorder: aReference! !

!SelectVisitor class methodsFor: 'instance creation' stamp: 'CamilloBruni 8/9/2011 15:50'!
select: aBlock
	^ self basicNew initializeWithBlock: aBlock! !

!SelectVisitor methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/9/2011 15:48'!
visitReference: anEntry
	(block value: anEntry) 
		ifTrue: [ out nextPut: anEntry ]! !

FileSystemVisitor subclass: #CopyVisitor
	instanceVariableNames: 'source dest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!CopyVisitor commentStamp: 'cwp 11/18/2009 12:30' prior: 0!
I create a copy of the directory tree that I visit. I use the PreorderGuide so that I can create directories before creating their contents. !

!CopyVisitor class methodsFor: 'instance creation' stamp: 'cwp 10/30/2009 13:44'!
copy: source to: dest
	(self from: source to: dest) visit! !

!CopyVisitor class methodsFor: 'instance creation' stamp: 'cwp 10/30/2009 13:41'!
from: srcReference to: dstReference
	^ self basicNew
		initializeWithSource: srcReference 
		dest: dstReference! !

!CopyVisitor methodsFor: 'visiting' stamp: 'cwp 11/17/2009 21:06'!
copyDirectory: aReference
	| directory |
	directory := dest resolve: (aReference relativeTo: source).
	directory createDirectory! !

!CopyVisitor methodsFor: 'visiting' stamp: 'cwp 11/15/2009 00:31'!
copyFile: aReference
	| copy |
	copy := dest resolve: (aReference relativeTo: source).
	aReference copyTo: copy! !

!CopyVisitor methodsFor: 'initialize-release' stamp: 'cwp 10/30/2009 13:42'!
initializeWithSource: srcReference dest: dstReference
	self initialize.
	source := srcReference.
	dest := dstReference! !

!CopyVisitor methodsFor: 'visiting' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
visit
	(PreorderGuide for: self) 
		show: source! !

!CopyVisitor methodsFor: 'visiting' stamp: 'cwp 11/16/2009 10:51'!
visitDirectory: anEntry
	| reference |
	reference := anEntry reference.
	reference = source
		ifTrue: [dest ensureDirectory]
		ifFalse: [self copyDirectory: reference]! !

!CopyVisitor methodsFor: 'visiting' stamp: 'cwp 11/16/2009 10:52'!
visitFile: anEntry
	| reference |
	reference := anEntry reference.
	reference = source
		ifTrue: [source copyTo: dest]
		ifFalse: [self copyFile: reference]! !

FileSystemVisitor subclass: #DeleteVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Implementation'!

!DeleteVisitor commentStamp: 'cwp 11/18/2009 12:30' prior: 0!
I delete the directory tree that I visit. I use the PostorderGuide so that I can delete files before deleting their containing directories.!

!DeleteVisitor class methodsFor: 'instance creation' stamp: 'cwp 11/17/2009 13:02'!
delete: aReference
	^ self new visit: aReference! !

!DeleteVisitor methodsFor: 'visiting' stamp: 'EstebanLorenzano 4/2/2012 11:38'!
visit: aReference
	PostorderGuide show: aReference to: self! !

!DeleteVisitor methodsFor: 'visiting' stamp: 'cwp 11/16/2009 10:53'!
visitReference: anEntry
	anEntry reference delete! !

!FileSystemVisitor methodsFor: 'visiting' stamp: 'cwp 11/15/2009 00:45'!
visitDirectory: aReference
	^ self visitReference: aReference! !

!FileSystemVisitor methodsFor: 'visiting' stamp: 'cwp 11/15/2009 00:45'!
visitFile: aReference
	^ self visitReference: aReference! !

!FileSystemVisitor methodsFor: 'visiting' stamp: 'cwp 11/15/2009 00:45'!
visitReference: aReference! !

Object variableSubclass: #Path
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!Path commentStamp: '<historical>' prior: 0!
I'm a private and abstract filesystem path, independent of the string representation used to describe paths on a specific filesystem. I provide methods for navigating the filesystem hierarchy and working with absolute and relative paths. I only refer to a concrete file or directory with regard to a specific filesystem. Normally users should not use me directly. 

Path independent representation of delimiter is defined by DiskFileSystem current delimiter.

API instance creation:
#* and #/ are mnemonic to . and /
whose arguments should  be string file- or directory names, not fragments of Unix path notation intended to be parsed.

#/ and #* provide a mini-DSL for building up paths, while
#readFrom:delimiter: parses path strings.

Note that (Path with: 'parent/child/') isRelative returns true
because it creates to a relative path to a file/directory called
'parent/child'. In bash you'd escape the slashes like this: parent\/child\/

similarly 
(Path with: '/parent/child/') isRelative returns true
That's a relative path to '/parent/child'. In bash: /\parent\/child\/

(Path with: '') isRelative returns false

Because this is an absolute path to the root of the file system. Absolute paths
have an empty first element. If you consider $/ the separator,
'/usr/local/bin' has an empty first element.

!

Path variableSubclass: #AbsolutePath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!AbsolutePath commentStamp: '<historical>' prior: 0!
I represent an absolute path (a position starting from Path root)!

!AbsolutePath class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 1/19/2012 15:12'!
addEmptyElementTo: result! !

!AbsolutePath class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 1/19/2012 15:02'!
from: aString delimiter: aDelimiterCharater
	aString = '/'
		ifTrue: [ ^ self root ].
		
	^ super from: aString delimiter: aDelimiterCharater! !

!AbsolutePath methodsFor: 'testing' stamp: 'cwp 2/26/2011 10:58'!
isAbsolute
	^ true! !

!AbsolutePath methodsFor: 'testing' stamp: 'cwp 2/26/2011 11:03'!
isRoot
	^ self size = 0! !

!AbsolutePath methodsFor: 'printing' stamp: 'EstebanLorenzano 4/3/2012 11:15'!
printOn: aStream
	aStream nextPutAll: 'Path'.
	self isRoot
		ifTrue: [aStream nextPutAll: ' root']
		ifFalse:
			[1 to: self size do:
				[:i |
				aStream 
					nextPutAll: ' / ''';
				 	nextPutAll: (self at: i);
					nextPut: $']]! !

!AbsolutePath methodsFor: 'enumerating' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
withParents
	^ super withParents addFirst: (Path root); yourself! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
* aString
	"Answer a relative path with aString as its sole segment. For example,
	FSPath * 'griffle' will produce the same result as ./griffle in a unix shell.
	The selector #* was chosen for it's visual similarity to $."
	
	"Note: aString is not parsed, so supplying a string like '/griffle/plonk'
	will not create an absolute path."
	
	^ RelativePath with: aString! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
/ aString
	"Answer an absolute path with aString as it's sole segment. The selector
	was chosen to allow path construction with Smalltalk syntax, which 
	neverthelesss resembles paths as they appear in a unix shell. Eg.
	FSPath / 'griffle' / 'plonk'."
	
	aString isEmptyOrNil 
		ifTrue: [ Error signal: 'Path element cannot be empty or nil'].
	^ AbsolutePath with: aString! !

!Path class methodsFor: 'private' stamp: 'CamilloBruni 1/19/2012 15:03'!
addElement: element to: result
	element = '..' ifTrue: [^ self addParentElementTo: result].
	element = ''   ifTrue: [^ self addEmptyElementTo: result].
	element = '.'  ifFalse: [result add: element]! !

!Path class methodsFor: 'private' stamp: 'cwp 10/26/2009 13:41'!
addEmptyElementTo: result
	result isEmpty ifTrue: [result add: '']
		! !

!Path class methodsFor: 'private' stamp: 'cwp 10/26/2009 13:39'!
addParentElementTo: result
	result isEmpty
		ifTrue: [result add: '..']
		ifFalse: [result removeLast]
		! !

!Path class methodsFor: 'private' stamp: 'cwp 10/26/2009 13:30'!
canonicalizeElements: aCollection
	| result |
	result := OrderedCollection new.
	aCollection do: [:element | self addElement: element to: result].
	^ result! !

!Path class methodsFor: 'encodings' stamp: 'StephaneDucasse 2/18/2011 22:31'!
extensionDelimiter
	"Return the extension delimiter character."
	^ $.! !

!Path class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/7/2012 02:19'!
from: aString
	^ self from: aString delimiter: $/! !

!Path class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/8/2012 17:48'!
from: aString delimiter: aDelimiterCharacter 
	"Answer a path composed of several elements delimited by aCharacter"
	| pathCls |
	aString isEmpty
		ifTrue: [ ^ self root ].
	
	pathCls :=  aString first = $/ 
		ifTrue: [AbsolutePath]
		ifFalse:[RelativePath].
	
	^ pathCls withAll: (pathCls canonicalizeElements: (aDelimiterCharacter split: aString))! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
parent
	"Answer a path that resolves to the parent of the current
	working directory. This is similar to .. in unix, but doesn't
	rely on actual hardlinks being present in the filesystem."

	^ RelativePath with: '..'! !

!Path class methodsFor: 'instance creation' stamp: 'cwp 11/15/2009 00:11'!
parents: anInteger
	| path |
	path := self new: anInteger.
	1 to: anInteger do: [:i | path at: i put: '..'].
	^ path! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:42'!
root
	"Answer the root path - ie, / on unix"
	
	^ AbsolutePath new! !

!Path class methodsFor: 'private' stamp: 'EstebanLorenzano 4/3/2012 13:42'!
with: aString
	"Answer a relative path of the given string. N.B. that the argument is not parsed; 
	it is the name of a single path element, and path separators in it do not have 
	special meaning."
	
	"(FSPath with: '/parent/child/') isRelative
	answers true because this is a relative path to a file or directory named 
	'/parent/child/'. In bash: \/parent\/child\/"

	| inst |
	inst := self new: 1.
	inst at: 1 put: aString.
	^ inst! !

!Path class methodsFor: 'private' stamp: 'CamilloBruni 5/8/2012 17:47'!
withAll: aCollection
	| inst |
	inst := self new: aCollection size.
	aCollection withIndexDo: [:segment :index |
		inst at: index put: segment].
	^ inst! !

!Path class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
workingDirectory
	"Answer a path that will always resolve to the current working directory."
	
	^ RelativePath new! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/17/2009 23:52'!
, extension 
	^ self withName: self basename extension: extension! !

!Path methodsFor: 'navigating' stamp: 'CamilloBruni 8/15/2011 17:36'!
/ aString
	| path |

	aString isEmptyOrNil 
		ifTrue: [ Error signal: 'Path element cannot be empty or nil'].
		
	path := self class new: self size + 1.
	path copyFrom: self.
	path at: path size put: aString.
	^ path! !

!Path methodsFor: 'comparing' stamp: 'cwp 12/14/2008 17:36'!
= other
	^ self species = other species
		and: [self size = other size
			and: [(1 to: self size) allSatisfy: [:i | (self at: i) = (other at: i)]]]! !

!Path methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:28'!
asFileReference
	^ FileSystem disk referenceTo: self! !

!Path methodsFor: 'converting' stamp: 'cwp 10/10/2009 18:04'!
asPathWith: anObject
	^ self! !

!Path methodsFor: 'converting' stamp: 'EstebanLorenzano 4/12/2012 14:30'!
asReference
	self deprecated: 'Use #asFileReference instead'  on: '12 April 2012' in:  'Pharo 1.4' .
	^ self asFileReference! !

!Path methodsFor: 'navigating' stamp: 'cwp 9/22/2009 09:08'!
asResolvedBy: anObject
	^ anObject resolvePath: self! !

!Path methodsFor: 'converting' stamp: 'abc 5/11/2012 17:33'!
asUrl 
	^ FileUrl
		pathParts: self segments asOrderedCollection 
		isAbsolute: self isAbsolute! !

!Path methodsFor: 'accessing' stamp: 'StephaneDucasse 2/15/2010 18:06'!
base
	"Returns the base of the basename, i.e. /foo/gloops.taz basename is 'gloops'"
	^ self basename copyUpTo: self extensionDelimiter! !

!Path methodsFor: 'accessing' stamp: 'CamilloBruni 8/9/2011 16:11'!
basename
	"Returns the base of the basename, 
		i.e. 
		/foo/gloops.taz basename is 'gloops.taz'
		/ basename is '/'"
	self size == 0
		"the root node"
		ifTrue: [ ^ '/'].
	^ self at: self size! !

!Path methodsFor: 'comparing' stamp: 'cwp 10/26/2009 01:03'!
contains: anObject
	^ anObject isContainedBy: self! !

!Path methodsFor: 'comparing' stamp: 'cwp 10/25/2009 22:59'!
containsPath: aPath
	self size < aPath size ifFalse: [^ false].
	1	to: self size 
		do: [:i | (self at: i) = (aPath at: i) ifFalse: [^ false]].
	^ true! !

!Path methodsFor: 'comparing' stamp: 'cwp 10/25/2009 23:05'!
containsReference: aReference
	^ false! !

!Path methodsFor: 'private' stamp: 'cwp 12/13/2008 21:08'!
copyFrom: aPath
	| size |
	size := aPath size min: self size.
	1 to: size do: [:i | self at: i put: (aPath at: i)].
	! !

!Path methodsFor: 'accessing' stamp: 'cwp 10/11/2009 11:05'!
delimiter
	^ $/! !

!Path methodsFor: 'enumerating' stamp: 'cwp 7/18/2009 01:13'!
do: aBlock 
	1 
		to: self size
		do: 
			[ :index || segment |
			segment := self at: index.
			segment isEmpty ifFalse: [ aBlock value: segment ] ]! !

!Path methodsFor: 'accessing' stamp: 'StephaneDucasse 2/15/2010 18:04'!
extension
	"Return the extension of path basename i.e., /foo/gloops.taz extension is 'taz'"
	
	^ self basename copyAfter: self extensionDelimiter! !

!Path methodsFor: 'accessing' stamp: 'cwp 12/23/2008 11:25'!
extensionDelimiter
	^ self class extensionDelimiter! !

!Path methodsFor: 'accessing' stamp: 'sd 2/11/2011 21:02'!
fullName
	"Return the fullName of the receiver."
	
	^ self printString! !

!Path methodsFor: 'comparing' stamp: 'cwp 12/14/2008 17:06'!
hash
	| hash |
	hash := self class identityHash.
	1 to: self size do:
		[:i | hash := String stringHash: (self at: i) initialHash: hash].
	^ hash! !

!Path methodsFor: 'testing' stamp: 'cwp 2/26/2011 10:58'!
isAbsolute
	self subclassResponsibility ! !

!Path methodsFor: 'private' stamp: 'cwp 10/25/2009 19:53'!
isAllParents
	1 to: self size do: [:i | (self at: i) = '..' ifFalse: [^ false]].
	^ true! !

!Path methodsFor: 'comparing' stamp: 'cwp 11/16/2009 09:06'!
isChildOf: anObject
	^ self parent = anObject! !

!Path methodsFor: 'comparing' stamp: 'cwp 10/25/2009 23:01'!
isContainedBy: anObject
	^ anObject containsPath: self! !

!Path methodsFor: 'testing' stamp: 'DamienPollet 2/20/2011 04:00'!
isEmpty
	^ self size = 0! !

!Path methodsFor: 'testing' stamp: 'cwp 12/13/2008 21:00'!
isRelative
	^ self isAbsolute not! !

!Path methodsFor: 'testing' stamp: 'cwp 2/26/2011 11:03'!
isRoot
	self subclassResponsibility ! !

!Path methodsFor: 'testing' stamp: 'cwp 7/18/2009 00:42'!
isWorkingDirectory
	^ self size = 0! !

!Path methodsFor: 'private' stamp: 'CamilloBruni 8/12/2011 16:42'!
lengthOfStemWith: aPath
	| limit index |
	limit := self size min: aPath size.
	index := 1.
	[index <= limit and: [(self at: index) = (aPath at: index)]] 
		whileTrue: [index := index + 1].
	^ index - 1! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/15/2009 00:00'!
makeRelative: anObject
	^ anObject relativeToPath: self! !

!Path methodsFor: 'navigating' stamp: 'cwp 10/25/2009 19:53'!
parent
	| size parent |
	self isRoot ifTrue: [^ self].
	self isAllParents ifTrue: [^ self / '..'].
	
	size := self size - 1.
	parent := self class new: size.
	1 to: size do: [:i | parent at: i put: (self at: i)].
	^ parent! !

!Path methodsFor: 'printing' stamp: 'cwp 11/17/2009 10:22'!
printOn: aStream 
	self printOn: aStream delimiter: self delimiter.
! !

!Path methodsFor: 'printing' stamp: 'cwp 2/26/2011 17:58'!
printOn: aStream delimiter: aCharacter
	(1 to: self size)
		do: [:index | aStream nextPutAll: (self at: index)]
		separatedBy: [aStream nextPut: aCharacter]! !

!Path methodsFor: 'printing' stamp: 'cwp 1/13/2009 21:27'!
printWithDelimiter: aCharacter
	^ String streamContents: [:out | self printOn: out delimiter: aCharacter]! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/15/2009 00:00'!
relativeTo: anObject
	^ anObject makeRelative: self! !

!Path methodsFor: 'navigating' stamp: 'EstebanLorenzano 4/2/2012 11:43'!
relativeToPath: aPath
	"Return the receiver as relative to the argument aPath"
	
	"(FSPath / 'griffle' / 'plonk' / 'nurp') 
		relativeToPath: (FSPath / 'griffle') 
			returns  plonk/nurp"

	| prefix relative |
	aPath isRelative ifTrue: [^ aPath].
	prefix := self lengthOfStemWith: aPath.
	relative := RelativePath parents: (aPath size - prefix).
	prefix + 1 to: self size do: [:i | relative := relative / (self at: i)].
	^ relative! !

!Path methodsFor: 'navigating' stamp: 'cwp 3/29/2011 16:23'!
relativeToReference: aReference 
	^ self relativeToPath: aReference path! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/16/2009 10:19'!
resolve
	^ self! !

!Path methodsFor: 'navigating' stamp: 'cwp 3/29/2011 16:25'!
resolve: anObject
	"Return a path in which the argument has been interpreted in the context of the receiver. Different 
	argument types have different resolution semantics, so we use double dispatch to resolve them correctly."
	
	^ anObject asResolvedBy: self! !

!Path methodsFor: 'navigating' stamp: 'CamilloBruni 1/19/2012 14:31'!
resolvePath: aPath
	"Answers an absolute path created by resolving the argument against the receiver.
	If the argument is abolute answer the argument itself. Otherwise, concatenate the
	two paths, then process all parent references '..', and create a path with the remaining
	elements."
	
	| elements |
	
	aPath isAbsolute ifTrue: [^ aPath].
	elements := Array new: self size + aPath size.
	
	1 to: self size do: [:i | elements at: i put: (self at: i)].
	1 to: aPath size do: [:i | elements at: self size + i put: (aPath at: i)].
	
	^ self class withAll: (self class canonicalizeElements: elements)! !

!Path methodsFor: 'navigating' stamp: 'cwp 9/22/2009 09:06'!
resolveReference: aReference
	^ aReference! !

!Path methodsFor: 'navigating' stamp: 'cwp 3/29/2011 16:30'!
resolveString: aString
	"Treat strings as relative paths with a single element."
	
	^ self / aString! !

!Path methodsFor: 'accessing' stamp: 'CamilloBruni 8/12/2011 20:16'!
segments
	"return an array with all the path segements separated"
	| segments index |
	segments := Array new: self size.
	index := 1.
	self do: [ :part|
		segments at: index put: part.
		index := index + 1 ].
	
	^ segments! !

!Path methodsFor: 'navigating' stamp: 'cwp 11/17/2009 23:51'!
withExtension: extension 
	| basename name |
	basename := self basename.
	^ (basename endsWith: extension) 
		ifTrue: [ self ]
		ifFalse: 
			[name := basename copyUpToLast: self extensionDelimiter.
			self withName: name extension: extension]! !

!Path methodsFor: 'private' stamp: 'cwp 11/17/2009 23:58'!
withName: name extension: extension
	| basename |
	basename :=String streamContents:
		[:out |
		out nextPutAll: name.
		out nextPut: self extensionDelimiter.
		out nextPutAll: extension].
	^ self copy 
		at: self size put: basename;
		yourself! !

!Path methodsFor: 'enumerating' stamp: 'cwp 3/29/2011 16:42'!
withParents
	| paths |
	paths := OrderedCollection new.
	1 to: self size -1 do: [ :index | paths add: ((self class new: index) copyFrom: self) ].
	paths add: self.
	
	^ paths! !

Path variableSubclass: #RelativePath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Core-Kernel'!

!RelativePath commentStamp: '<historical>' prior: 0!
I represent a relative path (a position starting from Path workingDirectory)!

!RelativePath methodsFor: 'testing' stamp: 'cwp 2/26/2011 10:58'!
isAbsolute
	^ false! !

!RelativePath methodsFor: 'testing' stamp: 'cwp 2/26/2011 11:03'!
isRoot
	^ false! !

!RelativePath methodsFor: 'printing' stamp: 'EstebanLorenzano 4/3/2012 11:15'!
printOn: aStream
	aStream nextPutAll: 'Path '.
	self isWorkingDirectory
		ifTrue: [aStream nextPutAll: 'workingDirectory']
		ifFalse: 
			[aStream 
				nextPutAll: '* ''';
				nextPutAll: (self at: 1) asString;
				nextPut: $'.
			2 to: self size do:
				[:i |
				aStream
					nextPutAll: ' / ''';
					nextPutAll: (self at: i);
					nextPut: $']]
					
			! !

!RelativePath methodsFor: 'printing' stamp: 'cwp 2/26/2011 18:00'!
printOn: aStream delimiter: aCharacter
	self isWorkingDirectory ifTrue: [aStream nextPut: $.. ^ self].
	super printOn: aStream delimiter: aCharacter! !
FileLocator initialize!
FileHandle initialize!
DiskStore initialize!
